/****
 * CFileWindow.c
 *
 *	Document methods for the SignWriter key editor.
 *
 *  Copyright  1995 The Center for Sutton Movement Writing.  All rights reserved.
 *
 ****/
 


#include "Global.h"
#include "Commands.h"
#include "CApplication.h"
#include "CBartender.h"
#include "CBureaucrat.h"
#include "CDataFile.h"
#include "CDecorator.h"
#include "CDesktop.h"
#include "CFileWindow.h"
#include "CFileWndKeys.h"
#include "CFileWndBox.h"
#include "CPaneBorder.h"
#include "CSelector.h"
#include "CSymWindow.h"
#include "CSymWndGrid.h"
#include "CWindow.h"
#include "Events.h"
#include "Keys.h"
#include "KeyEditConsts.h"
#include <Packages.h>
#include <string.h>
#include <StandardFile.h>
#include "Symbols.h"
#include "TBUtilities.h"
#include "TCLUtilities.h"
#include "UpdateFile.h"





/*** Global Variables ***/

extern  CApplication *gApplication; /* The application				 */
extern	CBureaucrat  *gGopher;		/* The gopher					 */
extern	CBartender	 *gBartender;	/* The menu handling object 	 */
extern	CDesktop	 *gDesktop;		/* The enclosure for all windows */
extern	OSType		  gSignature;	/* The application's signature 	 */

extern  ObjType	 	  gClipType;	/* Clipboard 					 */
extern  CSymWindow   *gSymWind;		/* Sym window  		   			 */
extern  CFileWndKeys *gKeyPanes;	/* Keyboard display   			 */
extern  CursHandle	  gWatchCursor;	/* Watch cursor handle			 */
extern  Str63		  gFileName;    /* Current key file name		 */

	



/*** Private Variables ***/

static Byte	keyRow1[] = KEY_ROW1;					/* Keyboard map key codes */								
static Byte keyRow2[] = KEY_ROW2;
static Byte keyRow3[] = KEY_ROW3;
static Byte keyRow4[] = KEY_ROW4;


static unsigned char signLabel[] = FBOX2_SIGN;		/* Keyboard type labels */									
static unsigned char fingLabel[] = FBOX2_FINGER;


static unsigned char initKey[] = FBOX1_INIT;		/* Key box init label */									





/*** Private Constants ***/

#define KEYBUF_SIZE		32000





/***
 *  IFileWindow
 *
 ***/
 
void CFileWindow::IFileWindow (CApplication *aSupervisor, Boolean printable)

{
	CDocument::IDocument(aSupervisor, printable);
	
	itsKeyPanes = NULL;			// pane list created in BuildWindow

	itsCurKeyPane = NULL;
	itsCurKbdPane = NULL;

	dirty = FALSE;				// file has not been modified

}



/***
 * Dispose
 *
 ***/

void CFileWindow::Dispose (void)

{
	inherited::Dispose();
}



/***
 * DoCommand
 *
 *	Manage file window commands.
 *
 ***/

void CFileWindow::DoCommand (long theCommand)

{	
	if (theCommand < 0) {
		
		switch (HiShort(-theCommand)) {
		
			case cmdSelectCell1:
			case cmdSelectCell2:
			case cmdSelectCell3:
			case cmdSelectCell4:
			
				itsKeyPanes->UpdateSelect(theCommand);
				itsCurKeyPane->Update(itsKeyPanes->KeyName(theCommand));
				break;
						
			default:
			
				inherited::DoCommand(theCommand);
				break;		
		}
		
	} else {
		switch (theCommand) {
		
					
			case cmdCut:
			
				itsKeyPanes->DoCopy();
				itsKeyPanes->DoClear(iCUT);
				break;
				
					
			case cmdCopy:
			
				itsKeyPanes->DoCopy();
				break;
				
				
			case cmdPaste:
			
				itsKeyPanes->DoPaste();
				break;
				
				
			case cmdClear:
			
				itsKeyPanes->DoClear(iCLEAR);
				break;
			
			
			case cmdUpdateSignKbd:
			
				this->UpdateFile();	
				break;
			
			
			case cmdKbdSign:
			
				itsKeyPanes->Update(sign);
				itsCurKbdPane->Update(signLabel);
				break;
				

			case cmdKbdFingerspelling:
						   
				itsKeyPanes->Update(fingerspelling);
				itsCurKbdPane->Update(fingLabel);
				break;
			
			
			default:
			
				inherited::DoCommand(theCommand);
				break;		
					
		}
    }
}



/***
 * DoKeyDown
 *
 *	Treat valid non-Apple-command keystroke as 
 *  double click on the corresponding key pane key.
 *
 ***/

void CFileWindow::DoKeyDown (char theChar, Byte keyCode, EventRecord *macEvent)

{
	if (!(macEvent->modifiers & cmdKey)) 	
		itsKeyPanes->DoKeyCommand(keyCode);
		
	else
		inherited::DoKeyDown(theChar, keyCode, macEvent);
		
}
		



/***
 * UpdateMenus
 *
 *  Manage file window menus.
 *
 ***/

void CFileWindow::UpdateMenus (void)

{
  	inherited::UpdateMenus();

 	gBartender->EnableCmd(cmdAbout);
 	
	gBartender->DisableCmd(cmdNew);
	gBartender->DisableCmd(cmdOpen);
	gBartender->DisableCmd(cmdPrint);
	gBartender->DisableCmd(cmdPageSetup);
				
 	gBartender->EnableCmd(cmdClose);
 	gBartender->EnableCmd(cmdSave);
 	gBartender->EnableCmd(cmdSaveAs);
 	
 	gBartender->EnableCmd(cmdUpdateSignKbd);

 	if (itsFile != NULL)
 		gBartender->EnableCmd(cmdRevert);
 	

	if (gGopher == this) {
		
		gBartender->EnableCmd(cmdCut);
	 	gBartender->EnableCmd(cmdCopy);
	 	gBartender->EnableCmd(cmdClear);

 		if ((gClipType == key && itsKeyPanes->itsCurKbd == sign) ||
 		    (gClipType == box && itsKeyPanes->itsCurKbd == fingerspelling))   
			gBartender->EnableCmd(cmdPaste);
 	
	}
  	
  	gBartender->EnableCmd(cmdKbd);	
  	gBartender->EnableCmd(cmdKbdSign);	
  	gBartender->EnableCmd(cmdKbdFingerspelling);	

}



/***
 * NewFile
 *
 *	When the user chooses New from the File menu, the CreateDocument()
 *	method in your Application class will send a newly created document
 *	this message. This method needs to create a new window ready for
 *	work with a new document.
 *
 ***/
 
void CFileWindow::NewFile (void)

{    
    short  keyCode;
    
    
		/**
		 **	Initialize all keys to be empty.
	     **
		 **/
    
    for (keyCode = 0; keyCode < KEY_CODES; keyCode++) {
    
    	NewFingerKey(keyCode); 
		NewSignKey(keyCode); 
	}
    
    
		/**
		 **	Build & display symbol window.
		 **
		 **/

	BuildSymWindow();
	gSymWind->itsWindow->Select();
	
	
	
		/**
		 **	Build & display new (empty) file window.
		 **
		 **/

	BuildWindow();
	itsWindow->Select();
	
}



/***
 * OpenFile
 *
 *	When the user chooses Open from the File menu, the OpenDocument()
 *	method in your Application class will let the user choose a file
 *	and then send a newly created document this message. The information
 *	about the file is in the SFReply record.
 *
 ***/

void CFileWindow::OpenFile (SFReply *macSFReply)

{
	CDataFile	*theFile = NULL;
	Str63		theName;
	short		keyCode;
	Handle		keyBuf = NULL;
	short 		keyBytes;
	
	
	TRY
	{
	
			/**
			 **	Initialize all keys to be empty.
		     **
			 **/
		 
		 for (keyCode = 0; keyCode < KEY_CODES; keyCode++) {
	    
	    	NewFingerKey(keyCode); 
			NewSignKey(keyCode); 			
		 }
		 
		 
		
			/**
			 ** Create a file and send it a SFSpecify()
			 **	message to set up the name, volume, and
			 **	directory.
			 **
			 **/
	
		theFile = new CDataFile;
		
		theFile->IDataFile();
		theFile->SFSpecify(macSFReply);
		
		
		
			/**
			 **	Be sure to set the instance variable
			 **	so other methods can use the file if they
			 **	need to. This is especially important if
			 **	you leave the file open in this method.
			 **
			 **/
	
		itsFile = theFile;
		
		
	
			/**
			 **	Send the file an Open() message to open it.
			 **
			 **/
	
		theFile->Open(fsRdWrPerm);
		
		
		
			/**
			 **	Make sure that the memory request to allocate
			 **	the read buffer doesn't use up any of our rainy
			 ** day fund and that the GrowMemory() method (in 
			 ** the application) knows that it's OK if we 
			 ** couldn't get enough memory.
			 **
			 **/
	
		SetCriticalOperation(FALSE);
		SetAllocation(kAllocCanFail);
		
		keyBuf = NewHandle(KEYBUF_SIZE);
		
		
			
			/**
			 **	Read fingerspell keyboard
			 **
			 **/
			 
		for (keyCode = 0; keyCode < KEY_CODES; keyCode++) {
			 
			theFile->ReadSome((Ptr) &keyBytes, sizeof(short));
		
			if (keyBytes != 0) {
			
				HLock(keyBuf);
				theFile->ReadSome(*keyBuf, keyBytes);
				HUnlock(keyBuf);
			}
		
			UnpackFingerKey(keyCode, keyBuf, keyBytes);
											
		}	
		
		
			/**
			 **	Read sign keyboard
			 **
			 **/
			 
		for (keyCode = 0; keyCode < KEY_CODES; keyCode++) {
			 
			theFile->ReadSome((Ptr) &keyBytes, sizeof(short));
		
			if (keyBytes != 0) {
			
				HLock(keyBuf);
				theFile->ReadSome(*keyBuf, keyBytes);
				HUnlock(keyBuf);
			}
		
			UnpackSignKey(keyCode, keyBuf, keyBytes);
												
		}	
			
			
			/**
			 **	Deallocate read buffer
			 **
			 **/
			 
		DisposHandle(keyBuf);
		keyBuf = NULL;
		
    
    
			/**
			 **	Build & display symbol window.
			 **
			 **/

		BuildSymWindow();
		gSymWind->itsWindow->Select();
	
		 
	
			/**
			 **	Build & display opened file window.
			 **
			 **/
		 
		BuildWindow();
		 
		theFile->GetName(theName);
		itsWindow->SetTitle(theName);
		itsWindow->Select();


	
			/**
			 **	Set current key file name
			 **
			 **/
		 
		CopyPString(theName, gFileName);
		
		
		
	}
	CATCH
	{
		 if (keyBuf) DisposHandle(keyBuf);
		 
		 for (keyCode = 0; keyCode < KEY_CODES; keyCode++) {
		
			DisposFingerKey(keyCode);
			DisposSignKey(keyCode);	
		 }
		 
	}
	ENDTRY;
	
}



/***
 * UpdateFile
 *
 *	When the user chooses Update Sign Keyboard from the File menu, 
 *  display the file dialog box to get the name of the key file to update.
 *  Update the specified key file so it contains the sign keyboard
 *  definition of the currently open file, while retaining its existing
 *  fingerspelling keyboard definition.
 *
 ***/
 
void CFileWindow::UpdateFile (void)

{
	SFReply 	macSFReply;
	CDataFile	*keyFile = NULL;
	short		keyCode;
	Handle		keyBuf = NULL;
	short 		keyBytes;
	
	
	gApplication->sfGetDLOGHook = (DlgHookProcPtr) UpdateFileHook;	
	gApplication->sfFileFilter  = (FileFilterProcPtr) UpdateFileFilter;	

	gApplication->ChooseFile(&macSFReply);				// Display file dialog
		
	gApplication->sfGetDLOGHook = NULL;
	gApplication->sfFileFilter  = NULL;

	if (!macSFReply.good) return;
	

	SetCursor(*gWatchCursor);							// Change cursor

	TRY
	{
		keyFile = new CDataFile;
		
		keyFile->IDataFile();							// Open file
		keyFile->SFSpecify(&macSFReply);
		keyFile->Open(fsRdWrPerm);
		
	
		SetCriticalOperation(FALSE);					// Create buffer
		SetAllocation(kAllocCanFail);
		
		keyBuf = NewHandle(KEYBUF_SIZE);
		
		
		for (keyCode = 0; keyCode < KEY_CODES; keyCode++) {		// skip finger keys
			 
			keyFile->ReadSome((Ptr) &keyBytes, sizeof(short));
		
			if (keyBytes != 0) {
			
				HLock(keyBuf);
				keyFile->ReadSome(*keyBuf, keyBytes);
				HUnlock(keyBuf);
			}
											
		}
		
				 
		for (keyCode = 0; keyCode < KEY_CODES; keyCode++) {		// update sign keys

 			PackSignKey(keyCode, keyBuf, &keyBytes);

			((CDataFile *) keyFile)->WriteSome((Ptr) &keyBytes, sizeof(short));
		
			if (keyBytes != 0) {
				HLock(keyBuf);
				((CDataFile *) keyFile)->WriteSome(*keyBuf, keyBytes);
				HUnlock(keyBuf);
			}
			
		}	
		
		
		DisposHandle(keyBuf);							// Dispose buffer
		keyBuf = NULL;
		
		
		keyFile->Dispose();								// Close file
		keyFile = NULL;		
		
	}
	
	CATCH
	{
		 if (keyBuf) DisposHandle(keyBuf);
		 if (keyFile) keyFile->Dispose();
	}
	ENDTRY;
	
}



/***
 * DoSave
 *
 *	This method handles what happens when the user chooses Save from the
 *	File menu. This method should return TRUE if the file save was successful.
 *	If there is no file associated with the document, you should send a
 *	DoSaveFileAs() message.
 *
 ***/

Boolean CFileWindow::DoSave(void)

{	
	short		keyCode;
	Handle		keyBuf = NULL;
	short 		keyBytes;
	
	
	if (itsFile == NULL)
		return DoSaveFileAs();
	
		
	TRY
	{
			/**
			 **	Reset file position.
			 **
			 **/
			 
		((CDataFile *) itsFile)->SetMark(0, fsFromStart);
			 
		
			/**
			 **	Alloc write buffer by any means necessary.
			 **
			 **/
			 
		SetCriticalOperation(TRUE);
		SetAllocation(kAllocCantFail);
		
		keyBuf = NewHandle(KEYBUF_SIZE);
		
							
			/**
			 **	Write fingerspell keyboard
			 **
			 **/
			 		 
		for (keyCode = 0; keyCode < KEY_CODES; keyCode++) {

 			PackFingerKey(keyCode, keyBuf, &keyBytes);

			((CDataFile *) itsFile)->WriteSome((Ptr) &keyBytes, sizeof(short));
		
			if (keyBytes != 0) {
				HLock(keyBuf);
				((CDataFile *) itsFile)->WriteSome(*keyBuf, keyBytes);
				HUnlock(keyBuf);
			}
			
		}	
		
							
			/**
			 **	Write sign keyboard
			 **
			 **/
			 		 
		for (keyCode = 0; keyCode < KEY_CODES; keyCode++) {

 			PackSignKey(keyCode, keyBuf, &keyBytes);

			((CDataFile *) itsFile)->WriteSome((Ptr) &keyBytes, sizeof(short));
		
			if (keyBytes != 0) {
				HLock(keyBuf);
				((CDataFile *) itsFile)->WriteSome(*keyBuf, keyBytes);
				HUnlock(keyBuf);
			}
			
		}	
		
			
			/**
			 **	Deallocate read buffer
			 **
			 **/
			 	    
		DisposHandle(keyBuf);
		keyBuf = NULL;
		
		
		dirty = FALSE;
		return (TRUE);					/* Save successful */
		
	}
	
	CATCH
	{
		 if (keyBuf) DisposHandle(keyBuf);		 
	}
	ENDTRY;
	
}



/***
 * DoSaveAs
 *
 *	This method handles what happens when the user chooses Save As from
 *	File menu. The default DoCommand() method for documents sends a DoSaveFileAs()
 *	message which displays a standard put file dialog and sends this message.
 *	The SFReply record contains all the information about the file you're about
 *	to create.
 *
 ***/

Boolean CFileWindow::DoSaveAs(SFReply *macSFReply)

{
		/**
		 **	If there's a file associated with this document
		 **	already, close it. The Dispose() method for files
		 **	sends a Close() message to the file before releasing
		 **	its memory.
		 **
		 **/
		 
	if (itsFile != NULL)
		itsFile->Dispose();


		/**
		 **	Create a new file, and then save it normally.
		 **
		 **/
		 
	SetCriticalOperation(TRUE);
	SetAllocation(kAllocCantFail);
	
	itsFile = new CDataFile;
	
	((CDataFile *) itsFile)->IDataFile();
	itsFile->SFSpecify(macSFReply);
	
	if (itsFile->ExistsOnDisk())
		itsFile->ThrowOut();
		
	itsFile->CreateNew(gSignature, 'SWKY');
	itsFile->Open(fsRdWrPerm);
	
	itsWindow->SetTitle(macSFReply->fName);
	 
	CopyPString(macSFReply->fName, gFileName);
	
	return DoSave();
}
	


/***
 * DoRevert
 *
 *  Close the current file (without writing anything out)
 *	and read the last saved version of the file.
 *
 ***/

void CFileWindow::DoRevert(void)

{
	short	keyCode;
	Handle	keyBuf = NULL;
	short 	keyBytes;

										/* Only file window can be open		*/
	if (itsKeyPanes->KeyWindowsOpen())
		return;
		
										/* Dispose existing key data		*/	
																		
	for (keyCode = 0; keyCode < KEY_CODES; keyCode++) {
	
		DisposFingerKey(keyCode);
		DisposSignKey(keyCode);
	}
										/* Reallocate new key data			*/
					
	for (keyCode = 0; keyCode < KEY_CODES; keyCode++) {
	    
	    NewFingerKey(keyCode); 
		NewSignKey(keyCode); 			
	}
		 
		 
	itsFile->Close();		
	itsFile->Open(fsRdWrPerm);			/* Reopen file and read key data	*/
			
									
	TRY
	{		
		SetCriticalOperation(FALSE);
		SetAllocation(kAllocCanFail);
		
		keyBuf = NewHandle(KEYBUF_SIZE);
		
			 
		for (keyCode = 0; keyCode < KEY_CODES; keyCode++) {
			 
			((CDataFile *) itsFile)->ReadSome((Ptr) &keyBytes, sizeof(short));
		
			if (keyBytes != 0) {
			
				HLock(keyBuf);
				((CDataFile *) itsFile)->ReadSome(*keyBuf, keyBytes);
				HUnlock(keyBuf);
			}
		
			UnpackFingerKey(keyCode, keyBuf, keyBytes);
												
		}
			
			 
		for (keyCode = 0; keyCode < KEY_CODES; keyCode++) {
			 
			((CDataFile *) itsFile)->ReadSome((Ptr) &keyBytes, sizeof(short));
		
			if (keyBytes != 0) {
			
				HLock(keyBuf);
				((CDataFile *) itsFile)->ReadSome(*keyBuf, keyBytes);
				HUnlock(keyBuf);
			}
		
			UnpackSignKey(keyCode, keyBuf, keyBytes);
												
		}	
		
			 
		DisposHandle(keyBuf);
		keyBuf = NULL;
		
		
		itsKeyPanes->Refresh();			/* Update with reverted contents   */	

	}
	
	CATCH
	{
		 if (keyBuf) DisposHandle(keyBuf);		 
 	}
	ENDTRY;
}



/***
 * Close
 *
 *  Close the current document and dispose 
 *	KeyEdit's keyboard data structures.
 *
 ***/

Boolean CFileWindow::Close (Boolean quitting)

{
	Boolean 	result = inherited::Close(quitting);
	short		keyCode;
	
							/* Dispose existing key data */		
	if (result)	{					
																											
		for (keyCode = 0; keyCode < KEY_CODES; keyCode++) {
	
			DisposFingerKey(keyCode);
			DisposSignKey(keyCode);

		}
	}
			
	return result;
}



/***
 * BuildSymWindow
 *
 *	The NewFile() and OpenFile() methods use
 *  this to create the symbol window.
 *
 ***/

void CFileWindow::BuildSymWindow()

{
  	CSymWindow	*theWindow = NULL;

	TRY
	{
		theWindow = new CSymWindow;
		
		theWindow->ISymWindow((CDirectorOwner*) this);
		
		gSymWind = theWindow;		// used in key box editing (to add symbols)
	}
	
	CATCH
	{
		 if (theWindow) theWindow->Dispose();
		 
		 gSymWind = NULL;
	}
	ENDTRY;
}

						
									
/***
 * BuildWindow
 *
 *	This is the auxiliary window-building method that the
 *	NewFile() and OpenFile() methods use to create the
 *  keyboard window.
 *
 ***/

void CFileWindow::BuildWindow()

{
	
	CFileWndKeys	*theKeyPane1;
	CFileWndKeys	*theKeyPane2;
	CFileWndKeys	*theKeyPane3;
	CFileWndKeys	*theKeyPane4;
		
	CFileWndBox		*theBoxPane1;
	CFileWndBox		*theBoxPane2;
	
	CPaneBorder		*theKeyBorder1;
	CPaneBorder		*theKeyBorder2;
	CPaneBorder		*theKeyBorder3;
	CPaneBorder		*theKeyBorder4;
	
	CPaneBorder		*theBoxBorder1;
	CPaneBorder		*theBoxBorder2;
	
	


	   /**
		**	First create the window and initialize
		**	it. The first argument is the resource ID
		**	of the window. The second argument specifies
		**	whether the window is a floating window.
		**	The third argument is the window's enclosure; it
		**	should always be gDesktop. The last argument is
		**	the window's supervisor in the Chain of Command;
		**	it should always be the Document object.
		**
		**/

	itsWindow = new CWindow;
	itsWindow->IWindow(FWND_RSRC, FALSE, gDesktop, this);
	
	
		/**
		 **	theBoxPane1 controls the box at the bottom left
		 ** corner of the file window.  It is used to 
		 ** display the letter name of the key currently
		 ** selected in the file window. Init value: <blank>
		 **
		 **/
		 
	theBoxPane1 = new CFileWndBox;
 	theBoxPane1->IFileWndBox(itsWindow, this,
    						 FBOX1_WIDTH, FBOX_HEIGHT,
    						 0, FWND_HEIGHT - FBOX_HEIGHT, 
    						 sizFIXEDLEFT, sizFIXEDBOTTOM);
    						
    itsCurKeyPane = theBoxPane1;

	
		/**
		 **	theBoxBorder1 is used to draw the top
		 ** and right borders on the box pane.
		 **
		 **/
		 
	theBoxBorder1 = new CPaneBorder;
	theBoxBorder1->IPaneBorder(kBorderTop + kBorderRight);
	
	theBoxPane1->SetBorder(theBoxBorder1);

	
		/**
		 **	theBoxPane2 controls the box at the bottom right
		 ** corner of the file window.  It is used to 
		 ** display the name (e.g. sign or fingerspelling)
		 ** of the currently displayed keyboard.  Init value: sign
		 **
		 **/
		 
	theBoxPane2 = new CFileWndBox;
 	theBoxPane2->IFileWndBox(itsWindow, this,
    						 FBOX2_WIDTH, FBOX_HEIGHT,
    						 FBOX1_WIDTH + 1, FWND_HEIGHT - FBOX_HEIGHT, 
    						 sizFIXEDRIGHT, sizFIXEDBOTTOM);
    						
    itsCurKbdPane = theBoxPane2; 
    strcpy((char *) itsCurKbdPane->itsLabel, (char *) signLabel);

	
		/**
		 **	theBoxBorder2 is used to draw
		 ** the top border on the box pane.
		 **
		 **/
		 
	theBoxBorder2 = new CPaneBorder;
	theBoxBorder2->IPaneBorder(kBorderTop);
	
	theBoxPane2->SetBorder(theBoxBorder2);


		/**
		 **	theKeyPane1 thru theKeyPane4 define the file window's
		 ** keyboard grid.  Each pane controls one row of keys.
		 **
		 **/

	theKeyPane1 = new CFileWndKeys;
	theKeyPane2 = new CFileWndKeys;
	theKeyPane3 = new CFileWndKeys;
	theKeyPane4 = new CFileWndKeys;


    theKeyPane1->IFileWndKeys(itsWindow, this,
    						  FKEY1_LEFT, FKEY1_TOP, 
    						  SIZ_ROW1, keyRow1,
    						  cmdSelectCell1, theKeyPane2);	
    
    theKeyPane2->IFileWndKeys(itsWindow, this,
    						  FKEY2_LEFT, FKEY2_TOP, 
    						  SIZ_ROW2, keyRow2,
    						  cmdSelectCell2, theKeyPane3);				
    
    theKeyPane3->IFileWndKeys(itsWindow, this,
    						  FKEY3_LEFT, FKEY3_TOP, 
    						  SIZ_ROW3, keyRow3,
    						  cmdSelectCell3, theKeyPane4);						  
    
    theKeyPane4->IFileWndKeys(itsWindow, this,
    						  FKEY4_LEFT, FKEY4_TOP, 
    						  SIZ_ROW4, keyRow4,
    						  cmdSelectCell4, NULL);
		  
		  
		  								// 'A' key selected as default
	theKeyPane3->ChangeSelection(1);
    strcpy((char *) itsCurKeyPane->itsLabel, (char *) initKey);

	itsKeyPanes = theKeyPane1;			// key panes list head
	gKeyPanes   = itsKeyPanes;			// used in key box editing (to update keyboard)
	

		/**
		 **	theKeyBorder1 thru theKeyBorder4 outline the
		 ** file window's keyboard grid. 
		 **
		 **/
		 		 
	theKeyBorder1 = new CPaneBorder;
	theKeyBorder2 = new CPaneBorder;
	theKeyBorder3 = new CPaneBorder;
	theKeyBorder4 = new CPaneBorder;

	theKeyBorder1->IPaneBorder(kBorderFrame);
	theKeyBorder2->IPaneBorder(kBorderFrame);
	theKeyBorder3->IPaneBorder(kBorderFrame);
	theKeyBorder4->IPaneBorder(kBorderFrame);

	theKeyPane1->SetBorder(theKeyBorder1);
	theKeyPane2->SetBorder(theKeyBorder2);
	theKeyPane3->SetBorder(theKeyBorder3);
	theKeyPane4->SetBorder(theKeyBorder4);

		 
		/**
		 **	itsMainPane is the document's focus
		 **	of attention. Some of the standard
		 **	classes (particularly CPrinter) rely
		 **	on itsMainPane pointing to the main
		 **	pane of your window.
		 **
		 **/
		 
	itsMainPane = theKeyPane1;
	

		/**
		 **
		 ** The document class fields all file window
		 ** commands.
		 ** 
		 **/
		 
	this->BecomeGopher(TRUE);
}
