/****
 * CFingKeyWindow.c
 *
 *	Fingerspell key window methods for the SignWriter key editor.
 *
 *  Copyright  1996 The Center for Sutton Movement Writing.  All rights reserved.
 *
 ****/

#include "CFingKeyWindow.h"
#include "Commands.h"
#include <Constants.h>
#include "CApplication.h"
#include "CBartender.h"
#include "CDesktop.h"
#include "CFileWndKeys.h"
#include "CFingKeyWndPane.h"
#include "CPaneBorder.h"
#include "CSymWindow.h"
#include "CWindow.h"
#include "Global.h"
#include "Keys.h"
#include "KeyEditConsts.h"
#include "Symbols.h"
#include "TBUtilities.h"





/*** Global Variables ***/


extern	CBartender	 *gBartender;	/* The menu handling object 			    */
extern	CDesktop	 *gDesktop;		/* The enclosure for all windows 			*/
extern	CBureaucrat	 *gGopher;		/* The current boss in the chain of command */

extern  CSymWindow   *gSymWind;		/* Sym window access */

extern  CFileWndKeys *gKeyPanes;	/* Key display access */

extern  void		 *gClipRec;		/* Clipboard */
extern  ObjType	 	  gClipType;





/***
 * IFingKeyWindow
 *
 *  Initialize a new instance of a fingerspell key window.
 *
 ***/
 
void CFingKeyWindow::IFingKeyWindow (CDirectorOwner *aSupervisor,
									 Byte keyCode, char keyName)

{
	itsPane 	= NULL;
	itsKeyCode  = keyCode;
	itsKeyName  = keyName;
	
	inherited::IDirector(aSupervisor);
}



/***
 * Dispose
 *
 *
 ***/

void CFingKeyWindow::Dispose()

{
	inherited::Dispose();
}
			
			
					
/***
 * Dawdle
 *
 *
 ***/

void CFingKeyWindow::Dawdle (long *maxSleep)

{
	itsPane->BlinkSelect(maxSleep);
}			
			
		
					
/***
 * DoCommand
 *
 *
 ***/

void CFingKeyWindow::DoCommand (long theCommand)

{
	switch (theCommand) {
			
	
		case cmdCut:
		
			itsPane->DoCopy();
			itsPane->DoClear(iCUT);
			break;
			
				
		case cmdCopy:
		
			itsPane->DoCopy();
			break;
			
			
		case cmdPaste:
		
			itsPane->DoPaste();
			break;
			
			
		case cmdClear:
		
			itsPane->DoClear(iCLEAR);
			break;
		
			
		case cmdAdd:
		
			itsPane->DoAdd();
			break;

		
		case cmdFill:
		
			itsPane->DoFill();
			break;	
			
			
		case cmdCursor:
		
			itsPane->DoCursor();
			break;	
		
			
		case cmdMirror:
		
			itsPane->DoMirror();
			break;
			
			
		case cmdChange:
		
			itsPane->DoChange();
			break;	
			
			
		case cmdRotate:
		
			itsPane->DoRotate();
			break;
				
				
		default:
		
			inherited::DoCommand(theCommand);
			break;
	}
}



/***
 * DoKeyDown
 *
 *	Handle SignWriter symbol command keys.
 *
 ***/

void CFingKeyWindow::DoKeyDown (char theChar, Byte keyCode, EventRecord *macEvent)

{
	if (!(macEvent->modifiers & cmdKey)) 	
	 	itsPane->DoSymCommand(keyCode);

	else
		inherited::DoKeyDown(theChar, keyCode, macEvent);	
			
}



/***
 * UpdateMenus
 *
 *
***/

void CFingKeyWindow::UpdateMenus (void)

{ 	
 	// do inherited
  	inherited::UpdateMenus();
  	
  	
  	// Apple menu
  	
  	gBartender->EnableCmd(cmdAbout);
 	
 	
  	// file management
  	
  	gBartender->DisableCmd(cmdNew);
	gBartender->DisableCmd(cmdOpen);
	gBartender->DisableCmd(cmdPrint);
	gBartender->DisableCmd(cmdPageSetup);
	
 	gBartender->EnableCmd(cmdClose);
 	gBartender->EnableCmd(cmdSave);
 	gBartender->EnableCmd(cmdSaveAs);
 	
 	gBartender->EnableCmd(cmdUpdateSignKbd);
	gBartender->EnableCmd(cmdRevert);	


	if (gGopher == this) {
	
		// symbol editing	 
	
		if (gSymWind->SelectedSymbol() > 0)
			gBartender->EnableCmd(cmdAdd);
		
		if (itsPane->SelectionType() == sym) {
		
			gBartender->EnableCmd(cmdFill);
		 	gBartender->EnableCmd(cmdCursor);
		 	gBartender->EnableCmd(cmdMirror);
		 	gBartender->EnableCmd(cmdChange);
		 	gBartender->EnableCmd(cmdRotate);		 	
		}
		
		if (itsPane->SelectionType() != none) {
		
		 	gBartender->EnableCmd(cmdCut);
		 	gBartender->EnableCmd(cmdCopy);
		 	gBartender->EnableCmd(cmdClear);	
		}
			
 		if (gClipType == box || gClipType == sym)
 			gBartender->EnableCmd(cmdPaste);	
 					
	}	
	
}
		
					
			
/***
 * BuildWindow
 *
 *	Build new instance of a sign key window and
 *	display the data specified by the argument.
 *
 ***/
	
void CFingKeyWindow::BuildWindow (void)

{
	CFingKeyWndPane	*thePane;
	CPaneBorder     *theBorder;

	Rect			wLoc;
	static short	wCount = 0;	
	Str15			winTitle = " ";

	
	   /**
		**	Get the (staggered) window location.
		**
		**/
		
	SetRect(&wLoc, FKWND_LEFT, FKWND_TOP, 
				   FKWND_LEFT + FKWND_WIDTH, FKWND_TOP + FKWND_HEIGHT);
				   
	OffsetRect(&wLoc, wCount * HORIZ_WOFFSET, wCount * VERT_WOFFSET);
	wCount = (wCount + 1) % MAX_WOFFSETS;
	
	
	   /**
		**	Make the (invisible) window.
		**
		**/
		
	itsWindow = new CWindow;
	itsWindow->INewWindow(&wLoc, FALSE, 4, FALSE, TRUE, gDesktop, this);
	
	
		/**
		 **  Set window title to key name
		 **
		 **/
		 
	winTitle[0] = itsKeyName;
	itsWindow->SetTitle(CtoPstr(winTitle));
	
	
		/**
		 **	thePane controls the symbol box that appears in
		 ** the fingerspell key window.  It is used to display
		 ** and edit the symbol definition of the selected key.
		 **
		 **/

	thePane = new CFingKeyWndPane;
    thePane->IFingKeyWndPane(itsWindow, this, itsKeyCode);
       
    thePane->SetWantsClicks(TRUE);
    						 
    itsPane = thePane;
	
	
		/**
		 **  Create pane border.
		 **
		 **/
		 
	theBorder = new CPaneBorder;
	theBorder->IPaneBorder(kBorderFrame);
	
	thePane->SetBorder(theBorder);
	
	
		/**
		 **
		 ** The current class fields all sym window
		 ** commands.
		 ** 
		 **/
		 
	this->BecomeGopher(TRUE);
	
	
		/**
		 **	Link the new window to its parent window.
		 **
		 **/
		 
	gKeyPanes->SetFingerKeyWindow(itsKeyCode, this);
	 
	 
		/**
		 **	Send the window a Select() message to make
		 **	it the active window.
		 **
		 **/
	
	itsWindow->Select();
}



/***
 * Close (override)
 *
 *	Update (& unlink from) file window before closing
 *
 ***/
 
Boolean	CFingKeyWindow::Close (Boolean quitting)

{
	itsPane->Close();
	
	if (gKeyPanes->itsCurKbd == fingerspelling)
		gKeyPanes->UpdateKey(itsKeyCode);
		
	gKeyPanes->SetFingerKeyWindow(itsKeyCode, NULL);
	
	return inherited::Close(quitting);
}
	
