/****
 * CSymWindow.c
 *
 *	Document methods for the symbol window class.
 *
 *  Copyright  1995 The Center for Sutton Movement Writing.  All rights reserved.
 *
 ****/
 


#include "Global.h"
#include "Commands.h"
#include "CApplication.h"
#include "CBartender.h"
#include "CBureaucrat.h"
#include "CDataFile.h"
#include "CDecorator.h"
#include "CDesktop.h"
#include "CPaneBorder.h"
#include "CPanorama.h"
#include "CScrollPane.h"
#include "CSelector.h"
#include "CSymWindow.h"
#include "CSymWndGrid.h"
#include "CSymWndBox.h"
#include "CWindow.h"
#include <Packages.h>
#include "Symbols.h"
#include "KeyEditConsts.h"
#include "TBUtilities.h"
#include "TCLUtilities.h"




/*** Global Variables ***/

extern	CBureaucrat  *gGopher;		/* The gopher					 */
extern	CBartender	 *gBartender;	/* The menu handling object 	 */
extern	CDecorator	 *gDecorator;	/* Window dressing object		 */
extern	CDesktop	 *gDesktop;		/* The enclosure for all windows */
extern	OSType		  gSignature;	/* The application's signature 	 */





/***
 *  ISymWindow
 *
 ***/
 
void CSymWindow::ISymWindow (CDirectorOwner *aSupervisor)

{
	inherited::IDirector(aSupervisor);
	
	itsGridPane = NULL;		// panes created in NewWindow
	itsBoxPane  = NULL;

	NewWindow();
	
	itsWindow->Select();
	
}



/***
 * Dispose
 *
 ***/

void CSymWindow::Dispose (void)

{
	inherited::Dispose();
}



/***
 * SelectedSymbol
 *
 *	Return the symcode of the 
 *  currently selected symbol.
 *
 ***/

short CSymWindow::SelectedSymbol (void)

{
	return itsBoxPane->itsSymCode;	
}



/***
 * DoCommand
 *
 *	Manage sym window commands.
 *
 ***/

void CSymWindow::DoCommand (long theCommand)

{
	if (theCommand < 0) {
		switch (HiShort(-theCommand)) {
		
			case cmdSelectCell:
				itsBoxPane->Update(itsGridPane->ItemToSymCode(LoShort(-theCommand)));
				break;
						
			default:
				inherited::DoCommand(theCommand);
		}
		
	} else 
	
		inherited::DoCommand(theCommand);
	
}



/***
 * UpdateMenus
 *
 *  Manage file window menus.
 *
 ***/

void CSymWindow::UpdateMenus (void)

{
  	inherited::UpdateMenus();

}



/***
 * NewWindow
 *
 *	Create the symbol window.  The symbol file has
 *	already been read into memory (in CKeyEditApp).
 *
 ***/

void CSymWindow::NewWindow ()

{
	Str15			winTitle = SWND_TITLE;
	CScrollPane		*theScrollPane;
	CSymWndGrid		*theMainPane;
	CSymWndBox		*theBoxPane;
	CPaneBorder		*theBoxBorder;
	Rect			aRect;

	   /**
		**	First create the window and initialize
		**	it. The first argument is the resource ID
		**	of the window. The second argument specifies
		**	whether the window is a floating window.
		**	The third argument is the window's enclosure; it
		**	should always be gDesktop. The last argument is
		**	the window's supervisor in the Chain of Command;
		**	it should always be the Document object.
		**
		**/

	itsWindow = new CWindow;
	itsWindow->IWindow(SWND_RSRC, FALSE, gDesktop, this);
	
	
		/**
		 **  Set the window title
		 **
		 **/
		 
	itsWindow->SetTitle(CtoPstr(winTitle));
	
	
		/**
		 **	After you create the window, you can use the
		 **	SetSizeRect() message to set the windows maximum
		 **	and minimum size. Be sure to set the max & min
		 **	BEFORE you send a PlaceNewWindow() message to the
		 **	decorator.
		 **
		 **/
		 
	SetRect(&aRect, SWND_MINWIDTH, SWND_HEIGHT, SWND_MAXWIDTH, SWND_HEIGHT);
	itsWindow->SetSizeRect(&aRect);


		/**
         ** The symfile window contains a display pane
         ** (for showing the current symbol code) and
         ** also a ScrollPane (which in turn contains
         ** a Panorama.
		 **
		 **/

    theScrollPane = new CScrollPane;
   
	theScrollPane->IScrollPane(itsWindow, this, 0, 0, 0, 0,
								sizELASTIC, sizFIXEDLEFT,
								TRUE, FALSE, TRUE);

		/**
		 **	The FitToEnclFrame() method makes the
		 **	scroll pane be as large as its enclosure.
		 **	In this case, the enclosure is the window,
		 **	so the scroll pane will take up the entire
		 **	window.
		 **
		 **/

	theScrollPane->FitToEnclFrame(TRUE, TRUE);


		/**
		 **	theBoxPane controls the box at the bottom left
		 ** corner of the symfile window.  It is used to 
		 ** display the code value of the symbol currently
		 ** selected in the symfile window.
		 **
		 **/
		 
	theBoxPane = new CSymWndBox;
 	theBoxPane->ISymWndBox(theScrollPane, this,
    						SBOX_WIDTH, SBOX_HEIGHT,
    						0, SWND_HEIGHT - SBOX_HEIGHT, 
    						sizFIXEDLEFT, sizFIXEDBOTTOM);
    						
    itsBoxPane = theBoxPane;

	
		/**
		 **	theBoxBorder is used to draw the top
		 ** border on the box pane.
		 **
		 **/
		 
	theBoxBorder = new CPaneBorder;
	theBoxBorder->IPaneBorder(kBorderTop);
	
	theBoxPane->SetBorder(theBoxBorder);


		/**
		 **	itsMainPane is the document's focus
		 **	of attention. Some of the standard
		 **	classes (particularly CPrinter) rely
		 **	on itsMainPane pointing to the main
		 **	pane of your window.
		 **
		 **	Note that the main pane is the
		 **	panorama in the scroll pane and not
		 **	the scroll pane itself.
		 **
		 **/

	theMainPane = new CSymWndGrid;
    theMainPane->ISymWndGrid(theScrollPane, this,
    						  0, 0, 0, 0, sizELASTIC, sizELASTIC);
    						  
    theMainPane->SetCommandBase(cmdSelectCell);
   						  
	itsGridPane = theMainPane;
	
	
		/**
		 **	The FitToEnclosure() method makes the pane
		 **	fit inside the enclosure. The inside (or
		 **	interior) of a scroll pane is defined as
		 **	the area inside the scroll bars.
		 **
		 **/

	theMainPane->FitToEnclosure(TRUE, TRUE);


		/**
		 **	Send the scroll pane an InstallPanorama()
         ** message to associate the panorama with 
         ** the scroll pane.
		 **
		 **/

	theScrollPane->InstallPanorama(theMainPane);
	
	
		/**
		 **
		 ** The document class fields all sym window
		 ** commands.
		 ** 
		 **/
		 
	this->BecomeGopher(TRUE);
}

