/******************************************************************************
 Keys.c

		Key data manager for the SignWriter key editor
		
		Copyright  1996 Center for Sutton Movement Writing. All rights reserved.

 ******************************************************************************/
 
 
 
#include "Keys.h"
#include "Exceptions.h"
#include "Global.h"
#include "LongCoordinates.h"
#include "Memory.h"
#include "KeyEditConsts.h"
#include "Symbols.h"





/**************************  Key data structures  ************************/



static BoxRow   signKey[KEY_CODES]   = {NULL};		// sign keyboard
static Box  	fingerKey[KEY_CODES] = {NULL};		// fingerspelling keyboard





/**************************  Private prototypes  ************************/


Box     NewBox (void);											// Allocation
BoxRow  NewBoxRow (void);


void    DisposBox (Box box);									// Deallocation
void    DisposBoxRow (BoxRow row);								


void    CopyBox (Box box, Box *copyBox);						// Duplication
void    CopyBoxRow (BoxRow row, BoxRow *copyRow);


void    ClearBox (Box box);										// Clearing
void    ClearBoxRow (BoxRow row);


void	DrawBox (Box box, BitMap *img, Rect *keyBox, 			// Drawing
				 Boolean thumbNail);
												   
void  	DrawBoxSelect (Box box, BitMap *img, Rect *keyBox);				 
	
				 
void	GetByte  (short *value);								// Packing support
void	GetBytes (void *base, short bytes);
void	PutByte  (short value);
void	PutBytes (void *base, short bytes);		 		 
				 
				 
void 	PackBox   (Box box);									// Packing
void 	UnpackBox (Box box);





/*****************************  Allocation  *****************************/


/***
 * NewBox
 *
 ***/
 
Box NewBox (void)

{
	Box	newBox;
	
	
	newBox = (Box) NewPtrClear(sizeof(BoxRec));
	FailMemError();
	
	newBox->descender = FKPANE_HEIGHT / 2;				//  initially centered in box
	newBox->selType = none;
	newBox->selSym = -1;
	newBox->numSyms = 0;

	return newBox;
	
}



/***
 * NewBoxRow
 *
 ***/
 
BoxRow NewBoxRow (void)

{
	BoxRow	newRow;
	short	box;
	
	
	newRow = (BoxRow) NewPtrClear(sizeof(RowRec));
	FailMemError();
	
	for (box = 0; box < KEY_VARS; box++)
		(*newRow)[box] = NewBox();
		
	return newRow;
	
}



/***
 * NewSignKey
 *
 ***/
 	
void NewSignKey (short keyCode)

{
	signKey[keyCode] = NewBoxRow();
}



/***
 * NewFingerKey
 *
 ***/
 
void NewFingerKey (short keyCode)

{
	fingerKey[keyCode] = NewBox();
}




/*****************************  Deallocation  *****************************/


/***
 * DisposBox
 *
 ***/
 
void DisposBox (Box box)

{
	if (box != NULL) DisposPtr(box);
}



/***
 * DisposBoxRow
 *
 ***/
 
void DisposBoxRow (BoxRow row)								

{
	short	box;
	
	
	if (row == NULL) return;
	
	for (box = 0; box < KEY_VARS; box++)
		DisposBox((*row)[box]);
		
	DisposPtr(row);
	
}



/***
 * DisposSignKey
 *
 ***/
 	
void DisposSignKey (short keyCode)

{
	DisposBoxRow(signKey[keyCode]);
}



/***
 * DisposFingerKey
 *
 ***/
 
void DisposFingerKey (short keyCode)

{
	DisposBox(fingerKey[keyCode]);
}



/***
 * DisposKey
 *
 ***/
 
void DisposKey (ObjType type, void *rec)		// may be Box or BoxRow

{
	if (rec == NULL) return;
	
	if (type == key)
		DisposBoxRow((BoxRow) rec);	
	else
		DisposBox((Box) rec);
		
}




/*****************************  Duplication  *****************************/


/***
 * CopyBox
 *
 ***/
 
void CopyBox (Box box, Box *copyBox)

{
	*copyBox = NULL;
	if (box == NULL) return;
	
	*copyBox = NewBox();
	
	**copyBox = *box;
	
}



/***
 * CopyBoxRow
 *
 ***/
 
void CopyBoxRow (BoxRow row, BoxRow *copyRow)

{
	short  box;
	
	
	*copyRow = NULL;
	if (row == NULL) return;
	
	*copyRow = NewBoxRow();
		
	for (box = 0; box < KEY_VARS; box++)
		*((**copyRow)[box]) = *((*row)[box]);
	
}



/***
 * CopySignKey
 *
 ***/
 	
void CopySignKey (BoxRow rec, BoxRow *copyRec)

{
	CopyBoxRow(rec, copyRec);
}



/***
 * CopyFingerKey
 *
 ***/
	
void CopyFingerKey (Box rec, Box *copyRec)

{
	CopyBox(rec, copyRec);
}



/***
 * CopySignKeyVar
 *
 ***/
	
void CopySignKeyVar (Box rec, Box *copyRec)

{
	CopyBox(rec, copyRec);
}




/*****************************  Clearification  *****************************/


/***
 * ClearBox
 *
 ***/
 
void ClearBox (Box box)

{
	if (box == NULL) return;	
	
	box->descender = 0;
	box->selType = none;
	box->selSym = -1;
	box->numSyms = 0;
}



/***
 * ClearBoxRow
 *
 ***/
 
void ClearBoxRow (BoxRow row)

{
	short  box;
	
	
	if (row == NULL) return;	
	
	for (box = 0; box < KEY_VARS; box++)
		ClearBox((*row)[box]);
		
}



/***
 * ClearSignKey
 *
 ***/
 	
void ClearSignKey (BoxRow rec)

{
	ClearBoxRow(rec);
}



/***
 * ClearFingerKey
 *
 ***/
	
void ClearFingerKey (Box rec)

{
	ClearBox(rec);
}



/***
 * ClearSignKeyVar
 *
 ***/
	
void ClearSignKeyVar (Box rec)

{
	ClearBox(rec);
}




/********************************  Access  *********************************/


/***
 * GetSignKey
 *
 ***/
 
void GetSignKey (short keyCode, BoxRow *rec)

{
	*rec = signKey[keyCode];
}


	  
/***
 * GetFingerKey
 *
 ***/
 
void GetFingerKey (short keyCode, Box *rec)

{
	*rec = fingerKey[keyCode];
}

	
				  
/***
 * GetSignKeyVar
 *
 ***/
 
void GetSignKeyVar (short keyCode, short keyVar, Box *rec)			  

{
	*rec = (*signKey[keyCode])[keyVar];
}



/***
 * SetSignKey
 *
 ***/
 
void SetSignKey (short keyCode, BoxRow rec)		

{
	signKey[keyCode] = rec;
}



/***
 * SetFingerKey
 *
 ***/
 
void SetFingerKey (short keyCode, Box rec)

{
	fingerKey[keyCode] = rec;
}



/***
 * SetSignKeyVar
 *
 ***/
 
void SetSignKeyVar (short keyCode, short keyVar, Box rec)		

{
	(*signKey[keyCode])[keyVar] = rec;
}




/****************************  Selection  *******************************/


/***
 * GetSelectType
 *
 ***/

void GetSelectType (Box rec, ObjType *selType)

{
	*selType = rec->selType;
}



/***
 * GetSelectSym
 *
 ***/
 
void GetSelectSym (Box rec, short *symInx)

{
	*symInx = rec->selSym;
}



/***
 * SetSelectType
 *
 ***/
 
void SetSelectType (Box rec, ObjType selType)

{
	rec->selType = selType;
}



/***
 * SetSelectSym
 *
 ***/
 
void SetSelectSym (Box rec, short symInx)

{
	rec->selSym = symInx;
}




/****************************  Descender  *******************************/


/***
 * GetDescender
 *
 ***/

void GetDescender (Box rec, short *descendVal)

{
	*descendVal = rec->descender;
}



/***
 * SetDescender
 *
 ***/
 
void SetDescender (Box rec, short descendVal)

{
	rec->descender = descendVal;
}
  
  
  
  
/****************************  Symbol count  *****************************/
	  	  	

/***
 * GetSymCount
 *
 ***/

void GetSymCount (Box rec, short *symCount)

{
	*symCount = rec->numSyms;
}



/***
 * SetSymCount
 *
 ***/
 
void SetSymCount (Box rec, short symCount)

{
	rec->numSyms = symCount;
}



    
/******************************  Editing  *******************************/


/***
 * GetSymBase
 *
 ***/
 
void GetSymBase (Box rec, short symInx, Byte **symBase)

{
	*symBase = (Byte *) &rec->sym[symInx];
}


			  				  
/***
 * CopySym
 *
 ***/
 
void CopySym (Box src, short srcInx, Box dest)

{
	BlockMove(src->sym[srcInx], dest->sym[dest->numSyms++], sizeof(SymCode));
}



/***
 * DeleteSym
 *
 ***/
 
void DeleteSym (Box rec, short symInx)

{
	BlockMove(rec->sym[--rec->numSyms], rec->sym[symInx], sizeof(SymCode));
}




/*******************************  Drawing  ******************************/


/***
 * DrawBox
 *
 ***/
 
void DrawBox (Box box, BitMap *img, Rect *keyBox, Boolean thumbNail)

{
	short   code, var, fill, rot, flop, junk, symX, symY;	
	short   symInx;
	LongPt  symPos, keyPos;
	Byte    *symBase;
	
	
	for (symInx = 0; symInx < box->numSyms; symInx++) {
	
		if (box->selType != sym || box->selSym != symInx) {
		
			GetSymBase(box, symInx, &symBase);
			GetSymInfo(symBase, &code, &var, &fill, &rot, &flop, &junk, &symX, &symY);
		
			if (thumbNail) {	
				symX /= THUMB_SCALE;
				symY /= THUMB_SCALE;			
			}		
			
			SetLongPt(&symPos, symX, symY);
			SetLongPt(&keyPos, keyBox->left, keyBox->top);
			AddLongPt(&keyPos, &symPos);							
			
			DrawSymbol(code, var, fill, rot, flop, img, &symPos, thumbNail, FALSE);
			
		}
	}
	
}


			      
/***
 * DrawSignKey
 *
 ***/
 			 
void DrawSignKey (Byte keyCode, Byte keyVar, BitMap *img, Rect *keyBox, 
				  Boolean thumbNail)

{
	Box  box;
		
	
	GetSignKeyVar(keyCode, keyVar, &box);		  
	DrawBox(box, img, keyBox, thumbNail);
	
}

			   
				   
/***
 * DrawFingerKey
 *
 ***/
 				   
void DrawFingerKey (Byte keyCode, BitMap *img, Rect *keyBox,
				    Boolean thumbNail)
				     
{
	Box  box;
		
	
	GetFingerKey(keyCode, &box);		  
	DrawBox(box, img, keyBox, thumbNail);
	
}




/****************************  Selection drawing  ***************************/


/***
 * DrawBoxSelect
 *
 ***/
 		   
void DrawBoxSelect (Box box, BitMap *img, Rect *keyBox)				 

{
	short  code, var, fill, rot, flop, junk, symX, symY;
	
	LongPt  symPos, keyPos;
	Byte    *symBase;
	

	GetSymBase(box, box->selSym, &symBase);
	GetSymInfo(symBase, &code, &var, &fill, &rot, &flop, &junk, &symX, &symY);
	
	SetLongPt(&symPos, symX, symY);
	SetLongPt(&keyPos, keyBox->left, keyBox->top);
	AddLongPt(&keyPos, &symPos);							
	
	DrawSymbol(code, var, fill, rot, flop, img, &symPos, FALSE, TRUE);
	
}


				     				   
/***
 * DrawSignSelect
 *
 ***/	
 			   
void DrawSignSelect (Byte keyCode, Byte keyVar, BitMap *img, Rect *keyBox)	

{
	Box  box;
		
	
	GetSignKeyVar(keyCode, keyVar, &box);		  
	DrawBoxSelect(box, img, keyBox);				 
	
}



/***
 * DrawFingerSelect
 *
 ***/
 
void DrawFingerSelect (Byte keyCode, BitMap *img, Rect *keyBox)

{
	Box  box;
		
	
	GetFingerKey(keyCode, &box);		  
	DrawBoxSelect(box, img, keyBox);				 
	
}




/***************************  Selection erasing  ***************************/

			 
/***
 * EraseSignSelect
 *
 ***/
 
void EraseSignSelect (BitMap *img)				 
	
{
	UndrawSymbol(img);
}
	
	
	
/***
 * EraseFingerSelect
 *
 ***/	
 
void EraseFingerSelect (BitMap *img)	  
	
{
	UndrawSymbol(img);
}		   		
		   		
		   		   			


/**************************   Packing support   **************************/



static	unsigned short  bufIndex;				// key buffer index

static	Handle			keyBuffer;				// key buffer handle



/***
 * GetByte
 *
 ***/
 
void  GetByte (short *value)

{
	Byte  b;
	
	BlockMove(*keyBuffer+bufIndex, &b, 1);
	bufIndex++;
	
	*value = b;
}



/***
 * GetBytes
 *
 ***/
 
void  GetBytes (void *base, short bytes)

{
	BlockMove(*keyBuffer+bufIndex, base, bytes);
	bufIndex += bytes;
}



/***
 * PutByte
 *
 ***/
 
void  PutByte (short value)

{
	Byte  b = value;
	
	BlockMove(&b,*keyBuffer+bufIndex, 1);
	bufIndex++;
}



/***
 * PutBytes
 *
 ***/
 
void  PutBytes (void *base, short bytes)

{
	BlockMove(base, *keyBuffer+bufIndex, bytes);
	bufIndex += bytes;
}




/*****************************   Key packing   *****************************/


/***
 * PackBox
 *
 ***/
 
void PackBox (Box box)

{	
	short	inx;
	
	
	PutByte(box->descender);
	PutByte(box->numSyms);

	for (inx = 0; inx < box->numSyms; inx++)
		PutBytes(&box->sym[inx], SYM_SIZE);
	
}


/***
 * PackSignKey
 *
 ***/
 
void PackSignKey (short keyCode, Handle keyBuf, short *keyBytes)

{
	short  keyVar, symCount;
	Box	   box;
	

	keyBuffer = keyBuf;
	bufIndex  = 0;
	
	symCount = 0;

	for (keyVar = 0; keyVar < KEY_VARS; keyVar++) {
		
		GetSignKeyVar(keyCode, keyVar, &box);
		PackBox(box);

		symCount += box->numSyms;		
		
	}
		
	if (symCount == 0) {		// empty key
	
		*keyBytes = 0;
		return;
	}
	
	*keyBytes = bufIndex;
	
}



/***
 * PackFingerKey
 *
 ***/
 
void PackFingerKey (short keyCode, Handle keyBuf, short *keyBytes)

{
	Box  box;
	
	
	keyBuffer = keyBuf;
	bufIndex  = 0;
	
	GetFingerKey(keyCode, &box);
	PackBox(box);
	
	if (box->numSyms == 0) {		// empty key
		*keyBytes = 0;
		return;
	}
		
	*keyBytes = bufIndex;
	
}




/****************************   Key unpacking   ******************************/


/***
 * UnpackBox
 *
 ***/
 
void UnpackBox (Box box)

{	
	short	inx;

	
	GetByte(&box->descender);
	GetByte(&box->numSyms);

	for (inx = 0; inx < box->numSyms; inx++)
		GetBytes(&box->sym[inx], SYM_SIZE);
	
}



/***
 * UnpackSignKey
 *
 ***/
 
void UnpackSignKey (short keyCode, Handle keyBuf, short keyBytes)

{
	short  keyVar;
	Box	   box;


	if (keyBytes == 0)  		// empty key
		return;
	
	
	keyBuffer = keyBuf;
	bufIndex  = 0;
	
		
	for (keyVar = 0; keyVar < KEY_VARS; keyVar++) {
		
		GetSignKeyVar(keyCode, keyVar, &box);
		UnpackBox(box);	
		
	}
	
}



/***
 * UnpackFingerKey
 *
 ***/
 
void UnpackFingerKey (short keyCode, Handle keyBuf, short keyBytes)

{
	Box  box;
	
	
	if (keyBytes == 0)  		// empty key
		return;
		
	keyBuffer = keyBuf;
	bufIndex  = 0;
	
	GetFingerKey(keyCode, &box);
	UnpackBox(box);
	
}

