/******************************************************************************
 SymCode.c

		Symbol code manager for the SignWriter key editor
		
		Copyright  1996 Center for Sutton Movement Writing. All rights reserved.

 ******************************************************************************/
 



#include "SymCode.h"




/*************************  SymCode data structure  ************************/

		
			/*
			
			Byte 0:
			
				Bit    7:  always 0
				Bits 0-6:  hi-order 7 bits of 10-bit symbol code (1..999)
				
			Byte 1:
			
				Bits 5-7:  lo-order 3 bits of 10-bit symbol code (1..999)
				Bits 2-4:  3-bit symbol variation (0..5)
				Bit    1:  symbol flop value (0..1)
				Bit    0:  symbol mark (used during searches) (0..1)

			Byte 2:
			
				Bits 0-7:  hi-order 8 bits of 10-bit X position (0..1023)
				
			Byte 3:
			
				Bits 6-7:  lo-order 2 bits of 10-bit X position (0..1023)
				Bits 0-5:  hi-order 6 bits of 7-bit Y position (0..59)
				
			Byte 4:
			
				Bit    7:  always 0
				Bit    6:  lo-order bit of 7-bit X position (0..59)
				Bits 3-5:  3-bit symbol rotation (0..7)
				Bits 0-2:  3-bit symbol fill (0..5)	 			

			*/
			
			
			
			
/*************************  SymCode initialization  ************************/


/***
 * NewSymCode
 *
 ***/	
 
void  NewSymCode (Byte *base)

{
	base[0] = 0;
	base[1] = 0;
	base[2] = 0;
	base[3] = 0;
	base[4] = 0;
}


			

/*************************  SymCode access routines  ************************/


/***
 * GetSymNum
 *
 ***/	
 			   
void  GetSymNum (Byte *base, short *numVal)

{	
	*numVal = (base[0] << 3) | (base[1] >> 5);
}



/***
 * SetSymNum
 *
 ***/	
 			   
void  SetSymNum (Byte *base, short numVal)

{	
	base[0] = numVal >> 3;
	base[1] = (numVal << 5) | (base[1] & 0x1F);
}
  
	   				  

/***
 * GetSymVar
 *
 ***/	
 			   
void  GetSymVar (Byte *base, short *varVal)

{	
	*varVal = (base[1] >> 2) & 0x07;
}



/***
 * SetSymVar
 *
 ***/	
 			   
void  SetSymVar (Byte *base, short varVal)

{	
	base[1] = (varVal << 2) | (base[1] & 0xE3);
}



/***
 * GetSymFill
 *
 ***/	
 			   
void  GetSymFill (Byte *base, short *fillVal)

{	
	*fillVal = base[4] & 0x07;
}



/***
 * SetSymFill
 *
 ***/	
 			   
void  SetSymFill (Byte *base, short fillVal)

{	
	base[4] = fillVal | (base[4] & 0xF8);
}    
  	
  	
  			
/***
 * GetSymRot
 *
 ***/	
 			   
void  GetSymRot (Byte *base, short *rotVal)

{	
	*rotVal = (base[4] >> 3) & 0x07;
}



/***
 * SetSymRot
 *
 ***/	
 			   
void  SetSymRot (Byte *base, short rotVal)

{	
	base[4] = (rotVal << 3) | (base[4] & 0xC7);
}		

 	
  			
/***
 * GetSymFlop
 *
 ***/	
 			   
void  GetSymFlop (Byte *base, short *flopVal)

{	
	*flopVal = (base[1] >> 1) & 0x01;
}



/***
 * SetSymFlop
 *
 ***/	
 			   
void  SetSymFlop (Byte *base, short flopVal)

{	
	base[1] = (flopVal << 1) | (base[1] & 0xFD);
}		



/***
 * GetSymMark
 *
 ***/	
 			   
void  GetSymMark (Byte *base, short *markVal)

{	
	*markVal = base[1] & 0x01;
}



/***
 * SetSymMark
 *
 ***/	
 			   
void  SetSymMark (Byte *base, short markVal)

{	
	base[1] = markVal | (base[1] & 0xFE);
}    
   				  


/***
 * GetSymXPos
 *
 ***/	
 			   
void  GetSymXPos (Byte *base, short *xPosVal)

{	
	*xPosVal = (base[2] << 2) | (base[3] >> 6);
}



/***
 * SetSymXPos
 *
 ***/	
 			   
void  SetSymXPos (Byte *base, short xPosVal)

{	
	base[2] = xPosVal >> 2;
	base[3] = (xPosVal << 6) | (base[3] & 0x3F);
}
  


/***
 * GetSymYPos
 *
 ***/	
 			   
void  GetSymYPos (Byte *base, short *yPosVal)

{	
	*yPosVal = ((base[3] << 1) & 0x7F) | (base[4] >> 6);
}



/***
 * SetSymYPos
 *
 ***/	
 			   
void  SetSymYPos (Byte *base, short yPosVal)

{	
	base[3] = (yPosVal >> 1) | (base[3] & 0xC0);
	base[4] = ((yPosVal << 6) & 0x40) | (base[4] & 0xBF);
}    
	
	

/***
 * GetSymInfo
 *
 ***/	
 			   
void  GetSymInfo (Byte *base, short *numVal ,
							  short *varVal ,
							  short *fillVal,
							  short *rotVal ,
							  short *flopVal,
							  short *markVal,
							  short *xPosVal,
							  short *yPosVal)
							  
{	
	*numVal  = (base[0] << 3) | (base[1] >> 5);
	*varVal  = (base[1] >> 2) & 0x07;
	*fillVal =  base[4] & 0x07;
	*rotVal  = (base[4] >> 3) & 0x07;
	*flopVal = (base[1] >> 1) & 0x01;
	*markVal =  base[1] & 0x01;
	*xPosVal = (base[2] << 2) | (base[3] >> 6);
	*yPosVal = ((base[3] << 1) & 0x7F) | (base[4] >> 6);
}
				 
	

/***
 * SetSymInfo
 *
 ***/	
 			   
void  SetSymInfo (Byte *base, short numVal ,
							  short varVal ,
							  short fillVal,
							  short rotVal ,
							  short flopVal,
							  short markVal,
							  short xPosVal,
							  short yPosVal)
							  
{	
	base[0] = numVal >> 3;
	base[1] = (numVal << 5) | (base[1] & 0x1F);
	base[1] = (varVal << 2) | (base[1] & 0xE3);
	base[4] = fillVal | (base[4] & 0xF8);
	base[4] = (rotVal << 3) | (base[4] & 0xC7);
	base[1] = (flopVal << 1) | (base[1] & 0xFD);
	base[1] = markVal | (base[1] & 0xFE);
	base[2] = xPosVal >> 2;
	base[3] = (xPosVal << 6) | (base[3] & 0x3F);
	base[3] = (yPosVal >> 1) | (base[3] & 0xC0);
	base[4] = ((yPosVal << 6) & 0x40) | (base[4] & 0xBF);
}




/*************************  CtrlCode data structure  ************************/

		
			/*

			Byte 0:
			
				Bit    7:  always 1
				Bits 5-6:  2-bit control code (0 = NewSgn, 1 = NewLn, 2 = EOF)
				Bits 0-4:  hi-order 5 bits of 10-bit sign length value (0..1023)

			Byte 1:
			
				Bit    7:  always 1
				Bits 5-6:  unused
				Bits 0-4:  lo-order 5 bits of 10-bit sign length value (0..1023)
			
			*/
			


			
/************************  CtrlCode initialization  ************************/


/***
 * NewCtrlCode
 *
 ***/	
 
void  NewCtrlCode (Byte *base)

{
	base[0] = 0x80;
	base[1] = 0x80;
}


			

/************************  CtrlCode access routines  ************************/
  				  

/***
 * GetCtrlCode
 *
 ***/	
 			   
void  GetCtrlCode (Byte *base, short *codeVal)

{	
	*codeVal = (base[0] >> 5) & 0x03;
}



/***
 * SetCtrlCode
 *
 ***/	
 			   
void  SetCtrlCode (Byte *base, short codeVal)

{	
	base[0] = (codeVal << 5) | (base[0] & 0x9F);
}
  
			  

/***
 * GetCtrlLen
 *
 ***/	
 			   
void  GetCtrlLen (Byte *base, short *lengthVal)

{	
	*lengthVal = ((base[0] & 0x1F) << 5) | (base[1] & 0x1F);
}



/***
 * SetCtrlLen
 *
 ***/	
 			   
void  SetCtrlLen (Byte *base, short lengthVal)

{	
	base[0] = (lengthVal >> 5) | (base[0] & 0xE0);
	base[1] = (lengthVal & 0x1F) | (base[1] & 0xE0);
}
  
	   			   				  
		

/*************************  TxtBlk data structure  ************************/

		
			/*

			Byte 0:
			
				Bits 0-7:  always 255

			Byte 1:
			
				Bits 0-7:  hi-order 8 bits of 10-bit X position (0..1023)
				
			Byte 2:
			
				Bits 6-7:  lo-order 2 bits of 10-bit X position (0..1023)
				Bits 0-5:  hi-order 6 bits of  7-bit Y position (0..59)
			
			Byte 3:
				
				Bit    7:  lo-order bit of 7-bit Y position (0..59)
				Bits 0-6:  unused
			
			
			(NOTE: Text block codes are followed by a string of ASCII codes
			       delimited by byte values of 255.)
			 
			*/



			
/************************  TxtBlk initialization  ************************/


/***
 * NewTxtBlk
 *
 ***/	
 
void  NewTxtBlk (Byte *base)

{
	base[0] = 255;			// text block code
	base[1] = 0;
	base[2] = 0;
	base[3] = 0;

	base[4] = 255;			// text delimiters (empty text string)
	base[5] = 255;
}


			

/************************  TxtBlk access routines  ***********************/
  				  

/***
 * GetTxtXPos
 *
 ***/	
 			   
void  GetTxtXPos (Byte *base, short *posVal)

{	
	*posVal = (base[1] << 2) | (base[2] >> 6);
}



/***
 * SetTxtXPos
 *
 ***/	
 			   
void  SetTxtXPos (Byte *base, short posVal)

{	
	base[1] = posVal >> 2;
	base[2] = (posVal << 6) | (base[2] & 0x3F);
}
  


/***
 * GetTxtYPos
 *
 ***/	
 			   
void  GetTxtYPos (Byte *base, short *posVal)

{	
	*posVal = ((base[2] & 0x3F) << 1) | (base[3] >> 7);
}



/***
 * SetTxtYPos
 *
 ***/	
 			   
void  SetTxtYPos (Byte *base, short posVal)

{	
	base[2] = (posVal >> 1) | (base[2] & 0xC0);
	base[3] = (posVal << 7) | (base[3] & 0x7F);
}    


