/****
 * CFileWndGrid.c
 *
 *	File window methods for the SignWriter symbol editor.
 *
 *  Copyright 1995 The Center for Sutton Movement Writing. All rights reserved.
 *
 ****/

 
/**
 *
 *  This class subclasses TCL's CGridSelector class to implement
 *  the symbol editor's file window.  It uses a hacked version of
 *  CScrollPane to display a "current symcode" box next to the
 *  scroll bar.  It also overrides CGridSelector's DrawGrid
 *  method to get what we need.
 *
 **/



#include "CFileWndGrid.h"
#include "CFileWndTask.h"
#include "CApplication.h"
#include "CBureaucrat.h"
#include "CSelector.h"
#include "CSymWindow.h"
#include "CWindow.h"
#include "Global.h"
#include "SymEditConsts.h"
#include "Symbols.h"
#include "TBUtilities.h"
#include <BDC.h>




/*** Global Variables ***/

extern	void   *gClipSym;			/* Clipboard */




/****************************  Initialization  ****************************/


void CFileWndGrid::IFileWndGrid (CView *anEnclosure, CBureaucrat *aSupervisor,
							     short aWidth, short aHeight,
								 short aHEncl, short aVEncl,
								 SizingOption aHSizing, SizingOption aVSizing)
{
	short	sym;
	
	itsSymWindows = 0;
	
	for (sym = 0; sym < FGRID_SYMS; sym++)
		itsSymWindow[sym] = NULL;	
			
	inherited::IGridSelector(anEnclosure, aSupervisor,
				aHEncl, aVEncl, aHSizing, aVSizing,
				SELnone, 0, FGRID_ROWS, FGRID_COLS,
				FCELL_WIDTH, FCELL_HEIGHT);
}




/****************************  Window management  ****************************/

 
/***
 * GetSymWindow
 *
 *	Return the window corresponding
 *  to the specified symbol code.
 *
 ***/
 
void CFileWndGrid::GetSymWindow (short symCode, void **wind)
{
	*wind = itsSymWindow[symCode];
}



/***
 * SetSymWindow
 *
 *	Assign a window to the 
 *  specified symbol code.
 *
 ***/
 
void CFileWndGrid::SetSymWindow (short symCode, void *wind)
{
	itsSymWindow[symCode] = wind;

	if (wind != NULL)
		itsSymWindows++;
	else 
		itsSymWindows--;
}



/***
 * SymWindowsOpen
 *
 *	Check for open sym windows.
 *
 ***/
 
Boolean CFileWndGrid::SymWindowsOpen()

{
	short junk;
	
	if (itsSymWindows > 0) {

		PositionDialog('ALRT', ALRTopenSyms);	
		junk = Alert(ALRTopenSyms, NULL);	
		
		return TRUE;
	}
	
	return FALSE;	
}




 /******************************  Grid mapping  *******************************/

 
/***
 * ValidItem
 *
 *  Return TRUE if the specified item
 *  is a defined symbol cell.
 *
 ***/
 
Boolean CFileWndGrid::ValidItem (short theItem)

{		
 	return theItem > FGRID_COLS;			/* ignore label cells */	
}



/***
 * ItemToSymCode
 *
 *	Convert a file grid item number to the equivalent
 *  SignWriter symbol code value.  Translate the grid
 *  class row-major form to the symbol code's column-
 *  major form.
 *
 ***/
 
 short CFileWndGrid::ItemToSymCode (short item)
 {
 	short	row, col;
 	
 	item--;
 	row = (item / FGRID_COLS) - 1;  /* label row adjust */
 	col = item % FGRID_COLS;
 	return (col * (FGRID_ROWS - 1)) + row;
 }
 
 
 
/***
 * SymCodeToItem
 *
 *	Convert a SignWriter symbol code value to 
 *  its equivalent item number in the file grid. 
 *
 ***/
 
 short CFileWndGrid::SymCodeToItem (short symCode)
 {
 	short	row, col;
 	
 	row = symCode % (FGRID_ROWS - 1);
 	col = symCode / (FGRID_ROWS - 1);
 	return ((row + 1) * FGRID_COLS) + col + 1;
 }



 	
 /********************************  Drawing  ********************************/
	

/***
 * DrawGrid (override default method)
 *
 *  Draw grid lines around the items.  Leave top row ungridded;
 *  instead, draw grid label values across it.  
 *
 ***/

void CFileWndGrid::DrawGrid (void)

{
	long 	hExtent, vExtent;
	short	coord, length, labelVal;
	Str15	str;
	
	GetPixelExtent( &hExtent, &vExtent);

	/* horizontal grid lines */
	length = hExtent - 1;
	for (coord = boxHeight - 1; coord < vExtent; coord += boxHeight) {
		MoveTo(0, coord);
		LineTo(length, coord);
	}
	
	/* vertical grid lines */
	length = vExtent - 1;
	for (coord = boxWidth - 1; coord < hExtent; coord += boxWidth) {
		MoveTo(coord, boxHeight - 1);  /* skip top row */
		LineTo(coord, length);
	}
	
	/* grid labels */
	TextFont(1); TextFace(0); TextSize(10);
	labelVal = 0;
	for (coord = 0; coord < hExtent; coord += boxWidth*2) {
		NumToString(labelVal,str);
		MoveTo(coord + ((boxWidth - StringWidth(str)) / 2), boxHeight - 4);
		DrawString(str);
		labelVal += 20;
	}
}



/***
 * DrawItem
 *
 *  Draw the specified grid item.  The item numbering
 *  scheme is as defined for CGridSelector.
 *
 ***/
 
void CFileWndGrid::DrawItem (short theItem, Rect *theBox)
 
{
	short	sym;
	LongPt	symPos, imgFrame, junk;

	
	if (!ValidItem(theItem))
		return;
	
	sym = ItemToSymCode(theItem);
	 
 	if (sym == 0) {								// sym 0 undefined
 		FillRect(theBox, ltGray);
 		return;
 	}
 	
	FillRect(theBox, white);

 	if (!SymEmpty(sym)) {						// draw thumbnail version of
 												// symbol's canonical image 

 		GetCoords(sym, 1, 1, 0, 0, &imgFrame, &junk, TRUE); 	
 		
	 	symPos.h = theBox->left + ((FCELL_WIDTH - imgFrame.h)  / 2);
	 	symPos.v = theBox->top + ((FCELL_HEIGHT - imgFrame.v) / 2);
	 	
 		DrawSymbol(sym, 1, 1, 0, 0, &thePort->portBits, &symPos, TRUE);	
 			 
	}
}



/***
 * UpdateItem
 *
 *  Update the specified grid
 *  item after image editing.
 *
 ***/
 
void CFileWndGrid::UpdateItem (short symCode)
 
{
	short	item = SymCodeToItem(symCode);
	Rect	box;

	Prepare();
	FindItemBox(item, &box);
	
	FillRect(&box, white);
	DrawItem(item, &box);	
	
	if (item == CSelector::GetSelection())
		HiliteItem (item, hiliteON);

}



/***
 * Update
 *
 *  Update the file window grid.
 *
 ***/
 
void CFileWndGrid::Update()

{
	LongRect	theFrame;
	Rect		theRect;
	
	Prepare();
	
	GetFrame(&theFrame);
	FrameToQDR(&theFrame, &theRect);
	
	inherited::Draw(&theRect);
}




/********************************  Commands  *******************************/


/***
/* 
/*  DoCopy
/*
/*  Copy selected symbol to the clipboard. Not undoable.
/*
 ***/
		
void CFileWndGrid::DoCopy()

{
	short	 symCode = ItemToSymCode(CSelector::GetSelection());
	void	*rec;
		
	if (gClipSym != NULL)
		DisposSymbolRec(gClipSym);
	
	GetSymbolRec(symCode, &rec);
	CopySymbolRec(rec, &gClipSym);

}
	


/***
/* 
/*  DoClear
/*
/*  Erase the currently selected symbol
/*
 ***/

void CFileWndGrid::DoClear (short taskIndex)

{
	short			select = CSelector::GetSelection();
	CFileWndClear  *theClearTask;
	
	if (SymWindowsOpen())
		return;
											
	theClearTask = new CFileWndClear;
	theClearTask->IFileWndClear(taskIndex, this, select);
	theClearTask->Do();
	
	itsSupervisor->Notify(theClearTask);
	itsLastTask = theClearTask;
}



/***
/* 
/*  DoPaste
/*
/*  Paste symbol from the clipboard.
/*
 ***/

void CFileWndGrid::DoPaste()

{
	short			select = CSelector::GetSelection();
	CFileWndPaste  *thePasteTask;
	
	if (SymWindowsOpen())
		return;									
	
	thePasteTask = new CFileWndPaste;
	thePasteTask->IFileWndPaste(this, select);
	thePasteTask->Do();
	
	itsSupervisor->Notify(thePasteTask);
	itsLastTask = thePasteTask;
}	
	


/***
/* 
/*  DoInsertCell
/*
/*  Insert empty symbol into symtable.
/*
 ***/

void CFileWndGrid::DoInsertCell()

{
	short			select = CSelector::GetSelection();
	CFileWndInsert  *theInsertTask;
		
	if (SymWindowsOpen())
		return;	
									
	theInsertTask = new CFileWndInsert;
	theInsertTask->IFileWndInsert(this, select);
	theInsertTask->Do();
	
	itsSupervisor->Notify(theInsertTask);
	itsLastTask = theInsertTask;
}



/***
/* 
/*  DoDeleteCell
/*
/*  Delete selected symbol from symtable.
/*
 ***/

void CFileWndGrid::DoDeleteCell()

{
	short			select = CSelector::GetSelection();
	CFileWndDelete  *theDeleteTask;
	
	if (SymWindowsOpen())
		return;								
	
	theDeleteTask = new CFileWndDelete;
	theDeleteTask->IFileWndDelete(this, select);
	theDeleteTask->Do();
	
	itsSupervisor->Notify(theDeleteTask);
	itsLastTask = theDeleteTask;
}	




/**********************************  Mouse  ***********************************/


/***
 * DoClick (override default method)
 *
 *  Filter out clicks on label cells and symbol 0.
 *
 ***/
 
void CFileWndGrid::DoClick (Point hitPt, short modifierKeys, long when)

{
	short item = FindItem(hitPt);
	
	if (!ValidItem(item) || ItemToSymCode(item) == 0)
		return;
		
	inherited::DoClick(hitPt, modifierKeys, when);
}



/***
 * DoDoubleClick (override default)
 *
 * 	Open the selected symbol for editing.
 *
 ***/

void CFileWndGrid::DoDoubleClick (void)

{
	short 		item = CSelector::GetSelection();
	short		symCode;
	CSymWindow	*symWind;
	
	if (!ValidItem(item))
		return;
	
	symCode = ItemToSymCode(item);	
	if (symCode == 0)
		return;
	
	// window already open?
	GetSymWindow(symCode, (void *) &symWind);
	
	if (symWind != NULL)
	  symWind->itsWindow->Select();
	else		
	  NewSymWindow(symCode);
}	  




/******************************  Symbol editing  *******************************/


/***
 * NewSymWindow
 *
 *	Create new instance of a symbol window
 *	and send it a NewWindow message.
 *
 ***/

void CFileWndGrid::NewSymWindow (short symCode)

{
  	CSymWindow	*theWindow = NULL;

	TRY
	{
		theWindow = new CSymWindow;
		
		theWindow->ISymWindow((CDirectorOwner*) this->itsSupervisor);
	
		theWindow->NewWindow(symCode, this);
	}
	
	CATCH
	{
		 if (theWindow) theWindow->Dispose();
	}
	ENDTRY;
}


