
/******************************************************************************
 *
 *  CFileWndTask.c
 *
 *	Symbol window task methods for the SignWriter symbol editor.
 *
 *  Copyright 1995 The Center for Sutton Movement Writing. All rights reserved.
 *
 ******************************************************************************/
 


#include "CFileWndTask.h"
#include "CFileWndGrid.h"
#include "Symbols.h"
#include "SymEditConsts.h"




/*** Global Variables ***/

extern void	  *gClipSym;		/* Clipboard */
extern void	  *gScratchSym;





/********************************  Clear  **********************************/


/***
 * IFileWndClear
 *
 ***/
 
void CFileWndClear::IFileWndClear (short nameIndex, CFileWndGrid *pane, short select)

{
	CTask::ITask(nameIndex);
	
	itsGridPane = pane;
	itsSelect   = select;
	itsSymCode  = itsGridPane->ItemToSymCode(itsSelect);
}	



/***
 * Do (override)
 *
 *	Move selected symbol to the global scratch 
 *  symbol. Clear the selected symbol and update it.
 *
 ***/

void CFileWndClear::Do()

{
	if (gScratchSym != NULL)
		DisposSymbolRec(gScratchSym);
	
	GetSymbolRec(itsSymCode, &gScratchSym);	
	SetSymbolRec(itsSymCode, NULL);
	
	itsGridPane->UpdateItem(itsSymCode);	
	itsGridPane->ChangeSelection(SELnone);			

}
	


/***
 * Undo (override)
 *
 *	Restore the cleared symbol
 *  from the global scratch symbol.
 *  Re-highlight the selection.
 *
 ***/

void CFileWndClear::Undo()

{
	SetSymbolRec(itsSymCode, gScratchSym);
	gScratchSym = NULL;
					
	itsGridPane->UpdateItem(itsSymCode);	
	itsGridPane->ChangeSelection(itsSelect);			

}
	


/***
 * Redo (override)
 *
 *	Remove selected symbol to the global
 *  scratch symbol. Reclear the selection.
 *
 ***/

void CFileWndClear::Redo()

{	
	GetSymbolRec(itsSymCode, &gScratchSym);	
	SetSymbolRec(itsSymCode, NULL);
	
	itsGridPane->UpdateItem(itsSymCode);
	itsGridPane->ChangeSelection(SELnone);			

}




/********************************  Paste  **********************************/


/***
 * IFileWndPaste
 *
 ***/
 
void CFileWndPaste::IFileWndPaste (CFileWndGrid *pane, short select)

{
	CTask::ITask(iPASTE);
	
	itsGridPane = pane;
	itsSelect   = select;
	itsSymCode  = itsGridPane->ItemToSymCode(itsSelect);
	
}	
	


/***
 * Do (override)
 *
 *	Move selected symbol to the global scratch symbol.
 *  Duplicate the clipboard symbol and update the 
 *  selection with the resulting copy.
 *
 ***/

void CFileWndPaste::Do()

{
	void  *rec;
	
	if (gScratchSym != NULL)
		DisposSymbolRec(gScratchSym);
	
	GetSymbolRec(itsSymCode, &gScratchSym);
	
	CopySymbolRec(gClipSym, &rec);
	SetSymbolRec(itsSymCode, rec);
					
	itsGridPane->UpdateItem(itsSymCode);		
	itsGridPane->ChangeSelection(SELnone);			

}
	


/***
 * Undo (override)
 *
 *	Exchange the pasted symbol
 *  with the global scratch symbol.
 *  Re-highlight the original selection.
 *
 ***/

void CFileWndPaste::Undo()

{
	void  *rec;
	
	GetSymbolRec(itsSymCode, &rec);
	SetSymbolRec(itsSymCode, gScratchSym);
	gScratchSym = rec;
					
	itsGridPane->UpdateItem(itsSymCode);	
	itsGridPane->ChangeSelection(itsSelect);			

}
	


/***
 * Redo (override)
 *
 *	Re-exchange the global scratch
 *  symbol with the pasted symbol.
 *  Reclear the selection.
 *
 ***/

void CFileWndPaste::Redo()

{
	void  *rec;
	
	GetSymbolRec(itsSymCode, &rec);
	SetSymbolRec(itsSymCode, gScratchSym);
	gScratchSym = rec;
					
	itsGridPane->UpdateItem(itsSymCode);	
	itsGridPane->ChangeSelection(SELnone);			

}




/********************************  Insert  **********************************/


/***
 * IFileWndInsert
 *
 ***/
 
void CFileWndInsert::IFileWndInsert (CFileWndGrid *pane, short select)

{
	CTask::ITask(iINSERT);
	
	itsGridPane = pane;
	itsSelect   = select;
	itsSymCode  = itsGridPane->ItemToSymCode(itsSelect);
	
}	



/***
 * Do (override)
 *
 *	Move selected symbol (and all subsequent symbols)
 *  up one cell in the symbol table.  Insert empty
 *  symbol at the selection. Re-highlight the selection.
 *
 ***/

void CFileWndInsert::Do()

{
	InsertSym(itsSymCode);	
	itsGridPane->Update();			
}
	


/***
 * Undo (override)
 *
 *	Delete the newly inserted symbol.
 *
 ***/

void CFileWndInsert::Undo()

{
	DeleteSym(itsSymCode);	
	itsGridPane->Update();	
}
	


/***
 * Redo (override)
 *
 *	Re-insert the symbol.
 *
 ***/

void CFileWndInsert::Redo()

{	
	InsertSym(itsSymCode);	
	itsGridPane->Update();	
}




/********************************  Delete  **********************************/


/***
 * IFileWndDelete
 *
 ***/
 
void CFileWndDelete::IFileWndDelete (CFileWndGrid *pane, short select)

{
	CTask::ITask(iDELETE);
	
	itsGridPane = pane;
	itsSelect   = select;
	itsSymCode  = itsGridPane->ItemToSymCode(itsSelect);
}	



/***
 * Do (override)
 *
 *	Move selected symbol to the global scratch symbol.
 *  Delete empty symbol at the selection.  Update the 
 *  sym table, leaving the original cell selected.
 *
 ***/

void CFileWndDelete::Do()

{
	if (gScratchSym != NULL)
		DisposSymbolRec(gScratchSym);
	
	GetSymbolRec(itsSymCode, &gScratchSym);		
	DeleteSym(itsSymCode);	
	
	itsGridPane->Update();	
}
	


/***
 * Undo (override)
 *
 *	Restore the deleted symbol from the global 
 *  scratch symbol. Re-highlight the selection.
 *
 ***/

void CFileWndDelete::Undo()

{
	InsertSym(itsSymCode);	
	SetSymbolRec(itsSymCode, gScratchSym);		
	gScratchSym = NULL;

	itsGridPane->Update();	
}
	


/***
 * Redo (override)
 *
 *	Re-delete selected symbol to the global
 *  scratch symbol. Re-highlight the selection.
 *
 ***/

void CFileWndDelete::Redo()

{	
	GetSymbolRec(itsSymCode, &gScratchSym);		
	DeleteSym(itsSymCode);	
	
	itsGridPane->Update();
}
