/******************************************************************************
 CImgWndToolDraw.c

							The ToolDraw Class
	
	This file implements the line (and circle) drawing tools 
	available in the symbol editor's image edit window.

	SUPERCLASS = CEdiTask
	
    Copyright  1995 The Center for Sutton Movement Writing.  All rights reserved.

 ******************************************************************************/


#include "CImgWndToolDraw.h"
#include "CImgWndPane.h"
#include <Global.h>
#include <TBUtilities.h>
#include "SymEditConsts.h"



extern CBitMap	*gScratchPad;



/******************************************************************************
 IToolDraw

		Initialize a ToolDraw object
 ******************************************************************************/

void	CToolDraw::IToolDraw(
	CImgWndPane	*aImgWndPane,
	CBitMap		*aImage)
{
	CImgWndTask::IImgWndTask(iDRAW, aImgWndPane, aImage);
}


/******************************************************************************
 BeginTracking {OVERRIDE}

		Mouse down tracking is about to begin at the specified starting
		LongPt. Set the pen size.
 ******************************************************************************/

void	CToolDraw::BeginTracking(
	LongPt		*startPt)
{
	gScratchPad->BeginDrawing();
	PenSize(PENCIL_WIDTH, PENCIL_WIDTH);
	gScratchPad->EndDrawing();
}


/******************************************************************************
 KeepTracking {OVERRIDE}

		Continuous mouse tracking while the mouse button is down. Drawing
		is performed in an offscreen scratch pad and then copied to the
		screen. The offscreen bitmap image does not change during this
		dynamic drawing.
 ******************************************************************************/

void	CToolDraw::KeepTracking(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt)
{
	LongPt	 	currMapPt;
	LongPt	 	prevMapPt;
	LongPt	 	startMapPt;
	
	LongRect	changedRect;
	LongRect	currRect;

	itsPane->PaneToBitMapPt(currPt,  &currMapPt);
	itsPane->PaneToBitMapPt(prevPt,  &prevMapPt);
	itsPane->PaneToBitMapPt(startPt, &startMapPt);
	
	TweakPoints(&currMapPt, &prevMapPt, &startMapPt);

	if (!EqualLongPt(&currMapPt, &prevMapPt)) {
	
										/* Find area of painting that has	*/
										/*   has changed					*/
		CalcChangedRect(&currMapPt, &prevMapPt, &startMapPt, &changedRect);
		
										/* Copy changed area to scratch pad	*/
		LCopyBits(itsBitMap->macBitMap, gScratchPad->macBitMap,
				 &changedRect, &changedRect, srcCopy, NULL);
				 
		gScratchPad->BeginDrawing();	/* Do drawing offscreen on the		*/
										/*   scratchpad						*/
		Pt2LongRect(&currMapPt, &startMapPt, &currRect);
		UseTool(&currRect, &currMapPt, &startMapPt);
		gScratchPad->EndDrawing();
		
										/* Update pane display				*/
		itsPane->Update(gScratchPad->macBitMap, &changedRect);
	}
}


/******************************************************************************
 EndTracking {OVERRIDE}

		Mouse down tracking has ended because the user has released the
		mouse button. Its now time to set up the undo buffer and change
		the actual offscreen painting image.
 ******************************************************************************/

void	CToolDraw::EndTracking(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt)
{
	LongPt	 	currMapPt;
	LongPt	 	prevMapPt;
	LongPt	 	startMapPt;
	
	LongRect	currRect;

	itsPane->PaneToBitMapPt(currPt,  &currMapPt);
	itsPane->PaneToBitMapPt(prevPt,  &prevMapPt);
	itsPane->PaneToBitMapPt(startPt, &startMapPt);
	
	TweakPoints(&currMapPt, &prevMapPt, &startMapPt);

									/* Track for the last time				*/
	KeepTracking(currPt, prevPt, startPt);

	CalcChangedRect(&currMapPt, &currMapPt, &startMapPt, &currRect);
	
	SaveForUndo(&currRect);
	
									/* Copy drawing area to scratch pad		*/
	LCopyBits(itsBitMap->macBitMap, gScratchPad->macBitMap,
			 &currRect, &currRect, srcCopy, NULL);
			 
	gScratchPad->BeginDrawing();	/* Draw new object on top of existing	*/
									/*   drawing on the scratchpad			*/
	UseTool(&currRect, &currMapPt, &startMapPt);
	gScratchPad->EndDrawing();

									/* Copy drawing area from scratchpad	*/
									/*   into the painting image			*/
	LCopyBits(gScratchPad->macBitMap, itsBitMap->macBitMap,
			 &currRect, &currRect, srcCopy, NULL);
}


/******************************************************************************
 TweakPoints
 ******************************************************************************/

void	CToolDraw::TweakPoints(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt)
{
}									/* Null Method							*/


/******************************************************************************
 CalcChangedRect

		Calculate bounding rectangle of area affected by drawing a shape
		at the specified locations
 ******************************************************************************/

void	CToolDraw::CalcChangedRect(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt,
	LongRect	*changedRect)
{
	LongRect	currRect;
	LongRect	prevRect;

	Pt2LongRect(startPt, currPt, &currRect);
	Pt2LongRect(startPt, prevPt, &prevRect);
	UnionLongRect(&currRect, &prevRect, changedRect);
}


/******************************************************************************
 UseTool
 ******************************************************************************/

void	CToolDraw::UseTool(
	LongRect	*currRect,
	LongPt		*currPt,
	LongPt		*startPt)
{
}									/* Null Method							*/



/******************************************************************************
 UseTool {Line}
 ******************************************************************************/

void	CToolLine::UseTool(
	LongRect	*currRect,
	LongPt		*currPt,
	LongPt		*startPt)
{
	LMoveTo(startPt->h, startPt->v);
	LLineTo(currPt->h, currPt->v);
}


/******************************************************************************
 CalcChangedRect {Line}

		Calculate bounding rectangle of area affected by drawing a line
		at the specified locations
 ******************************************************************************/

void	CToolLine::CalcChangedRect(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt,
	LongRect	*changedRect)
{
	LongPt		a, curr, prev;
	LongRect	currRect;
	LongRect	prevRect;
	
	curr = *currPt;
	prev = *prevPt;
	
	a = *startPt;						/* We have to adjust the points a	*/
	if (curr.h >= startPt->h) {			/*   bit to compensate for the pen,	*/
		curr.h += PENCIL_WIDTH;			/*   which hangs below and to the	*/
	} else {							/*   right its QuickDraw coordinate	*/
		a.h += PENCIL_WIDTH;
	}
	
	if (curr.v >= startPt->v) {
		curr.v += PENCIL_WIDTH;
	} else {
		a.v += PENCIL_WIDTH;
	}
	
	Pt2LongRect(&a, &curr, &currRect);
	
	a = *startPt;
	if (prev.h >= startPt->h) {
		prev.h += PENCIL_WIDTH;
	} else {
		a.h += PENCIL_WIDTH;
	}
	
	if (prev.v >= startPt->v) {
		prev.v += PENCIL_WIDTH;
	} else {
		a.v += PENCIL_WIDTH;
	}
	
	Pt2LongRect(&a, &prev, &prevRect);
	
	UnionLongRect(&currRect, &prevRect, changedRect);
}


/******************************************************************************
 UseTool {Circle}
 ******************************************************************************/

void	CToolCircle::UseTool(
	LongRect		*currRect,
	LongPt		*currPt,
	LongPt		*startPt)
{
	LFrameOval(currRect);
}


/******************************************************************************
 CalcChangedRect {Circle}

		Calculate bounding rectangle of area affected by drawing a circle
		at the specified locations
 ******************************************************************************/

void	CToolCircle::CalcChangedRect(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt,
	LongRect	*changedRect)
{
	inherited::CalcChangedRect(currPt, prevPt, startPt, changedRect);
	
	if (prevPt->v > currPt->v) {  /* Adjust for prevPt's tweaked startPt */
		InsetLongRect(changedRect, -(prevPt->v - currPt->v + 1) / 2, 0);
	}
}


/******************************************************************************
 TweakPoints {Circle}

		Adjust locations of the construction points. Constrains
		currPt so that the rectangle formed by startPt & currPt is
		a square centered below the start point.
 ******************************************************************************/

void	CToolCircle::TweakPoints(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt)
{
	short 		length;

	length = currPt->v - startPt->v;
	if (length < 0) length = 0;
	
	startPt->h = startPt->h - (length / 2);
	currPt->h = startPt->h + length;
	
	prevPt->h = currPt->h;  // only vertical movement is relevant
}

