/******************************************************************************
 CImgWndToolEdit.c

							The ToolEdit Class
		
	SUPERCLASS = CImgWndTask
	
    Copyright  1995 The Center for Sutton Movement Writing.  All rights reserved.

 ******************************************************************************/


#include "CImgWndToolEdit.h"
#include <Global.h>
#include <TBUtilities.h>
#include "SymEditConsts.h"



extern RgnHandle	gUtilRgn;			/* Utility region					*/

extern CBitMap		*gScratchPad;
extern Pattern		gPattern;			/* The current fill pattern			*/
extern short		gLineSize;





/******************************************************************************
 BeginTracking {OVERRIDE}

		Mouse down tracking is about to begin at the specified starting
		point. Set the pen size and fill pattern.
 ******************************************************************************/

void CToolEdit::BeginTracking (LongPt *startPt)
{
	/* Copy entire bitmap to scratch pad	*/
	CopyBits(itsBitMap->macBitMap, gScratchPad->macBitMap,
			 &itsBitMap->macBitMap->bounds, &itsBitMap->macBitMap->bounds,
			 srcCopy, NULL);
	
	PenNormal();
}


/******************************************************************************
 KeepTracking {OVERRIDE}

		Continuous mouse tracking while the mouse button is down. Edit only
		when the mouse crosses a bit boundary.
 ******************************************************************************/

void	CToolEdit::KeepTracking(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt)
{
	LongPt	 currMapPt;
	LongPt	 prevMapPt;

	itsPane->PaneToBitMapPt(currPt, &currMapPt);
	itsPane->PaneToBitMapPt(prevPt, &prevMapPt);

	if (!EqualLongPt(&currMapPt, &prevMapPt)) {
		UseTool(&currMapPt, &prevMapPt);
	}
}



/******************************************************************************
 CalcPenRect

		Calculate the rectangle affected by a pen tool located at a
		specified point
 ******************************************************************************/

void CToolEdit::CalcPenRect (LongPt *thePt, LongRect *penRect)
{
	penRect->left = thePt->h;			/* Pen hangs below and to the right	*/
	penRect->top = thePt->v;			/*   of its coordinate location		*/
	penRect->right = thePt->h + penLength;
	penRect->bottom = thePt->v + penLength;
}


/******************************************************************************
 CalcDirtyRect

		Calculate the portion of the bitmap drawn on by a pen. Find the
		rectangle which encloses the current dirtyRect and the current
		pen rectangle.
 ******************************************************************************/

void CToolEdit::CalcDirtyRect (LongRect *penRect)
{
	dirtyRect.left = Min(dirtyRect.left, penRect->left);
	dirtyRect.top = Min(dirtyRect.top, penRect->top);
	dirtyRect.right = Max(dirtyRect.right, penRect->right);
	dirtyRect.bottom = Max(dirtyRect.bottom, penRect->bottom);
}


/******************************************************************************
 CalcDrawRgn

		Calculate the draw region affected by a pen. What a pain. We have
		to tweak the region depending on where the current point is in
		relation to the previous point because the pen hangs below and
		to the right of its location.
 ******************************************************************************/

void CToolEdit::CalcDrawRgn (LongRect *currRect, LongRect *prevRect, RgnHandle drawRgn)
{
	OpenRgn();
	
	if (currRect->left > prevRect->left) {
		if (currRect->top > prevRect->top) {
			LMoveTo(currRect->right, currRect->top);
			LLineTo(currRect->right, currRect->bottom);
			LLineTo(currRect->left, currRect->bottom);
			LLineTo(prevRect->left, prevRect->bottom);
			LLineTo(prevRect->left, prevRect->top);
			LLineTo(prevRect->right, prevRect->top);
			LLineTo(currRect->right, currRect->top);
		
		} else {
			LMoveTo(currRect->left, currRect->top);
			LLineTo(currRect->right, currRect->top);
			LLineTo(currRect->right, currRect->bottom);
			LLineTo(prevRect->right, prevRect->bottom);
			LLineTo(prevRect->left, prevRect->bottom);
			LLineTo(prevRect->left, prevRect->top);
			LLineTo(currRect->left, currRect->top);
		}
		
	} else {
		if (currRect->top > prevRect->top) {
			LMoveTo(currRect->right, currRect->bottom);
			LLineTo(currRect->left, currRect->bottom);
			LLineTo(currRect->left, currRect->top);
			LLineTo(prevRect->left, prevRect->top);
			LLineTo(prevRect->right, prevRect->top);
			LLineTo(prevRect->right, prevRect->bottom);
			LLineTo(currRect->right, currRect->bottom);
		
		} else {
			LMoveTo(currRect->left, currRect->bottom);
			LLineTo(currRect->left, currRect->top);
			LLineTo(currRect->right, currRect->top);
			LLineTo(prevRect->right, prevRect->top);
			LLineTo(prevRect->right, prevRect->bottom);
			LLineTo(prevRect->left, prevRect->bottom);
			LLineTo(currRect->left, currRect->bottom);
		}
	}
	
	CloseRgn(drawRgn);
}


/******************************************************************************
 EndTracking {OVERRIDE}

		Mouse down tracking has ended because the user has released the
		mouse button. Its now time to set up the undo buffer and change
		the actual offscreen painting bitmap.
 ******************************************************************************/

void	CToolEdit::EndTracking(
	LongPt *currPt,
	LongPt *prevPt, 
	LongPt *startPt)
{
									/* Track for the last time				*/
	KeepTracking(currPt, prevPt, startPt);
	
									/* Create an undo bitmap				*/
	itsUndoBits = new CBitMap;
	itsUndoBits->IBitMap(dirtyRect.right - dirtyRect.left,
						 dirtyRect.bottom - dirtyRect.top, FALSE);
	
	if (itsUndoBits->macBitMap == NULL) {
		itsUndoBits->Dispose();
		itsUndoBits = NULL;

	} else {
		itsUndoBits->SetBoundsOrigin(dirtyRect.left, dirtyRect.top);
		
									/* Original bitmap is on the scratchpad	*/
									/* Copy dirty area to undo bitmap		*/
		LCopyBits(gScratchPad->macBitMap, itsUndoBits->macBitMap,
					&dirtyRect, &dirtyRect, srcCopy, NULL);
	}
}


/******************************************************************************
 UseTool
 ******************************************************************************/

void	CToolEdit::UseTool(
	LongPt		*currPt,
	LongPt		*prevPt)
{
}									/* Null Method							*/


/**** Methods for Different Pen Tools ****/


/******************************************************************************
 IToolEraser {Eraser}
 ******************************************************************************/

void	CToolEraser::IToolEraser(
	CImgWndPane	*aPane,
	CBitMap		*aBitMap)
{
	CImgWndTask::IImgWndTask(iERASER, aPane, aBitMap);
	
	penLength = ERASER_WIDTH;
}


/******************************************************************************
 BeginTracking {Eraser}
 ******************************************************************************/

void	CToolEraser::BeginTracking (LongPt *startPt)
{
	LongRect rect;
	LongPt	 startMapPt;
	
	inherited::BeginTracking(startPt);
	
	itsPane->PaneToBitMapPt(startPt, &startMapPt);
	CalcPenRect(&startMapPt, &rect);
	dirtyRect = rect;
	
	itsBitMap->BeginDrawing();			/* Do erasure on offscreen bitmap	*/
	LEraseRect(&rect);
	itsBitMap->EndDrawing();	
										/* Copy erasure to pane				*/
	itsPane->Update(itsBitMap->macBitMap, &rect);
}


/******************************************************************************
 UseTool {Eraser}
 ******************************************************************************/

void	CToolEraser::UseTool(
	LongPt		*currPt,
	LongPt		*prevPt)
{
	LongRect	currRect;
	LongRect	prevRect;
	LongRect	dispRect;
	
	CalcPenRect(currPt, &currRect);
	CalcPenRect(prevPt, &prevRect);
	
	CalcDirtyRect(&currRect);			/* Determine changed area			*/
	CalcDrawRgn(&currRect, &prevRect, gUtilRgn);
	
	itsBitMap->BeginDrawing();			/* Do erasure on offscreen bitmap	*/
	EraseRgn(gUtilRgn);
	itsBitMap->EndDrawing();
	
	UnionLongRect(&prevRect, &currRect, &dispRect);  	/* Copy to pane		*/
	itsPane->Update(itsBitMap->macBitMap, &dispRect);
}


/******************************************************************************
 IToolPencil {Pencil}
 ******************************************************************************/

void	CToolPencil::IToolPencil(
	CImgWndPane	*aPane,
	CBitMap		*aBitMap)
{
	CImgWndTask::IImgWndTask(iPENCIL, aPane, aBitMap);
	
	penLength = PENCIL_WIDTH;
}


/******************************************************************************
 BeginTracking {Pencil}
 ******************************************************************************/

void	CToolPencil::BeginTracking(
	LongPt		*startPt)
{
	LongRect	rect;
	LongPt		startMapPt;
	Pattern		thePenPat;

	inherited::BeginTracking(startPt);
	
	itsPane->PaneToBitMapPt(startPt, &startMapPt);

	CalcPenRect(&startMapPt, &rect);
	dirtyRect = rect;
	
	PenNormal();						/* Start with a normal pen			*/
	if (itsBitMap->PixelIsBlack(&startMapPt)) {
										/* Pencil is on a black pixel		*/
										/*   Subsequent drawing will create	*/
										/*   white pixels					*/
		BlockMove(white, thePenPat, 8);
		
	} else {							/* Pencil is on a white pixel		*/
										/*   Subsequent drawing will create	*/
										/*   black pixels					*/
		BlockMove(black, thePenPat, 8);
	}
	
	itsBitMap->BeginDrawing();			/* Do drawing on offscreen bitmap	*/
	PenPat(thePenPat);
	LMoveTo(startMapPt.h, startMapPt.v);
	Line(0,0);
	itsBitMap->EndDrawing();
										/* Update pane display				*/
	itsPane->Update(itsBitMap->macBitMap, &rect);			
}


/******************************************************************************
 UseTool {Pencil}
 ******************************************************************************/

void	CToolPencil::UseTool(
	LongPt		*currPt,
	LongPt		*prevPt)
{
	LongRect	currRect;
	LongRect	prevRect;
	LongRect	dispRect;

	CalcPenRect(currPt, &currRect);		/* Determine changed area			*/
	CalcPenRect(prevPt, &prevRect);
	CalcDirtyRect(&currRect);
	
	itsBitMap->BeginDrawing();			/* Do drawing on offscreen bitmap	*/
	LMoveTo(prevPt->h, prevPt->v);
	LLineTo(currPt->h, currPt->v);
	itsBitMap->EndDrawing();

	UnionLongRect(&prevRect, &currRect, &dispRect);  	/* Copy to pane		*/
	itsPane->Update(itsBitMap->macBitMap, &dispRect);
}


/******************************************************************************
 IToolDrag {Drag}
 ******************************************************************************/

void	CToolDrag::IToolDrag(
	CImgWndPane	*aPane,
	CBitMap		*aBitMap)
{
	CImgWndTask::IImgWndTask(iDRAG, aPane, aBitMap);	
	saveCenter = itsPane->itsCenter;
}


/******************************************************************************
 BeginTracking {Drag}
 ******************************************************************************/

void	CToolDrag::BeginTracking(
	LongPt		*startPt)
{
	int		x, x1, x2;
	int		y, y1, y2;
	
	LongPt		pixPt;
	
	inherited::BeginTracking(startPt);	
	
	// image left bound
	for (x1 = 0; x1 < IMAP_WIDTH; x1++) {
		for (y = 0; y < IMAP_HEIGHT; y++) {
			SetLongPt(&pixPt, x1, y); 
			if (itsBitMap->PixelIsBlack(&pixPt))
				goto exit1;
		}
	} exit1:
	
	// image right bound
	for (x2 = IMAP_WIDTH - 1; x2 > x1; x2--) {
		for (y = 0; y < IMAP_HEIGHT; y++) {
			SetLongPt(&pixPt, x2, y); 
			if (itsBitMap->PixelIsBlack(&pixPt))
				goto exit2;
		}
	} exit2:
	
	// image top bound
	for (y1 = 0; y1 < IMAP_HEIGHT; y1++) {
		for (x = x1; x <= x2; x++) {
			SetLongPt(&pixPt, x, y1); 
			if (itsBitMap->PixelIsBlack(&pixPt))
				goto exit3;
		}
	} exit3:
	
	// image bottom bound
	for (y2 = IMAP_HEIGHT - 1; y2 > y1; y2--) {
		for (x = x1; x <= x2; x++) {
			SetLongPt(&pixPt, x, y2); 
			if (itsBitMap->PixelIsBlack(&pixPt))
				goto exit4;
		}	
	} exit4:
	
	// image bounds	
	SetLongRect(&dirtyRect, x1, y1, x2 + 1, y2 + 1);
}


/******************************************************************************
 UseTool {Drag}
 ******************************************************************************/

void	CToolDrag::UseTool(
	LongPt		*currPt,
	LongPt		*prevPt)
{
	int			dx, dy;
	LongRect	destRect;
	LongRect	rect;
	Rect		qdRect;
	RgnHandle	junk;
	LongPt		centerPt;
	

	// bail if empty bitmap
	if (EmptyLongRect(&dirtyRect))
		return;
	
	// movement offset
	dx = currPt->h - prevPt->h;
	dy = currPt->v - prevPt->v;
	
	// bitmap destination
	destRect = dirtyRect;
	OffsetLongRect(&destRect, dx, dy);
	
	// bail if invalid dest
	if (destRect.top  < 0 || destRect.bottom > IMAP_HEIGHT ||
		destRect.left < 0 || destRect.right  > IMAP_WIDTH)
		return;
		
	// scroll bitmap
	junk = NewRgn();
	itsBitMap->GetBounds(&rect);
	LongToQDRect(&rect, &qdRect);
	
	itsBitMap->BeginDrawing();
	ScrollRect(&qdRect, dx, dy, junk);
	itsBitMap->EndDrawing();
	
	DisposeRgn(junk);

	// update display
	UnionLongRect(&dirtyRect, &destRect, &rect);	
	itsPane->Update(itsBitMap->macBitMap, &rect);
	
	// update bitmap rect	
	dirtyRect = destRect;
	
	// update center point
	itsPane->PaneToBitMapPt(&itsPane->itsCenter, &centerPt);
	centerPt.h += dx;
	centerPt.v += dy;
	
	if (centerPt.h < 1) centerPt.h = 1;		
	if (centerPt.v < 1) centerPt.v = 1;
	if (centerPt.h >= IMAP_WIDTH) centerPt.h = IMAP_WIDTH - 1;		
	if (centerPt.v >= IMAP_HEIGHT) centerPt.v = IMAP_HEIGHT - 1;
	
	itsPane->BitMapToPanePt(&centerPt, &itsPane->itsCenter);
}


/******************************************************************************
 EndTracking {Drag}
 				
 				Set dirtyRect to full bitmap bounds for undo.
 ******************************************************************************/

void	CToolDrag::EndTracking(
	LongPt *currPt,
	LongPt *prevPt, 
	LongPt *startPt)
{
	itsBitMap->GetBounds(&dirtyRect);
	inherited::EndTracking(currPt, prevPt, startPt);
}


/******************************************************************************
 Undo {Drag} 	
 ******************************************************************************/

void	CToolDrag::Undo()
{
	LongPt	centerPt;
	
	centerPt = itsPane->itsCenter;
	itsPane->itsCenter = saveCenter;
	saveCenter = centerPt;
	
	inherited::Undo();
}