/******************************************************************************
 Symbols.c

		Symbol data manager for the SignWriter symbol editor
		
		Copyright  1995 Center for Sutton Movement Writing. All rights reserved.

 ******************************************************************************/
 
 
 
#include "Symbols.h"
#include "Exceptions.h"
#include "Global.h"
#include "LongCoordinates.h"
#include "Memory.h"
#include "SymEditConsts.h"





/**************************  Symbol data structures  ************************/




typedef struct {		// image 

	Point   framePt;
	Point	centerPt;
	short	numBytes;
	Byte	bitMap[1];
	
} ImageRec, *Image;



typedef struct {		// fill variation 

	ImageClass  imgClass;
	short	  	numRots;
	short	  	numFlops;
	short	  	numImgs;
	Image	  	img[1];
	
} FillVarRec, *FillVar;



typedef struct {		// symbol variation

	short	numFills;
	FillVar fill[1];
	
} SymVarRec, *SymVar;



typedef struct {		// symbol

	SymbolType  symType;
	short		numVars;
	SymVar  	var[1];
	
} SymRec, *Sym;



static Sym  symTable[FGRID_SYMS] = {NULL};		// symbol table





/**************************  Private prototypes  ************************/


Image   NewImage (short numBytes);								// Allocation
FillVar NewFillVar (short numImgs);
SymVar  NewSymVar (short numFills);
Sym     NewSym (short numVars);


void    DisposImage (Image img);							    // Deallocation
void    DisposFillVar (FillVar fill, Boolean subRecs);
void    DisposSymVar (SymVar var, Boolean subRecs);
void    DisposSym (Sym sym, Boolean subRecs);


Image   IImage (void);											// Initialization
FillVar IFillVar (void);
SymVar  ISymVar (void);
Sym     ISym (void);


Image 	CopyImage (Image img);									// Duplication
FillVar CopyFillVar (FillVar fill);
SymVar  CopySymVar (SymVar var);
Sym  	CopySym (Sym sym);


void    ClearImage (Image *img);							    // Clearing
void    ClearFillVar (FillVar fill);
void    ClearSymVar (SymVar var);
//      no ClearSym


void    Translate (ImageClass imgClass, short rot, short flop,	// Translation
			       short *image, short *transform);
	
				 
void	GetByte (short *value);									// Packing support
void	GetBytes (void *base, short bytes);
void	GetWord (short *value);
void	PutByte (short value);
void	PutBytes (void *base, short bytes);
void	PutWord (short value);
void	PutWordAt (short value, unsigned short inx);		 		 
				 
				 
void	PackImage (Image img, unsigned short inx);				// Packing
void	PackFillVar (FillVar fillVar, unsigned short inx);
void	PackSymVar (SymVar symVar, unsigned short inx);
void	PackSym (Sym sym);


void	UnpackImage (Image *img, unsigned short inx);			// Unpacking
void	UnpackFillVar (FillVar *fillVar, unsigned short inx);
void	UnpackSymVar (SymVar *symVar, unsigned short inx);
void	UnpackSym (Sym *sym);




/****************************  Symbol status  ****************************/


/***
 * SymEmpty
 *
 ***/
 
 Boolean SymEmpty (short symCode)

{
	return symTable[symCode] == NULL;
}




/****************************  Symbol count  *****************************/


/***
 * GetSymbols
 *
 ***/
 
void  GetSymbols (short *numSyms)			 				  

{
	short	x;
	short	count = 0;
	
	for (x = 1; x < FGRID_SYMS; x++)
		if (symTable[x])
			count++;
			
	*numSyms = count;
}




/****************************  Symbol table  ****************************/


/***
 * InsertSym
 *
 ***/
 
void  InsertSym (short symCode)

{
	short   size = (FGRID_SYMS - symCode - 1) * sizeof(Sym);
			
	BlockMove(&symTable[symCode], &symTable[symCode+1], size);
	
	symTable[symCode] = NULL;
}



/***
 * DeleteSym
 *
 ***/
 
void  DeleteSym (short symCode)

{
	short   size = (FGRID_SYMS - symCode - 1) * sizeof(Sym);
	
	BlockMove(&symTable[symCode+1], &symTable[symCode], size);
	
	symTable[FGRID_SYMS-1] = NULL;
}
 
 
			

/*****************************  Allocation  *****************************/


/***
 * NewImage
 *
 ***/
 
Image NewImage (short numBytes)

{
	short   newSize = sizeof(ImageRec) + ((numBytes - 1) * sizeof(Byte));
	Image	newImg;
	
	newImg  = (Image) NewPtrClear(newSize);
	FailMemError();
	
	newImg->numBytes = numBytes;
	return newImg;
}
	  


/***
 * NewFillVar
 *
 ***/
 
FillVar NewFillVar (short numImgs)

{
	short   newSize = sizeof(FillVarRec) + ((numImgs - 1) * sizeof(Image));
	FillVar	newFill;
	
	newFill = (FillVar) NewPtrClear(newSize);
	FailMemError();
	
	newFill->numImgs = numImgs;
	return newFill;
}



/***
 * NewSymVar
 *
 ***/
 	
SymVar NewSymVar (short numFills)

{
	short   newSize = sizeof(SymVarRec) + ((numFills - 1) * sizeof(FillVar));
	SymVar	newVar;
	
	newVar  = (SymVar) NewPtrClear(newSize);
	FailMemError();
	
	newVar->numFills = numFills;
	return newVar;
}



/***
 * NewSym
 *
 ***/
 
Sym NewSym (short numVars)

{
	short newSize = sizeof(SymRec) + ((numVars - 1) * sizeof(SymVar));
	Sym	  newSym;
	
	newSym  = (Sym) NewPtrClear(newSize);
	FailMemError();
	
	newSym->numVars = numVars;
	return newSym;
}



/***
 * NewSymbol
 *
 *	Initialize an empty symbol entry
 *  for the specified symbol code.
 *
 ***/
 
void NewSymbol (short symCode)

{	
	symTable[symCode] = NULL;	
}




/*****************************  Deallocation  *****************************/



/***
 * DisposImage
 *
 ***/
 
void  DisposImage (Image img)

{
	if (img == NULL) return;
	
	DisposPtr(img);
	FailMemError();	
}



/***
 * DisposFillVar
 *
 ***/
 
 void  DisposFillVar (FillVar fill, Boolean subRecs)

{
	short x;
	
	if (subRecs)
		for (x = 0; x < fill->numImgs; x++)
			DisposImage(fill->img[x]);
	
	DisposPtr(fill);
	FailMemError();
}



/***
 * DisposSymVar
 *
 ***/
 
void  DisposSymVar (SymVar var, Boolean subRecs)

{
	short x;
	
	if (subRecs)
		for (x = 0; x < var->numFills; x++)
			DisposFillVar(var->fill[x], TRUE);
	
	DisposPtr(var);
	FailMemError();
}



/***
 * DisposSym
 *
 ***/
 
void  DisposSym (Sym sym, Boolean subRecs)

{
	short x;
	
	if (sym == NULL) return;

	if (subRecs)
		for (x = 0; x < sym->numVars; x++)
			DisposSymVar(sym->var[x], TRUE);
	
	DisposPtr(sym);
	FailMemError();
}



/***
 * DisposSymbol
 *
 ***/
 
void  DisposSymbol (short symCode)

{	
	DisposSym(symTable[symCode], TRUE);	
}




/**************************  Initialization  **************************/



/***
 * IImage
 *
 *	Return a value indicating an empty
 *  image.  The actual image record is
 *  created when an image is saved.
 *
 ***/

Image IImage (void)

{
	return NULL;
}



/***
 * IFillVar
 *
 *	Create a fill variation record and
 *  initialize it to contain a single
 *  (empty) image.
 *
 ***/

FillVar IFillVar (void)

{
	FillVar fillVar = NewFillVar(1);
	
	fillVar->imgClass = class1;
	fillVar->numRots  = 0;
	fillVar->numFlops = 0;
	fillVar->img[0]   = IImage();
	
	return fillVar;
}



/***
 * ISymVar
 *
 *	Create the specified symbol variation
 *  and initialize it to contain a single
 *  fill variation.
 *
 ***/
 
SymVar ISymVar (void)

{
	SymVar symVar = NewSymVar(1);	
	
	symVar->fill[0] = IFillVar();
	
	return symVar;	
}



/***
 *  ISym
 *
 *	Create the specified symbol record
 *  and initialize it to contain a single
 *  symbol variation.
 *
 ***/
 
Sym ISym (void)

{	
	Sym sym = NewSym(1);

	sym->symType = other;	
	sym->var[0] = ISymVar();
	
	return sym;	
}



/***
 * ISymbol
 *
 *	Create a symbol entry for the specified
 *  symcode and initialize it to contain a 
 *  single symbol variation.
 *
 ***/
 
void ISymbol (short symCode)

{	
	symTable[symCode] = ISym();	
}




/*****************************  Duplication  *****************************/


/***
 * CopyImage
 *
 ***/
 
Image CopyImage (Image img)

{
	Image  newImg;
	
	if (img == NULL) return NULL;
	
	newImg = NewImage(img->numBytes);
	
	newImg->framePt  = img->framePt;	
	newImg->centerPt = img->centerPt;	
		
	BlockMove(&img->bitMap[0], &newImg->bitMap[0], img->numBytes);
	
	return newImg;
}



/***
 * CopyFillVar
 *
 ***/
 
FillVar CopyFillVar (FillVar fill)

{
	FillVar	  newFillVar = NewFillVar(fill->numImgs);
	short 	  x;

	newFillVar->imgClass = fill->imgClass;	
	newFillVar->numFlops = fill->numFlops;	
	newFillVar->numRots  = fill->numRots;	

	for (x = 0; x < fill->numImgs; x++)
		newFillVar->img[x] = CopyImage(fill->img[x]);
	
	return newFillVar;
}



/***
 * CopySymVar
 *
 ***/
 	
SymVar CopySymVar (SymVar var)

{
	SymVar	newSymVar = NewSymVar(var->numFills);
	short 	x;

	for (x = 0; x < var->numFills; x++)
		newSymVar->fill[x] = CopyFillVar(var->fill[x]);
	
	return newSymVar;
}



/***
 * CopySym
 *
 ***/
	
Sym CopySym (Sym sym)

{
	Sym		newSym;
	short 	x;
	
	if (sym == NULL) return NULL;
	
	newSym = NewSym(sym->numVars);

	newSym->symType = sym->symType;	

	for (x = 0; x < sym->numVars; x++)
		newSym->var[x] = CopySymVar(sym->var[x]);
	
	return newSym;
}




/*****************************  Clearification  *****************************/


/***
 * ClearImage
 *
 ***/
 
void  ClearImage (Image *img)

{
	DisposImage(*img);
	*img = NULL;
}



/***
 * ClearFillVar
 *
 ***/
 
 void  ClearFillVar (FillVar fill)

{
	short x;
	
	for (x = 0; x < fill->numImgs; x++)
		ClearImage(&fill->img[x]);
}



/***
 * ClearSymVar
 *
 ***/
 
void  ClearSymVar (SymVar var)

{
	short x;
		
	for (x = 0; x < var->numFills; x++)
		ClearFillVar(var->fill[x]);
}



// no ClearSymbol




/***********************  Attribute operations  ************************/



/***
 * GetSymType
 *
 ***/
 
void GetSymType (short symCode, SymbolType *symType)

{
	*symType = symTable[symCode]->symType;
}



/***
 * SetSymType
 *
 ***/
 
void SetSymType (short symCode, SymbolType symType)

{
	symTable[symCode]->symType = symType;
}



/***
 * GetSymVars
 *
 ***/
 
void GetSymVars (short symCode, short *numVars)

{
	*numVars = symTable[symCode]->numVars;
}



/***
 * SetSymVars
 *
 ***/
 
void SetSymVars (short symCode, short numVars)

{
	Sym		old = symTable[symCode];
	Sym     nu  = NewSym(numVars);
	short	x, count;
	
	
	// copy sym type
	
	nu->symType = old->symType;
	
	
	// copy sym variations
	
	count = Min(numVars, old->numVars);
	
	for (x = 0; x < count; x++)
		nu->var[x] = old->var[x];


	// process leftovers
	
	count = Max(numVars, old->numVars);	
		
	for (; x < count; x++)
		if (numVars > old->numVars)
			nu->var[x] = ISymVar();						
		else
			DisposSymVar(old->var[x], TRUE);	
	
		
	// update sym record
		
	symTable[symCode] = nu;
	DisposSym(old, FALSE);
	
}



/***
 * GetFillVars
 *
 ***/
 
void GetFillVars (short symCode, short symVar, short *numFills)

{
	*numFills = symTable[symCode]->var[symVar-1]->numFills;
}



/***
 * SetFillVars
 *
 ***/
 
void SetFillVars (short symCode, short symVar, short numFills)

{
	SymVar  old = symTable[symCode]->var[symVar-1];
	SymVar  nu  = NewSymVar(numFills);
	short	x, count;
	
	
	// copy fill variations
	
	count = Min(numFills, old->numFills);
	
	for (x = 0; x < count; x++)
		nu->fill[x] = old->fill[x];


	// process leftovers
	
	count = Max(numFills, old->numFills);	
		
	for (; x < count; x++)
		if (numFills > old->numFills)
			nu->fill[x] = IFillVar();						
		else
			DisposFillVar(old->fill[x], TRUE);	
	
		
	// update symvar record
		
	symTable[symCode]->var[symVar-1] = nu;
	DisposSymVar(old, FALSE);
	
}



/***
 * GetClass
 *
 ***/	
 			   
void GetClass (short symCode, short symVar, short fillVar, ImageClass *imgClass)

{
	*imgClass = symTable[symCode]->var[symVar-1]->fill[fillVar-1]->imgClass;
}	



/***
 * SetClass
 *
 ***/
 			 
void SetClass (short symCode, short symVar, short fillVar, ImageClass imgClass)

{
	FillVar  old, nu;
	short	 numImgs;
	short	 x, count;
	
	GetImages(imgClass, &numImgs);
	
	old = symTable[symCode]->var[symVar-1]->fill[fillVar-1];	
	nu  = NewFillVar(numImgs);


	// set nu class
	
	nu->imgClass = imgClass;
	
	
	// copy images
	
	count = Min(numImgs, old->numImgs);
	
	for (x = 0; x < count; x++)
		nu->img[x] = old->img[x];


	// process leftovers
	
	count = Max(numImgs, old->numImgs);	
		
	for (; x < count; x++)
		if (numImgs > old->numImgs)
			nu->img[x] = IImage();						
		else
			DisposImage(old->img[x]);	
	
		
	// update fillvar record
		
	symTable[symCode]->var[symVar-1]->fill[fillVar-1] = nu;
	DisposFillVar(old, FALSE);
}				    



/***
 * GetRotations
 *
 ***/
 
void  GetRotations (short symCode, short symVar, short fillVar, short *numRots)

{
	*numRots = symTable[symCode]->var[symVar-1]->fill[fillVar-1]->numRots;
}	

		

/***
 * SetRotations
 *
 ***/
 				 
void  SetRotations (short symCode, short symVar, short fillVar, short numRots)

{
	symTable[symCode]->var[symVar-1]->fill[fillVar-1]->numRots = numRots;
}					   



/***
 * GetFlops
 *
 ***/
 
void  GetFlops (short symCode, short symVar, short fillVar, short *numFlops)

{
	*numFlops = symTable[symCode]->var[symVar-1]->fill[fillVar-1]->numFlops;
}		

		 

/***
 * SetFlops
 *
 ***/
 		 
void  SetFlops (short symCode, short symVar, short fillVar, short numFlops)

{
	symTable[symCode]->var[symVar-1]->fill[fillVar-1]->numFlops = numFlops;
}	
	
				

/***
 * GetImages
 *
 ***/		
 		
void  GetImages (ImageClass imgClass, short *numImgs)

{
	switch (imgClass) {
	
		case class1:
			*numImgs = 1;
			break;
			
		case class2:
			*numImgs = 2;
			break;	
			
		case class3A:
		case class3B:
			*numImgs = 3;
			break;	
			
		case class4:
			*numImgs = 4;
			break;
			
		case class5:
			*numImgs = 5;
			break;
		
		default:
			*numImgs = 1;
			break;
	}	
}



		   		
/*************************  Image operations  ************************/




static Byte   defaultBitMap[2] = {0};			// default sym image
static Point  defaultFramePt   = {24, 24};
static Point  defaultCenterPt  = {12, 12};



/***
 * GetImage
 *
 ***/	
 	
void  GetImage (short symCode, short symVar, short fillVar, short image, 
	 			BitMap *img, LongPt *frame, LongPt *center)
	 						
{
	Image	im = symTable[symCode]->var[symVar-1]->fill[fillVar-1]->img[image-1];
	BitMap	map;

	
	if (im == NULL) {					// get default image
		
		map.baseAddr = (void *) &defaultBitMap;
		map.rowBytes = 2;
		SetRect(&map.bounds, 0, 0, 0, 0);
		
		QDToLongPt(defaultFramePt, frame);
		QDToLongPt(defaultCenterPt, center);
				
	
	} else {							// get sym image
	
		map.baseAddr = (void *) &im->bitMap;
		map.rowBytes = 2 * ((im->framePt.h + 15) / 16);

		SetRect(&map.bounds, 0, 0, im->framePt.h, im->framePt.v);
	
		QDToLongPt(im->framePt, frame);
		QDToLongPt(im->centerPt, center);	
	}
					
										// copy image to edit area
		
	CopyBits(&map, img, &map.bounds, &map.bounds, srcCopy, NULL);
	
}



/***
 * SetImage
 *
 ***/
 
void  SetImage (short symCode, short symVar, short fillVar, short image,
				BitMap *img, LongPt *frame, LongPt *center)

{
	FillVar	fill = symTable[symCode]->var[symVar-1]->fill[fillVar-1];
	short	rowBytes;
	BitMap	map;
	Image	im;
		
												// dump old image
	if (fill->img[image-1] != NULL)	{	
			
		DisposImage(fill->img[image-1]);
		fill->img[image-1] = NULL;
	}
	
												// Alloc new image rec
	rowBytes = 2 * ((frame->h + 15) / 16);		
	
	im = NewImage(rowBytes * frame->v);
	
	LongToQDPt(frame, &im->framePt);
	LongToQDPt(center, &im->centerPt);	
	
	map.baseAddr = (void *) &im->bitMap;
	map.rowBytes = rowBytes;	

	SetRect(&map.bounds, 0, 0, frame->h, frame->v);
									
												// copy image to image rec	
												
	CopyBits(img, &map, &map.bounds, &map.bounds, srcCopy, NULL);   
	
	
	fill->img[image-1] = im;					// update symbol table		
	
}



/***
 * DrawImage
 *
 ***/	
 	
void  DrawImage (short symCode, short symVar, short fillVar, short image, 
				 BitMap *img, LongPt *imgPos)		
				  						
{
	Image	im = symTable[symCode]->var[symVar-1]->fill[fillVar-1]->img[image-1];
	Rect	bounds;
	BitMap	map;
	
	
	if (im != NULL) {
	
		map.baseAddr = (void *) &im->bitMap;
		map.rowBytes = 2 * ((im->framePt.h + 15) / 16);
		
		SetRect(&map.bounds, 0, 0, im->framePt.h, im->framePt.v);
		bounds = map.bounds;
	
		OffsetRect(&bounds, imgPos->h, imgPos->v);	
		CopyBits(&map, img, &map.bounds, &bounds, srcCopy, NULL);	
	}	
}




/***********************  Symbol drawing data structures  **********************/



//  Map rotation and flop values into image numbers and image transforms.


typedef enum {n, X, Y, XY} Transform;

			  //     n  no image transform
			  //     X  invert image horizontally
			  //     Y  invert image vertically
			  //    XY  invert image horizontally and vertically
			  
			  
typedef struct {

	Byte	image;
	Byte	transform;	
	
} ImageMap;

	
static ImageMap  mapClass1		[2] = { {0,n},{0,Y}  };
static ImageMap  mapClass1Flop  [2] = { {0,X},{0,XY} };

static ImageMap  mapClass2      [3] = { {0,n},{1,n},{0,Y}  };
static ImageMap  mapClass2Flop  [3] = { {0,X},{1,X},{0,XY} };

static ImageMap  mapClass3A     [8] = { {0,n},{1,n},{2,n},{1,Y},{0,Y},{1,XY},{2,X},{1,X} };
static ImageMap  mapClass3AFlop [8] = { {0,X},{1,X},{2,X},{0,n},{0,n},{0,n}, {0,n},{0,n} };

static ImageMap  mapClass3B     [8] = { {0,n},{1,n}, {2,n},{1,X},{0,X},{1,XY},{2,Y},{1,Y} };
static ImageMap  mapClass3BFlop [8] = { {0,X},{1,XY},{2,Y},{1,Y},{0,n},{0,n}, {0,n},{0,n} };

static ImageMap  mapClass4     [8] = { {0,n},{1,n},{2,n},{3,n},{0,XY},{1,XY},{2,XY},{3,XY} };
static ImageMap  mapClass4Flop [8] = { {0,X},{3,Y},{2,Y},{1,Y},{0,Y}, {3,X}, {2,X}, {1,X}  };

static ImageMap  mapClass5     [8] = { {0,n},{1,n},{2,n},{3,n},{4,n},{3,X},{2,X},{1,X} };
static ImageMap  mapClass5Flop [8] = { {0,X},{1,X},{2,X},{3,X},{4,X},{0,n},{0,n},{0,n} };
	
	
	

//  Map byte value into its bit-wise mirror image.
    
static Byte  flipByte[256] = { 
								0x00, 0x80, 0x40, 0xC0, 0x20, 0xA0, 0x60, 0xE0, 
								0x10, 0x90, 0x50, 0xD0, 0x30, 0xB0, 0x70, 0xF0, 
								
								0x08, 0x88, 0x48, 0xC8, 0x28, 0xA8, 0x68, 0xE8, 
								0x18, 0x98, 0x58, 0xD8, 0x38, 0xB8, 0x78, 0xF8, 
								
								0x04, 0x84, 0x44, 0xC4, 0x24, 0xA4, 0x64, 0xE4, 
								0x14, 0x94, 0x54, 0xD4, 0x34, 0xB4, 0x74, 0xF4, 
								
								0x0C, 0x8C, 0x4C, 0xCC, 0x2C, 0xAC, 0x6C, 0xEC, 
								0x1C, 0x9C, 0x5C, 0xDC, 0x3C, 0xBC, 0x7C, 0xFC, 
								
								0x02, 0x82, 0x42, 0xC2, 0x22, 0xA2, 0x62, 0xE2, 
								0x12, 0x92, 0x52, 0xD2, 0x32, 0xB2, 0x72, 0xF2, 
								
								0x0A, 0x8A, 0x4A, 0xCA, 0x2A, 0xAA, 0x6A, 0xEA, 
								0x1A, 0x9A, 0x5A, 0xDA, 0x3A, 0xBA, 0x7A, 0xFA, 
								
								0x06, 0x86, 0x46, 0xC6, 0x26, 0xA6, 0x66, 0xE6, 
								0x16, 0x96, 0x56, 0xD6, 0x36, 0xB6, 0x76, 0xF6, 
								
								0x0E, 0x8E, 0x4E, 0xCE, 0x2E, 0xAE, 0x6E, 0xEE, 
								0x1E, 0x9E, 0x5E, 0xDE, 0x3E, 0xBE, 0x7E, 0xFE, 
								
								0x01, 0x81, 0x41, 0xC1, 0x21, 0xA1, 0x61, 0xE1, 
								0x11, 0x91, 0x51, 0xD1, 0x31, 0xB1, 0x71, 0xF1, 
								
								0x09, 0x89, 0x49, 0xC9, 0x29, 0xA9, 0x69, 0xE9, 
								0x19, 0x99, 0x59, 0xD9, 0x39, 0xB9, 0x79, 0xF9, 
								
								0x05, 0x85, 0x45, 0xC5, 0x25, 0xA5, 0x65, 0xE5, 
								0x15, 0x95, 0x55, 0xD5, 0x35, 0xB5, 0x75, 0xF5, 
								
								0x0D, 0x8D, 0x4D, 0xCD, 0x2D, 0xAD, 0x6D, 0xED, 
								0x1D, 0x9D, 0x5D, 0xDD, 0x3D, 0xBD, 0x7D, 0xFD, 
								
								0x03, 0x83, 0x43, 0xC3, 0x23, 0xA3, 0x63, 0xE3, 
								0x13, 0x93, 0x53, 0xD3, 0x33, 0xB3, 0x73, 0xF3, 
								
								0x0B, 0x8B, 0x4B, 0xCB, 0x2B, 0xAB, 0x6B, 0xEB, 
								0x1B, 0x9B, 0x5B, 0xDB, 0x3B, 0xBB, 0x7B, 0xFB, 
								
								0x07, 0x87, 0x47, 0xC7, 0x27, 0xA7, 0x67, 0xE7, 
								0x17, 0x97, 0x57, 0xD7, 0x37, 0xB7, 0x77, 0xF7, 
								
								0x0F, 0x8F, 0x4F, 0xCF, 0x2F, 0xAF, 0x6F, 0xEF, 
								0x1F, 0x9F, 0x5F, 0xDF, 0x3F, 0xBF, 0x7F, 0xFF
							 };





/****************************  Symbol translation  *****************************/



/***
 * Translate
 *
 ***/
 
void  Translate (ImageClass imgClass, short rot, short flop,
	  		  	 short *image, short *transform)

{
	switch (imgClass) {
	
		case class1:
			
			if (flop) {
				*image     = mapClass1Flop[rot].image;
				*transform = mapClass1Flop[rot].transform;
				
			} else {
				*image     = mapClass1[rot].image;
				*transform = mapClass1[rot].transform;
			}
			break;
		
			
		case class2:
		
			if (flop) {
				*image     = mapClass2Flop[rot].image;
				*transform = mapClass2Flop[rot].transform;
				
			} else {
				*image     = mapClass2[rot].image;
				*transform = mapClass2[rot].transform;
			}
			break;
			
			
		case class3A:
		
			if (flop) {
				*image     = mapClass3AFlop[rot].image;
				*transform = mapClass3AFlop[rot].transform;
				
			} else {
				*image     = mapClass3A[rot].image;
				*transform = mapClass3A[rot].transform;
			}
			break;
			
		
		case class3B:
		
			if (flop) {
				*image     = mapClass3BFlop[rot].image;
				*transform = mapClass3BFlop[rot].transform;
				
			} else {
				*image     = mapClass3B[rot].image;
				*transform = mapClass3B[rot].transform;
			}
			break;
			
			
		case class4:
		
			if (flop) {
				*image     = mapClass4Flop[rot].image;
				*transform = mapClass4Flop[rot].transform;
				
			} else {
				*image     = mapClass4[rot].image;
				*transform = mapClass4[rot].transform;
			}
			break;
			
			
		case class5:
		
			if (flop) {
				*image     = mapClass5Flop[rot].image;
				*transform = mapClass5Flop[rot].transform;
				
			} else {
				*image     = mapClass5[rot].image;
				*transform = mapClass5[rot].transform;
			}
			break;
			
		
		default:
			
			*image     = 0;
			*transform = n;
			break;
						
	}		
}




/************************  Symbol drawing operations  ************************/


/***
 * GetCoords
 *
 ***/
 
void  GetCoords (short sym, short symVar, short fillVar, short rot, short flop, 
				 LongPt *frame, LongPt *center, Boolean thumbNail)
				   
{
	FillVar 	fill = symTable[sym]->var[symVar-1]->fill[fillVar-1];
	ImageClass  imgClass;
	Image		im;
	short		image, transform;
	Point		fram, cent;
	
															// get sym image
	GetClass(sym, symVar, fillVar, &imgClass);
	Translate(imgClass, rot, flop, &image, &transform);
	
	im = fill->img[image];
															// get image coords
	if (im == NULL) {
		fram = defaultFramePt;
		cent = defaultCenterPt;

	} else {
		fram = im->framePt;
		cent = im->centerPt;
	}
															// adjust coords
	if (transform == X || transform == XY) 
		cent.h = fram.h - cent.h;	
		
	if (transform == Y || transform == XY) 
		cent.v = fram.v - cent.v;
															// scale coords
	if (thumbNail) {
		fram.h /= FCELL_SCALE;   cent.h /= FCELL_SCALE;
		fram.v /= FCELL_SCALE;   cent.v /= FCELL_SCALE;
	}
	
	QDToLongPt(fram, frame);
	QDToLongPt(cent, center);	
	
}



/***
 * DrawSymbol
 *
 ***/
 
void  DrawSymbol (short sym, short symVar, short fillVar, short rot, short flop,
			      BitMap *img, LongPt *imgPos, Boolean thumbNail)
			      
{
	FillVar 	fill = symTable[sym]->var[symVar-1]->fill[fillVar-1];
	Byte		bitMap[IMAP_BYTES];
	short		i, j, count, xSize;
	short		image, transform;
	Rect		source, dest;
	ImageClass  imgClass;
	BitMap		map;
	Image		im;

														// get image
	GetClass(sym, symVar, fillVar, &imgClass);
	Translate(imgClass, rot, flop, &image, &transform);
	
	im = fill->img[image];	
	
	if (im == NULL)
		return;
	
														// init draw bitmap
	map.baseAddr = (void *) &bitMap[0];
	map.rowBytes = 2 * ((im->framePt.h + 15) / 16);
	SetRect(&map.bounds, 0, 0, im->framePt.h, im->framePt.v);
	
	count = im->numBytes;
	xSize = map.rowBytes;
	
														// transform image
	switch (transform) {
	
	
		case n:											//  no transform
		
			for (i = 0; i < count; i++)
				bitMap[i] = im->bitMap[i];
						
			break;
		
		
		case X:				  							//  invert horizontally
	
			for (j = 0; j < count; j += xSize) 
				for (i = 0; i < xSize; i++)
					bitMap[i+j] = flipByte[im->bitMap[xSize-1-i+j]];	
					
			break;
	
	
		case Y:			  								//  invert vertically

			for (j = 0; j < count; j += xSize) 
				for (i = 0; i < xSize; i++)
					bitMap[i+j] = im->bitMap[i+count-xSize-j];	
			
			break;
		
		
		case XY:			  							//  invert horiz. and vert.
	
			for (i = 0; i < count; i++)
				bitMap[i] = flipByte[im->bitMap[count-1-i]];	
		
			break;			
	}
		
														//  init image source
	source = map.bounds;	
	if (transform == X || transform == XY) 
		OffsetRect(&source, (xSize * 8) - im->framePt.h, 0);	
		
														// init image dest
	dest = map.bounds;
	if (thumbNail) {	
		dest.bottom /= FCELL_SCALE;
		dest.right  /= FCELL_SCALE;
	}	
	OffsetRect(&dest, imgPos->h, imgPos->v);
		
														// draw symbol														
	CopyBits(&map, img, &source, &dest, srcCopy, NULL);
			
}				 




/**************************  Symbol editing  ****************************/
				 				  

/***
 * GetSymbolRec
 *
 ***/
 
void  GetSymbolRec (short symCode,
					void **symbolRec)

{
	*symbolRec = (void *) symTable[symCode];
}



/***
 * SetSymbolRec
 *
 ***/	
 			  
void  SetSymbolRec (short symCode,
					void *symbolRec)

{
	symTable[symCode] = (Sym) symbolRec;
}

			

/***
 * CopySymbolRec
 *
 ***/
 
void  CopySymbolRec (void *symbolRec, void **copyRec)

{
	*copyRec = (void *) CopySym((Sym) symbolRec);
}



// no ClearSymbolRec



/***
 * DisposSymbolRec
 *
 ***/
 				  
void  DisposSymbolRec (void *symbolRec)

{
	DisposSym((Sym) symbolRec, TRUE);
}




/***********************  Sym variation editing  ************************/
				 				  

/***
 * GetSymVarRec
 *
 ***/
 
void  GetSymVarRec (short symCode, short symVar, 
					void **symVarRec)

{
	*symVarRec = (void *) symTable[symCode]->var[symVar-1];
}



/***
 * SetSymVarRec
 *
 ***/	
 			  
void  SetSymVarRec (short symCode, short symVar,
					void *symVarRec)

{
	symTable[symCode]->var[symVar-1] = (SymVar) symVarRec;
}

			

/***
 * CopySymVarRec
 *
 ***/

void  CopySymVarRec (void *symVarRec, void **copyRec)

{
	*copyRec = (void *) CopySymVar((SymVar) symVarRec);
}



/***
 * ClearSymVarRec
 *
 ***/
 
void  ClearSymVarRec (void **symVarRec)

{
	ClearSymVar((SymVar) *symVarRec);
}



/***
 * DisposSymVarRec
 *
 ***/
 				  
void  DisposSymVarRec (void *symVarRec)

{
	DisposSymVar((SymVar) symVarRec, TRUE);
}


	 	
				 	
/***********************  Fill variation editing  ************************/
				 	

/***
 * GetFillVarRec
 *
 ***/
 
void  GetFillVarRec (short symCode, short symVar, short fillVar,
					void **fillVarRec)

{
	*fillVarRec = (void *) symTable[symCode]->var[symVar-1]->fill[fillVar-1];
}



/***
 * SetFillVarRec
 *
 ***/	
 			  
void  SetFillVarRec (short symCode, short symVar, short fillVar,
					void *fillVarRec)

{
	symTable[symCode]->var[symVar-1]->fill[fillVar-1] = (FillVar) fillVarRec;
}

			

/***
 * CopyFillVarRec
 *
 ***/

void  CopyFillVarRec (void *fillVarRec, void **copyRec)

{
	*copyRec = (void *) CopyFillVar((FillVar) fillVarRec);
}



/***
 * ClearFillVarRec
 *
 ***/
 				  
void  ClearFillVarRec (void **fillVarRec)

{
	ClearFillVar((FillVar) *fillVarRec);
}



/***
 * DisposFillVarRec
 *
 ***/
 				  
void  DisposFillVarRec (void *fillVarRec)

{
	DisposFillVar((FillVar) fillVarRec, TRUE);
}




/***************************  Image editing  ****************************/


/***
 * GetImageRec
 *
 ***/
 
void  GetImageRec (short symCode, short symVar, short fillVar, short image,
				   void **imageRec)
				   
{
	*imageRec = (void *) symTable[symCode]->var[symVar-1]->fill[fillVar-1]->img[image-1];
}



/***
 * SetImageRec
 *
 ***/
 				  
void  SetImageRec (short symCode, short symVar, short fillVar, short image,
				   void *imageRec)
				   
{
	symTable[symCode]->var[symVar-1]->fill[fillVar-1]->img[image-1] = (Image) imageRec;
}		

		

/***
 * CopyImageRec
 *
 ***/

void  CopyImageRec (void *imageRec, void **copyRec)

{
	*copyRec = (void *) CopyImage((Image) imageRec);
}



/***
 * ClearImageRec
 *
 ***/
 				  
void  ClearImageRec (void **imageRec)

{
	ClearImage((Image *) imageRec);
}



/***
 * DisposImageRec
 *
 ***/
 				  
void  DisposImageRec (void *imageRec)

{
	DisposImage((Image) imageRec);
}




/************************   Symbol packing & unpacking  ************************/



static	unsigned short  bufIndex;				// symbol buffer index

static	Handle			symBuffer;				// symbol buffer handle



/***
 * GetByte
 *
 ***/
 
void  GetByte (short *value)

{
	Byte  b;
	
	BlockMove(*symBuffer+bufIndex, &b, 1);
	bufIndex++;
	
	*value = b;
}



/***
 * GetBytes
 *
 ***/
 
void  GetBytes (void *base, short bytes)

{
	BlockMove(*symBuffer+bufIndex, base, bytes);
	bufIndex += bytes;
}



/***
 * GetWord
 *
 ***/
 
void  GetWord (short *value)

{
	BlockMove(*symBuffer+bufIndex, value, 2);
	bufIndex += 2;
}



/***
 * PutByte
 *
 ***/
 
void  PutByte (short value)

{
	Byte  b = value;
	
	BlockMove(&b,*symBuffer+bufIndex, 1);
	bufIndex++;
}



/***
 * PutBytes
 *
 ***/
 
void  PutBytes (void *base, short bytes)

{
	BlockMove(base, *symBuffer+bufIndex, bytes);
	bufIndex += bytes;
}



/***
 * PutWord
 *
 ***/
 
void  PutWord (short value)

{
	BlockMove(&value, *symBuffer+bufIndex, 2);
	bufIndex += 2;
}



/***
 * PutWordAt
 *
 ***/
 
void  PutWordAt (short value, unsigned short inx)

{
	BlockMove(&value, *symBuffer+inx, 2);
}




/*****************************   Symbol packing   *****************************/


/***
 * PackImage
 *
 ***/
 
void  PackImage (Image img, unsigned short inx)

{
	Point	framePt  = img->framePt;
	Point	centerPt = img->centerPt;
	short	numBytes = img->numBytes;
	short	x;
	
	
	PutWordAt(bufIndex, inx);
	
	PutByte(framePt.v);
	PutByte(framePt.h);
	PutByte(centerPt.v);
	PutByte(centerPt.h);
	
	PutWord(numBytes);

	PutBytes(&img->bitMap, numBytes);
	
}
	  


/***
 * PackFillVar
 *
 ***/
 
void  PackFillVar (FillVar fillVar, unsigned short inx)

{
	short			imgClass = fillVar->imgClass;
	short			numRots  = fillVar->numRots;
	short			numFlops = fillVar->numFlops;
	short			numImgs  = fillVar->numImgs;
	unsigned short	img[SGRID_ROWS];
	short			x;
	
	
	PutWordAt(bufIndex, inx);
	
	PutByte(imgClass);
	PutByte(numRots);
	PutByte(numFlops);
	
	PutByte(numImgs);

	for (x = 0; x < numImgs; x++) {
		img[x] = bufIndex;
		PutWord(0);
	}
	
	for (x = 0; x < numImgs; x++)
		if (fillVar->img[x] != NULL)					// skip empty images
			PackImage(fillVar->img[x], img[x]);
	
}



/***
 * PackSymVar
 *
 ***/
 	
void  PackSymVar (SymVar symVar, unsigned short inx)

{
	short			numFills = symVar->numFills;
	unsigned short	fill[SGRID_COLS];
	short			x;
	
	
	PutWordAt(bufIndex, inx);

	PutByte(numFills);

	PutByte(0);		// word-aligning filler
	
	for (x = 0; x < numFills; x++) {
		fill[x] = bufIndex;
		PutWord(0);
	}
	
	for (x = 0; x < numFills; x++)
		PackFillVar(symVar->fill[x], fill[x]);

}



/***
 * PackSym
 *
 ***/
 
void PackSym (Sym sym)

{
	short			symType = sym->symType;
	short			numVars = sym->numVars;
	unsigned short  var[SGRID_VARS];
	short			x;
	
		
	PutByte(symType);
	
	PutByte(numVars);
	
	for (x = 0; x < numVars; x++) {
		var[x] = bufIndex;
		PutWord(0);
	}
	
	for (x = 0; x < numVars; x++)
		PackSymVar(sym->var[x], var[x]);

}



/***
 * PackSymbol
 *
 ***/
 
void  PackSymbol (short symCode, Handle symBuf, unsigned short *symBytes)

{
	if (symTable[symCode] == NULL) {				/* empty sym */
		*symBytes = 0;
		return;
	}
	
	symBuffer = symBuf;
	bufIndex  = 0;
	
	PackSym(symTable[symCode]);
	
	*symBytes = bufIndex;
}




/***************************   Symbol unpacking  ******************************/


/***
 * UnpackImage
 *
 ***/
 
void  UnpackImage (Image *img, unsigned short inx)

{
	Point	framePt;
	Point	centerPt;
	short	numBytes;
	short	x;
	
	
	if (inx == 0) {			// skip empty images
		*img = NULL;
		return;
	}
	
	bufIndex = inx;
	
	GetByte(&framePt.v);
	GetByte(&framePt.h);
	GetByte(&centerPt.v);
	GetByte(&centerPt.h);
	
	GetWord(&numBytes);
		
	*img = NewImage(numBytes);	
	
	(*img)->framePt  = framePt;
	(*img)->centerPt = centerPt;

	GetBytes(&((*img)->bitMap), numBytes);
	
}
	  


/***
 * UnpackFillVar
 *
 ***/
 
void  UnpackFillVar (FillVar *fillVar, unsigned short inx)

{
	short			imgClass;
	short			numRots;
	short			numFlops;
	short			numImgs;
	unsigned short  img[SGRID_ROWS];
	short			x;
	
	
	bufIndex = inx;
	
	GetByte(&imgClass);
	GetByte(&numRots);
	GetByte(&numFlops);
	
	GetByte(&numImgs);

	for (x = 0; x < numImgs; x++)
		GetWord((short*) &img[x]);
		
	*fillVar = NewFillVar(numImgs);	
	
	(*fillVar)->imgClass = imgClass;
	(*fillVar)->numRots  = numRots;
	(*fillVar)->numFlops = numFlops;

	for (x = 0; x < numImgs; x++)
		UnpackImage(&((*fillVar)->img[x]), img[x]);
		
}



/***
 * UnpackSymVar
 *
 ***/
 	
void  UnpackSymVar (SymVar *symVar, unsigned short inx)

{
	short			numFills;
	unsigned short	fill[SGRID_COLS];
	short			x, junk;
	
	
	bufIndex = inx;
	
	GetByte(&numFills);
	
	GetByte(&junk);		// word-aligning filler

	for (x = 0; x < numFills; x++)
		GetWord((short*) &fill[x]);
		
	*symVar = NewSymVar(numFills);	
	
	for (x = 0; x < numFills; x++)
		UnpackFillVar(&((*symVar)->fill[x]), fill[x]);
		
}



/***
 * UnpackSym
 *
 ***/
 
void UnpackSym (Sym *sym)

{
	short			symType;
	short			numVars;
	unsigned short	var[SGRID_VARS];
	short			x;
	
	
	GetByte(&symType);
	
	GetByte(&numVars);
	
	for (x = 0; x < numVars; x++)
		GetWord((short*) &var[x]);
		
	*sym = NewSym(numVars);	
	
	(*sym)->symType = symType;
	
	for (x = 0; x < numVars; x++)
		UnpackSymVar(&((*sym)->var[x]), var[x]);

}



/***
 * UnpackSymbol
 *
 ***/
 
void  UnpackSymbol (short symCode, Handle symBuf, unsigned short symBytes)

{
	if (symBytes == 0) {				/* empty sym */
		symTable[symCode] = NULL;
		return;
	}
	
	symBuffer = symBuf;
	bufIndex  = 0;
	
	UnpackSym(&symTable[symCode]);
	
}


