/******************************************************************************
 CDragger.c

							The Dragger Class
		
	SUPERCLASS = CPaintTask
	
	Copyright  1989 Symantec Corporation. All rights reserved.

 ******************************************************************************/

#include <Global.h>
#include <TBUtilities.h>
#include <CDocument.h>
#include "CDragger.h"

/*** Global Variables ***/

extern CBitMap		*gScratchPad;

/*** Class Constants ***/

#define		iDRAG			10			/* Index for Drag task name			*/
#define		OPTKEY_CODE		0x3A		/* Hardware code for the option key	*/


/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 IDragger

		Initialize a Dragger object
 ******************************************************************************/

void	CDragger::IDragger(
	CPaintPane	*aPaintPane,
	CBitMap		*aPainting)
{
	CPaintTask::IPaintTask(iDRAG, aPaintPane, aPainting);
	
	newDrag = TRUE;
	done = FALSE;
	undone = FALSE;
}


/******************************************************************************
 BeginTracking
 ******************************************************************************/

void	CDragger::BeginTracking(
	LongPt		*startPt)
{
	LongRect	currRect;
	LongRect	undoSel;

	if (newDrag) {						/* A first time drag				*/
		newDrag = FALSE;
		undoSel = itsPaintPane->selRect;
		currRect = undoSel;
		this->undoSel = undoSel;
		
		SaveForUndo( &undoSel);			/* Save selection in undo bit map	*/
					
		if (!KeyIsDown(OPTKEY_CODE)) {	/* Option key not down, we clear	*/
										/*   the area under the selection	*/
			itsPainting->BeginDrawing();
			LEraseRect(&undoSel);
			itsPainting->EndDrawing();
			cloned = FALSE;
			
		} else {						/* Option key down, we leave the	*/
			cloned = TRUE;				/*    image along, thereby creating	*/
										/*    a clone of the selection		*/
		}

	} else {							/* Dragging again after a delay		*/
										/*   Selection is already "wet"		*/
		currRect = itsPaintPane->selRect;
		LCopyBits(gScratchPad->macBitMap, itsPainting->macBitMap,
					&currRect, &currRect, srcCopy, NULL);
					
		if (undone) {					/* Correct wording of Undo menu		*/
										/*   item							*/
			((CDocument*) itsPaintPane->itsSupervisor)->undone = FALSE;
			undone = FALSE;
		}
	}
}


/******************************************************************************
 KeepTracking
 ******************************************************************************/

void	CDragger::KeepTracking(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt)
{
	LongRect	currRect;
	LongRect	prevRect;
	LongRect	changedRect;
	LongRect	bounds;
	
										/* Smoothly drag selection			*/
	if (itsPaintPane->AutoScroll( currPt) || !EqualLongPt( currPt, prevPt)) {
										/* Don't allow selection to be		*/
										/*   dragged outside the painting	*/
		itsPaintPane->Prepare();
		
		prevRect = itsPaintPane->selRect;
		itsPaintPane->GetFrame(&currRect);
		currRect.left += startPt->h - prevRect.left;
		currRect.top += startPt->v - prevRect.top;
		currRect.right += startPt->h - prevRect.right;
		currRect.bottom += startPt->v - prevRect.bottom;
		
		PinInRect(&currRect, currPt);
		
		currRect = prevRect;
		OffsetLongRect(&currRect, currPt->h - startPt->h,
							  currPt->v - startPt->v);
		OffsetLongRect(&prevRect, prevPt->h - startPt->h,
							  prevPt->v - startPt->v);
		UnionLongRect(&currRect, &prevRect, &changedRect);
					  
										/* Copy changed area to scratch pad	*/
		LCopyBits(itsPainting->macBitMap, gScratchPad->macBitMap,
				 &changedRect, &changedRect, srcCopy, NULL);

										/* Copy selection to scratch pad	*/
		QDToLongRect( &itsUndoBits->macBitMap->bounds, &bounds);
		LCopyBits(itsUndoBits->macBitMap, gScratchPad->macBitMap,
				 &bounds, &currRect, srcCopy, NULL);
				 
										/* Copy from scratch pad onto the	*/
										/*   screen							*/
		gScratchPad->CopyFrom(&changedRect, &changedRect, NULL);
	}
}


/******************************************************************************
 EndTracking
 ******************************************************************************/

void	CDragger::EndTracking(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt)
{
	LongRect	currRect;
	LongRect		undoSel;

	KeepTracking(currPt, prevPt, startPt);
	
	currRect = itsPaintPane->selRect;	/* The current selection has moved	*/
	OffsetLongRect(&currRect, currPt->h - startPt->h,
						  currPt->v - startPt->v);
	itsPaintPane->selRect = currRect;
	
	itsPaintPane->Prepare();
	
										/* Save area under where selection	*/
										/*   was dragged so we can undo		*/
	LCopyBits(itsPainting->macBitMap, gScratchPad->macBitMap,
				&currRect, &currRect, srcCopy, NULL);
			
	undoSel = this->undoSel;			/* The selected area is actually in	*/
										/*   the undo bits now. Update		*/
										/*   painting to reflect new		*/
										/*   location of selection.			*/
	LCopyBits(itsUndoBits->macBitMap, itsPainting->macBitMap,
				&undoSel, &currRect, srcCopy, NULL);
				
										/* At this point, the paint is		*/
										/*   still "wet". The selection is	*/
										/*   in the undo bits and the		*/
										/*   scratchpad holds what was		*/
										/*   underneath the selection.		*/
}


/******************************************************************************
 Do {OVERRIDE}
 
 	"Dry" the paint of the dragged area
 ******************************************************************************/

void	CDragger::Do()
{
	LongRect	currRect;

	if (!done) {
		currRect = itsPaintPane->selRect;
		
		itsPaintPane->Prepare();
											/* Save bits under selection		*/
		itsUndoBits->SetBoundsOrigin(currRect.left, currRect.top);
		LCopyBits(gScratchPad->macBitMap, itsUndoBits->macBitMap,
				 &currRect, &currRect, srcCopy, NULL);
		
		done = TRUE;
	}
}


/******************************************************************************
 Undo {OVERRIDE}

		Undo drag
 ******************************************************************************/

void	CDragger::Undo()
{
	LongRect	currRect;
	LongRect	undoSel;
	
	if (itsUndoBits != NULL) {			/* Make sure we can undo			*/
		undone = TRUE;
		itsPaintPane->Prepare();
		
		if (done) {						/* Paint has dried					*/
			
										/* Get current and former locations	*/
			itsUndoBits->GetBounds(&currRect);
			undoSel = this->undoSel;
			
										/* Copy both the old and new select	*/
										/*   areas to the scratchpad		*/
			LCopyBits(itsPainting->macBitMap, gScratchPad->macBitMap,
						&currRect, &currRect, srcCopy, NULL);
			LCopyBits(itsPainting->macBitMap, gScratchPad->macBitMap,
						&undoSel, &undoSel, srcCopy, NULL);
						
										/* Put undo area back in painting	*/
			LCopyBits(itsUndoBits->macBitMap, itsPainting->macBitMap,
						&currRect, &currRect, srcCopy, NULL);
			
										/* Old selected area now goes into	*/
										/*    undo bits						*/
			itsUndoBits->SetBoundsOrigin(undoSel.left, undoSel.top);
			LCopyBits(gScratchPad->macBitMap, itsUndoBits->macBitMap,
					 &undoSel, &undoSel, srcCopy, NULL);
					 
					 					/* Put selection back in its old	*/
					 					/*   location						*/
			LCopyBits(gScratchPad->macBitMap, itsPainting->macBitMap,
					 &currRect, &undoSel, srcCopy, NULL);
					 
			itsPaintPane->Prepare();	/* Now redraw the screen			*/
			itsPainting->CopyFrom(&currRect, &currRect, NULL);
			itsPainting->CopyFrom(&undoSel, &undoSel, NULL);
			
			this->undoSel = currRect;
		
		} else {						/* Paint is still wet				*/
										/*   Undo bits hold the selection	*/
			undoSel = this->undoSel;
			currRect = itsPaintPane->selRect;
			
			itsPaintPane->Prepare();	/* Update the screen				*/
			gScratchPad->CopyFrom(&currRect, &currRect, NULL);
			itsUndoBits->CopyFrom(&undoSel, &undoSel, NULL);
			
										/* Update painting					*/
			LCopyBits(gScratchPad->macBitMap, itsPainting->macBitMap,
						&currRect, &currRect, srcCopy, NULL);

			LCopyBits(itsUndoBits->macBitMap, itsPainting->macBitMap,
						&undoSel, &undoSel, srcCopy, NULL);
			itsPaintPane->selRect = undoSel;
			redoSel = currRect;
		}
	}
}


/******************************************************************************
 Redo {OVERRIDE}

		Redo drag
 ******************************************************************************/

void	CDragger::Redo()
{
	LongRect		currRect;
	LongRect		undoSel;
	LongRect		redoSel;
	
	undone = FALSE;
	if (done) {							/* Same as Undo for dry paint		*/
		Undo();
		
	} else {							/* Dragged area is still wet		*/
		itsPaintPane->Prepare();
		currRect = itsPaintPane->selRect;
		undoSel = this->undoSel;
		redoSel = this->redoSel;
		
		if (!cloned) {					/* No clone, so old area is white	*/
			itsPainting->BeginDrawing();
			LEraseRect(&undoSel);
			itsPainting->EndDrawing();
		}
		
		itsPaintPane->Prepare();		/* Undo bits and the painting hold	*/
										/*   all the original info, it's	*/
										/*   just been moved around.		*/
		itsPainting->CopyFrom(&undoSel, &undoSel, NULL);
		LCopyBits(itsUndoBits->macBitMap, itsPainting->macBitMap,
					&undoSel, &redoSel, srcCopy, NULL);
		
		itsPainting->CopyFrom(&redoSel, &redoSel, NULL);
		itsPaintPane->selRect = redoSel;
	}	
}
