/******************************************************************************
 CRadioGroup.c

							The RadioGroup Class
		
		A group of radio buttons where only one button is selected at any
		one time. Clicking on one button in the group, selects that
		button and unselects the previous choice.
	
	SUPERCLASS = CBureaucrat
	
	Copyright  1989 Symantec Corporation. All rights reserved.

 ******************************************************************************/

#include "Global.h"
#include "CRadioGroup.h"
	

/******************************************************************************
 IRadioGroup

		Initialize a RadioGroup object
 ******************************************************************************/

void	CRadioGroup::IRadioGroup(
	CBureaucrat		*aSupervisor)		/* Boss in chain of command			*/
{
										/* Initialize superclass			*/
	CBureaucrat::IBureaucrat(aSupervisor);
	
	itsButtons = new(CCluster);			/* Create cluster for storing		*/
	itsButtons->ICluster();				/*   buttons belonging to the group	*/
	
	station = NULL;						/* No station is selected yet		*/
}


/******************************************************************************
 Dispose {OVERRIDE}

		Dispose of a RadioGroup object
 ******************************************************************************/

void	CRadioGroup::Dispose()
{
	itsButtons->Dispose();				/* Throw out Cluster object. Note	*/
										/*   that this does NOT dispose of	*/
										/*   the Buttons themselves.		*/
	inherited::Dispose();
}


/******************************************************************************
 AddButton

		Add a RadioButton to the group
 ******************************************************************************/

void	CRadioGroup::AddButton(
	CRadioButton	*theRadioButton)	/* RadioButton to add				*/
{
	itsButtons->Add(theRadioButton);
}
	

/******************************************************************************
 RemoveButton

		Remove a RadioButton from the group
 ******************************************************************************/

void	CRadioGroup::RemoveButton(
	CRadioButton	*theRadioButton)	/* RadioButton to remove			*/
{
	itsButtons->Remove(theRadioButton);
}
	

/******************************************************************************
 AddButtonID

		Add a RadioButton, specified by a CNTL ID number, to the group
 ******************************************************************************/

void	CRadioGroup::AddButtonID(
	short		CNTLid,					/* CNTL Res ID for radio button		*/
	CView		*anEnclosure)			/* View containing the radio button	*/
{
	CRadioButton	*theRadioButton;
	
	theRadioButton = new(CRadioButton);
	theRadioButton->IRadioButton(CNTLid, anEnclosure, this);
	AddButton(theRadioButton);
}
	

/******************************************************************************
 ChangeStation

		Change the current station
 ******************************************************************************/

void	CRadioGroup::ChangeStation(
	CRadioButton	*theStation)		/* Make this the current station	*/
{
	if (station != NULL) {
		station->SetValue(BUTTON_OFF);
	}
	station = theStation;
	station->SetValue(BUTTON_ON);
}
	

/******************************************************************************
 GetStation

		Return the current station
 ******************************************************************************/

CRadioButton*	CRadioGroup::GetStation()
{
	return(station);
}	


/******************************************************************************
 SetStationID

		Set the current station, specified by an ID number
 ******************************************************************************/

		/*
		 * Check for match of station ID numbers
		 */
		static Boolean	Test_StationID(
			CRadioButton	*theRadioButton,
			long			theStationID)
		{
			return(theRadioButton->GetIdNumber() == (short)theStationID);
		}

void	CRadioGroup::SetStationID(
	short		aStationID)				/* ID number of the new station		*/
{
	CRadioButton	*newStation;

	if ( (station != NULL) && (station->GetIdNumber() != aStationID) ) {
		station->SetValue(BUTTON_OFF);
	}
		
	newStation = (CRadioButton*)
					itsButtons->FindItem1(Test_StationID, (long)aStationID);
					
	if (newStation != NULL) {
		station = newStation;
		station->SetValue(BUTTON_ON);
	}
}
	

/******************************************************************************
 GetStationID

		Return the current station's ID number
 ******************************************************************************/

short	CRadioGroup::GetStationID()
{
	if (station == NULL) {
		return(NO_STATION);
	} else {
		return(station->GetIdNumber());
	}
}