/******************************************************************************
 OList.c

							The List Class
	
	A linear list of objects.
	
	SUPERCLASS = CCluster
	
	Copyright  1989 Symantec Corporation. All rights reserved.

 ******************************************************************************/

#include "Global.h"
#include "OList.h"


/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 IList

		Initialize a List object
 ******************************************************************************/

void	OList::IList()
{
	CCluster::ICluster();				/* Initialize superclass			*/
}
	

/******************************************************************************
 Append

		Add an item to the end of a list
 ******************************************************************************/

void	OList::Append(
	CObject 	*theObject)
{
	CCluster::Add(theObject);
}
	

/******************************************************************************
 Prepend

		Add an item to the beginning of a list
 ******************************************************************************/

void	OList::Prepend(
	CObject 	*theObject)
{
	if (numItems >= slots)				/* Check if we need more space		*/
		MoreSlots();
		
										/* Shift all items down one slot	*/
	if (numItems > 0)									
		BlockMove(*items, *items+1, numItems * SLOT_SIZE);
	(*items)[0] = (long) theObject;		/* Stick new object at the front	*/
	numItems++;							/* There's another item in the list	*/
}
	

/******************************************************************************
 InsertAfter

		Add an item to a list at the position after a specified item
 ******************************************************************************/

void	OList::InsertAfter(
	CObject		*theObject,				/* Object to insert in the list		*/
	CObject		*afterObject)			/* Object to insert it after		*/
{
	register long	afterOffset;
	
	if (numItems >= slots)				/* Check if we need more space		*/
		MoreSlots();
		
	afterOffset = Offset(afterObject);	/* Find location of afterObject		*/
	
	if (afterOffset < numItems - 1) {	/* Move items following afterObject	*/
										/*   down one slot, unless it is	*/
										/*   the last item					*/
		BlockMove(*items+afterOffset+1, *items+afterOffset+2,
					(numItems - afterOffset - 1) * SLOT_SIZE);
	}
										/* Stick new object in the vacated	*/
										/*   slot following afterObject		*/
	(*items)[afterOffset+1] = (long) theObject;
	numItems++;							/* There's another item in the list	*/
}
	

/******************************************************************************
 InsertAt

		Add an item to a list at the specified position
 ******************************************************************************/

void	OList::InsertAt(
	CObject				*theObject,
	register long		index)
{
	if (numItems >= slots)				/* Check if we need more space		*/
		MoreSlots();
		
	if (index <= numItems) {			/* Shift items at index and beyond	*/
										/*   down one slot					*/
		BlockMove(*items+index-1, *items+index,
					(numItems - index + 1) * SLOT_SIZE);
	}
	
										/* Stick new object in vacated slot	*/
	(*items)[index-1] = (long) theObject;
	numItems++;							/* There's another item in the list	*/
}
	

/******************************************************************************
 Remove {OVERRIDE}

		Remove an object from a list
 ******************************************************************************/

void	OList::Remove(
	CObject		*theObject)				/* Object to remove					*/
{
	register long	i;					/* Index into list					*/
	
	i = Offset(theObject);				/* Find location of the object		*/
	
	if (i == BAD_INDEX) {				/* Do nothing if object is not in	*/
		return;							/*   the List						*/
	}

	numItems--;							/* We're gonna get rid of an item	*/
	
	if (i < numItems) {					/* Shift items following the object	*/
										/*   to remove up one slot, thereby	*/
										/*   overwriting it					*/
		BlockMove(*items+i+1, *items+i, (numItems - i) * SLOT_SIZE);
	}
	
	(*items)[numItems] = 0L;			/* Be safe and clear the slot		*/
										/*   formerly occupied by the last	*/
										/*   item							*/
	
	if (slots > numItems + blockSize) {
										/* The number of free slots is		*/
										/*   greater than the blockSize.	*/
										/*   Reduce the size of the items	*/
										/*   handle.						*/
		slots -= blockSize;
		SetHandleSize( (Handle) items, (long) (slots * SLOT_SIZE));
	}
}
	

/******************************************************************************
 BringFront

		Bring an item to the front of a list. Item must already be in the list.
 ******************************************************************************/

void	OList::BringFront(
	CObject 	*theObject)				/* Object to move to the front		*/
{
	register long		i;				/* Index into list					*/
	
	i = Offset(theObject);				/* Find location of the object		*/
										
	if (i > 0) {						/* If it's not already in front ...	*/
										/* Shift items in front of object	*/
										/*   down one slot					*/
		BlockMove(*items, *items+1, i * SLOT_SIZE);
		(*items)[0] = (long) theObject;	/* Stick object at front of list	*/
	}
}
	

/******************************************************************************
 SendBack

		Send an item to the back of a list. Item must already be in the list.
 ******************************************************************************/

void	OList::SendBack(
	CObject 	*theObject)				/* Object to move to the back		*/
{
	register long	i;					/* Index into list					*/
	
	i = Offset(theObject);				/* Find location of the object		*/
										
	if (i < numItems - 1) {				/* If it's not already in back ...	*/
										/* Shift items behind object up		*/
										/*   one slot						*/
		BlockMove(*items+i+1, *items+i, (numItems - i - 1) * SLOT_SIZE);
										/* Stick object at end of list		*/
		(*items)[numItems-1] = (long) theObject;
	}
}
	

/******************************************************************************
 MoveUp

		Move an item up (closer to the front) one position in the list.
 ******************************************************************************/

void	OList::MoveUp(
	CObject		*theObject)				/* Object to move up in list		*/
{
	register long	i;					/* Index into list					*/
	long			swap;				/* Space for swapping values		*/
	
	i = Offset(theObject);				/* Find location of object			*/
										
	if (i > 0) {						/* If it's not at the front ...		*/
		swap = (*items)[i-1];			/*   Swap places with the item		*/
		(*items)[i-1] = (*items)[i];	/*   in front of it					*/
		(*items)[i] = swap;
	}
}
	

/******************************************************************************
 MoveDown

		Move an item down (closer to the back) one position in the list.
 ******************************************************************************/

void	OList::MoveDown(
	CObject		*theObject)				/* Object to move down				*/
{
	register long	i;					/* Index into list					*/
	long			swap;
	
	i = Offset(theObject);				/* Find locaton of object			*/
                                                            
    if (i >= 0  &&  i < numItems-1) {   /* If it's not at the back ...      */
		swap = (*items)[i+1];			/*   Swap places with the item		*/
		(*items)[i+1] = (*items)[i];	/*   behind it						*/
		(*items)[i] = swap;
	}
}
	

/******************************************************************************
 MoveToIndex

		Move an item to a specified position in the list, shifting
		other items in the list appropriately.
 ******************************************************************************/

void	OList::MoveToIndex(
	CObject				*theObject,		/* Object to move within the list	*/
	register long		index)			/* Index at which to move object	*/
{
	register long	i;					/* Index into the list				*/
	
	i = Offset(theObject);				/* Find location of the object		*/
	
	index--;							/* Convert to zero-based indexing	*/
	
	if (i < index) {					/* Object is before target location	*/
										/* Shift items between current		*/
										/*   and target locations down one	*/
		BlockMove(*items+i+1, *items+i, (index - i) * SLOT_SIZE);
		
	} else if (i > index) {				/* Object is after target location	*/
										/* Shift items between target and	*/
										/*   current locations up one		*/
		BlockMove(*items+index, *items+index+1, (i - index) * SLOT_SIZE);
	}
										
	(*items)[index] = (long) theObject;	/* Stick object at target location	*/
}
	

/******************************************************************************
 FirstItem

		Return the first item in the list
 ******************************************************************************/

CObject*	OList::FirstItem()
{
	return(NthItem(1));
}


/******************************************************************************
 LastItem

		Return the last item in the list
 ******************************************************************************/

CObject*	OList::LastItem()
{
	return(NthItem(numItems));
}


/******************************************************************************
 NthItem

		Return the Nth item in the list
 ******************************************************************************/

CObject*	OList::NthItem(
	long		n)
{
	if (n < 1  ||  n > numItems)		/* If index is out of range			*/
		return(NULL);					/*   pass back NULL					*/
	else
										/* Do direct indexing into list		*/
		return( (CObject*) (*items)[n-1]);
}


/******************************************************************************
 FindIndex

		Return the index number in the list of a specified object. Index
		ranges from 1 to numItems.
 ******************************************************************************/

long	OList::FindIndex(
	CObject		*theObject)
{
	return(Offset(theObject) + 1);		/* Index is one greater than the	*/
										/*   offset, which is zero-based	*/
}


/******************************************************************************
 FirstSuccess

		Find the first item in the list which successfully passes (returns
		TRUE) the test function. Return NULL if no item passes the test.
 ******************************************************************************/

CObject*	OList::FirstSuccess(
	BooleanFunc		testFunc)			/* Ptr to test function				*/
{
	inherited::FindItem(testFunc);
}


/******************************************************************************
 FirstSuccess1

		Find the first item in the list which successfully passes (returns
		TRUE) the test function. Return NULL if no item passes the test.
 ******************************************************************************/

CObject*	OList::FirstSuccess1(
	BooleanFunc		testFunc,			/* Ptr to test function				*/
	long			param)				/* Parameter for test function		*/
{
	inherited::FindItem1(testFunc, param);
}


/******************************************************************************
 LastSuccess

		Find the last item in the list which successfully passes (returns
		TRUE) the test function. Return NULL if no item passes the test.
 ******************************************************************************/

CObject*	OList::LastSuccess(
	BooleanFunc		testFunc)			/* Ptr to test function				*/
{
	register long	i;					/* Index into list of items			*/
	
	for (i = numItems-1; i >= 0; i--) {	/* Scan from last item to first		*/
		if ( (*testFunc)((*items)[i]) )
										/* Success! Pass back the object	*/
			return((CObject*)(*items)[i]);
	}
	
	return(NULL);						/* Entire list has been scanned		*/
										/*   with no successes				*/
}


/******************************************************************************
 LastSuccess1

		Find the last item in the list which successfully passes (returns
		TRUE) the test function. Return NULL if no item passes the test.
 ******************************************************************************/

CObject*	OList::LastSuccess1(
	BooleanFunc		testFunc,			/* Ptr to test function				*/
	long			param)				/* Parameter for test function		*/
{
	register long	i;					/* Index into list of items			*/
	
	for (i = numItems-1; i >= 0; i--) {	/* Scan from last item to first		*/
		if ( (*testFunc)((*items)[i], param) )
										/* Success! Pass back the object	*/
			return((CObject*)(*items)[i]);
	}
	
	return(NULL);						/* Entire list has been scanned		*/
										/*   with no successes				*/
}
