/******************************************************************************
 CArray.c

		Implements a dynamically sized array. Elements may be of any
		size, but all elements must be the same size.
		
	SUPERCLASS = CCollection
	
	Copyright  1991 Symantec Corporation. All rights reserved.
	

 ******************************************************************************/

#include "CArray.h"
#include "TCLUtilities.h"

		/* Macros for calculating item offsets and addresses.	*/
		/* They are all one-based indices						*/

#define ASSERT_INDEX( index)	ASSERT( (index > 0)&&(index <= numItems))
#define ITEM_OFFSET( index) ((index-1) * elementSize)
#define ITEM_PTR( index) (&(*hItems)[ ITEM_OFFSET(index) ])

		/* Macro for obtaining a pointer to temporary storage for one 		*/
		/* element The temporary storage is an extra slot kept at the		*/
		/* end of the item handle.											*/
		
#define TMP_ITEM_PTR	(&(*hItems)[ ITEM_OFFSET(numItems+1) ])

/******************************************************************************
 IArray

		Initialize an array object, specifying the element size. The
		initialized array is empty.
		
 ******************************************************************************/

void	CArray::IArray( long anElementSize)
{
	CCollection::ICollection();			/* Initialize superclass			*/
	
	blockSize = 3;
	elementSize = anElementSize;						
	slots = 0;
	lockChanges = FALSE;
	usingTemporary = FALSE;
	hItems = NewHandleCanFail( elementSize);
	FailNIL( hItems);
	
}	/* CArray::IArray */
	

/******************************************************************************
 Dispose {OVERRIDE}

		Dispose of an Array. 
 ******************************************************************************/

void	CArray::Dispose()
{
	ForgetHandle( hItems);				/* Free handle to items				*/
	inherited::Dispose();				/* Pass message on to superclass	*/
	
}	/* CArray::Dispose */

/******************************************************************************
 SetBlockSize

		Set the blockSize which specifies the number of empty slots to
		allocate when more space is needed.
 ******************************************************************************/

void	CArray::SetBlockSize(
	short		aBlockSize)				/* Specified blockSize				*/
{
	blockSize = aBlockSize;				/* Set instance variable			*/
	
}	/* CArray::SetBlockSize */
	
/******************************************************************************
 SetItem

	Set the value of the element at index. Index must be within the array.
	Sends dependents an arrayElementChanged message.
	
 ******************************************************************************/

void CArray::SetItem( void *itemPtr, long index)
{
	ASSERT_INDEX( index);
	
	Store( itemPtr, index);
	BroadcastChange( arrayElementChanged, &index);
	
}	/* CArray::SetItem */

/******************************************************************************
 Store

	Protected method to set an array element.
 ******************************************************************************/

void CArray::Store( void *itemPtr, long index)
{	
	BlockMove( itemPtr, ITEM_PTR( index), elementSize);
	
}	/* CArray::Store */

/******************************************************************************
 GetItem

	Retrieve the value of the array element. Index must be within the array.
	itemPtr must point to storage of sufficient size to hold one element.
 ******************************************************************************/

void CArray::GetItem( void *itemPtr, long index)
{
	ASSERT_INDEX( index);
	
	Retrieve( itemPtr, index);
	
}	/* CArray::GetItem */


/******************************************************************************
 Retrieve

	Protected method to retrieve an array element.
 ******************************************************************************/

void CArray::Retrieve( void *itemPtr, long index)
{	
	BlockMove( ITEM_PTR( index), itemPtr, elementSize);

}	/* CArray::Retrieve */

/******************************************************************************
 InsertAtIndex

		Insert an item into the array at the specified index. Any items
		at or above index are moved down. If index is any value greater
		than the current size, then the item is appended to the end
		of the array.
		Sends dependents an arrayInsertElement message.
		
 ******************************************************************************/

void CArray::InsertAtIndex( void *itemPtr, long index)
{	
	ASSERT(index > 0);
	ASSERT( lockChanges == FALSE);
	
	if (lockChanges) return;
			
	if (numItems >= slots)				/* Check if we need more space		*/
		MoreSlots();

	if (index <= numItems) 
	{									/* Move items at position >= index	*/
										/*   down one slot, unless it is	*/
										/*   the last item					*/
		BlockMove( ITEM_PTR( index), ITEM_PTR( index + 1),
					(numItems - index + 1) * elementSize);
	}
	else
		index = numItems + 1;
		
	numItems++;							/* There's another item in the list	*/

										/* Stick new object in the empty slot*/
	Store( itemPtr, index);
	BroadcastChange( arrayInsertElement, &index);

}	/* CArray::InsertAtIndex */

/******************************************************************************
 DeleteItem

	Delete an item from the Array. Index must be within the array.
	Sends dependents an arrayDeleteElement message.
	
 ******************************************************************************/

void CArray::DeleteItem( long index)
{
	ASSERT_INDEX( index);
	ASSERT( lockChanges == FALSE);

	if (lockChanges) return;

	numItems--;							/* We're gonna get rid of an item	*/
	
	if (index <= numItems) {					/* Shift items following the object	*/
										/*   to remove up one slot, thereby	*/
										/*   overwriting it					*/
		BlockMove( ITEM_PTR( index+1), ITEM_PTR( index),
						(numItems - index + 1) * elementSize);
	}
		
	if (slots > numItems + blockSize) {
										/* The number of free slots is		*/
										/*   greater than the blockSize.	*/
										/*   Reduce the size of the items	*/
										/*   handle.						*/
		Resize( slots-blockSize);
	}
	
	BroadcastChange( arrayDeleteElement, &index);
	
}	/* CArray::DeleteItem */

/******************************************************************************
 Resize
 
 	Resize an array to the desired number of slots. numItems is not affected.
******************************************************************************/

void CArray::Resize( long numSlots)
{
	slots = numSlots;
										/* Be sure to take the scratch		*/
										/* element storage at the beginning */
										/* of the handle into account when	*/
										/* resizing	the item handle			*/
											
	ResizeHandleCanFail( hItems,  (slots+1L) * elementSize);	
	FailMemError();

}	/* CArray::Resize */

/******************************************************************************
 MoreSlots

		Grows the array by blockSize slots.
 ******************************************************************************/

void	CArray::MoreSlots( void)
{
	Resize( slots + blockSize);
	
}	/* CArray::MoreSlots */

/******************************************************************************
 MoveItemToIndex

		Move an element to a specified position in the array, shifting
		other element in the list appropriately.
		Sends dependents an arrayMoveElement message.
 ******************************************************************************/

void CArray::MoveItemToIndex( long currentIndex, long newIndex)
{
	tMovedElementInfo moveInfo;
	
	ASSERT_INDEX( currentIndex);
	ASSERT_INDEX( newIndex);
	
	if (currentIndex == newIndex) return;
	
	/* In order to move the item, we must first make a copy of it.	*/
	/* Since the items could be of any size, we keep storage for	*/
	/* exactly one item at the end of the items handle. In order	*/
	/* to guard against the two methods simultaneously using the	*/
	/* temporary buffer, we mark it in use by setting the			*/
	/* usingTemporary instance variable.							*/
	
	CopyToTemporary( currentIndex);
	
	if (currentIndex < newIndex) 
	{									/* Element is before target location*/
										/* Shift items between current		*/
										/*   and target locations down one	*/
										
		BlockMove( ITEM_PTR( currentIndex+1), ITEM_PTR( currentIndex),
						(newIndex - currentIndex) * elementSize);
									
	}
	else if (currentIndex > newIndex)
	{									/* Element is after target location	*/
										/* Shift items between target and	*/
										/*   current locations up one		*/
		BlockMove( ITEM_PTR( newIndex), ITEM_PTR( newIndex+1),
						(currentIndex - newIndex) * elementSize);
							
	}
	
	/* copy element into new position */
	
	CopyFromTemporary( newIndex);
	
	moveInfo.originalIndex = currentIndex;
	moveInfo.newIndex = newIndex;
	BroadcastChange( arrayMoveElement, &moveInfo);
	
}	/* CArray::MoveItemToIndex */

/******************************************************************************
 SetLockChanges
 
 	Set the lockChanges instance variable and return its old value. Setting
 	lockChanges to TRUE prevents the InsertAtIndex and Delete methods
 	from operating.
******************************************************************************/

Boolean CArray::SetLockChanges( Boolean fLockChanges)
{
	Boolean wasLocked = lockChanges;
	
	lockChanges = fLockChanges;
	
	return wasLocked;

}	/* CArray::SetLockChanges */

/******************************************************************************
 Search
 
 	Find an array element matching the data at itemPtr. matchFunc is used 
 	to perform the comparison and should be declared:
 		int compare( void *item1, void *item2);
 		
 	and should return 0 if the items are equivalent.
 	NOTE: this makes compare compatible with the compare function passed
 	to qsort.
 ******************************************************************************/

long CArray::Search( void *itemPtr, CompareFunc compare)
{
	SignedByte	state;
	register Ptr  items;
	register long i;
	long	foundIndex = BAD_INDEX;
	
	state = HGetState( hItems);
	HLock( hItems);
	
	items = *hItems;
	for (i = 0; i < numItems; i++)
	{
		if (compare( itemPtr, items) == 0)
		{
			foundIndex = i+1;
			break;
		}
		items += elementSize;
	}
	HSetState( hItems, state);
	
	return foundIndex;

}	/* CArray::Search */

/******************************************************************************
 Swap
 
 	Swap two items. Sends dependents an arrayElementChanged message for both
 	items.
 ******************************************************************************/

void CArray::Swap( long index1, long index2)
{
	ASSERT_INDEX( index1);
	ASSERT_INDEX( index2);
	
	CopyToTemporary( index1);
	BlockMove( ITEM_PTR( index2), ITEM_PTR( index1), elementSize);
	CopyFromTemporary( index2);
	
	BroadcastChange( arrayElementChanged, &index1);
	BroadcastChange( arrayElementChanged, &index2);

}	/* CArray::Swap */

/******************************************************************************
 CopyToTemporary
 
 	Place an item into temporary storage
 ******************************************************************************/

void CArray::CopyToTemporary( long index)
{
	ASSERT( usingTemporary == FALSE);
	
	usingTemporary = TRUE;
	
	BlockMove( ITEM_PTR( index), TMP_ITEM_PTR, elementSize);

}	/* CArray::CopyToTemporary */

/******************************************************************************
 CopyFromTemporary
 
 	Get an item from temporary storage
******************************************************************************/

void CArray::CopyFromTemporary( long index)
{
	ASSERT( usingTemporary == TRUE);

	usingTemporary = FALSE;
	
	BlockMove( TMP_ITEM_PTR, ITEM_PTR( index), elementSize);

}	/* CArray::CopyFromTemporay */

/******************************************************************************
 ItemOffset
 
 	Get the offset of an item into the item handle
******************************************************************************/

long CArray::ItemOffset( long itemIndex)
{
	ASSERT_INDEX( itemIndex);
	
	return ITEM_OFFSET( itemIndex);

}	/* CArray::ItemOffset */

/******************************************************************************
 Copy {OVERRIDE}
 
 	Make a copy of the array, including the item handle.
******************************************************************************/

CObject *CArray::Copy(void)
{
	CArray *theCopy;
	Handle	itemsCopy;
	OSErr	err;
	Boolean	savedAlloc;
	
	theCopy = (CArray*) inherited::Copy();
	
	TRY
	{
		itemsCopy = theCopy->hItems;
		
		savedAlloc = SetAllocation( kAllocCanFail);
		err = HandToHand( &itemsCopy);
		SetAllocation( savedAlloc);
		
		FailOSErr( err);
		
		theCopy->hItems = itemsCopy;
	}
	CATCH
	{
		// if copying item handle failed, then we should
		// kill the copy created by CObject::Copy. We
		// clear the item handle so disposing of the
		// copy doesn't dispose of this array's item handle
		
		theCopy->hItems = NULL;
		theCopy->Dispose();
	}
	ENDTRY;
	
	return theCopy;

}	/* CArray::Copy */
