/******************************************************************************
 CBureaucrat.c

									The Bureaucrat Class

		A bureaucrat is a link in the chain of command. It can perform tasks,
		notify its supervisor of accomplished tasks, or refer tasks to its
		supervisor (passing the buck).
	
	SUPERCLASS = CObject
	
	Copyright  1989 Symantec Corporation. All rights reserved.
	
	TCL 1.1 CHANGES
	[
		- CBureaucrat is now a subclass of CCollaborator
		- implemented BecomeGopher, and DoAppleEvent methods
		- implemented override of BroadcastChange
	]

 ******************************************************************************/

#include "Global.h"
#include "CBureaucrat.h"

/*** Global Variables ***/

extern CBureaucrat		*gGopher;		/* First in line to get commands	*/

/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 IBureaucrat

		Initialize a Bureaucrat object
 ******************************************************************************/

void	CBureaucrat::IBureaucrat(
	CBureaucrat		*aSupervisor)		/* Supervisor for the bureaucrat	*/
{
	CCollaborator::ICollaborator();
	itsSupervisor = aSupervisor;
}
	

/******************************************************************************
 Dispose {OVERRIDE}

		Dispose of a Bureaucrat
 ******************************************************************************/

void	CBureaucrat::Dispose()
{
	if (gGopher == this) {				/* Don't leave a dangling gopher	*/
		itsSupervisor->BecomeGopher( TRUE);
	}
		
	inherited::Dispose();				/* Pass message on to superclass	*/
}


/**** A C C E S S I N G   M E T H O D S ****/


/******************************************************************************
 GetSupervisor

		Return a reference to its boss in the chain of command
 ******************************************************************************/

CBureaucrat	*CBureaucrat::GetSupervisor()
{
	return(itsSupervisor);
}


/**** C O M M A N D I N G   M E T H O D S ****/
	

/******************************************************************************
 Notify

		A subordinate is notifying a bureaucrat that a task has been
		completed.
 ******************************************************************************/

void	CBureaucrat::Notify(
	CTask		*theTask)				/* The completed task				*/
{
										/* By default, a bureaucrat just	*/
	itsSupervisor->Notify(theTask);		/*   notifies its supervisor		*/
	
}
	

/******************************************************************************
 DoKeyDown

		Respond to a keystroke
 ******************************************************************************/

void	CBureaucrat::DoKeyDown(
	char			theChar,			/* The associated character			*/
	Byte			keyCode,			/* Code for the associated key		*/
	EventRecord		*macEvent)			/* Key down event record			*/
{
										/* Pass it on to the supervisor		*/
	itsSupervisor->DoKeyDown(theChar, keyCode, macEvent);
}
	

/******************************************************************************
 DoAutoKey

		Respond to a key being held down
 ******************************************************************************/

void	CBureaucrat::DoAutoKey(
	char			theChar,			/* The associated character			*/
	Byte			keyCode,			/* Code for the associated key		*/
	EventRecord		*macEvent)			/* Autokey event record				*/
{
										/* Pass it on to the supervisor		*/
	itsSupervisor->DoAutoKey(theChar, keyCode, macEvent);
}
	

/******************************************************************************
 DoKeyUp

		Respond to a key being released. Usually, these events will be
		masked out by the operating system.
 ******************************************************************************/

void	CBureaucrat::DoKeyUp(
	char			theChar,			/* The associated character			*/
	Byte			keyCode,			/* Code for the associated key		*/
	EventRecord		*macEvent)			/* Key up event record				*/
{
										/* Pass it on to the supervisor		*/
	itsSupervisor->DoKeyUp(theChar, keyCode, macEvent);
}
	

/******************************************************************************
 DoCommand

		Handle a command. Subclasses which respond to commands must
		override this method. Commands not recognized should be passed
		on to the inherited::DoCommand Method. Commands which move up
		the inheritance all the way to here get passed on to the
		Bureaucrat's supervisor.
		
		Sample DoCommand Method for a subclass:
		
		void	CMyBureaucrat::DoCommand(
			long		theCommand)
		{
			switch (theCommand) {
			
				case cmdForMe:
					[take some action ...]
					break;
					
				case cmdAnotherOneForMe:
					[do something ...]
					break;
					
				.
				.
				.
				
				default:
					inherited::DoCommand(theCommand);
					break;
			}
		}
 ******************************************************************************/

void	CBureaucrat::DoCommand(
	long		theCommand)				/* Command number					*/
{
										/* Pass it on to the supervisor		*/
	itsSupervisor->DoCommand(theCommand);
}
	

/******************************************************************************
 Dawdle

		Kill some time while the user isn't doing anything. Subclasses
		should override this method if they want to perform some
		repetitive active, such as twiddling their thumbs or flashing
		a text insertion cursor. If the Bureaucrat must regularly receive
		Dawdle() messages, the maxSleep parameter should be set to the
		maximum time, in ticks, between successive Dawdle() messages.
 ******************************************************************************/

void	CBureaucrat::Dawdle(
	long	*maxSleep)
{
}										/* Null Method						*/
	

/******************************************************************************
 UpdateMenus

		Perform menu management tasks
 ******************************************************************************/

void	CBureaucrat::UpdateMenus()
{
    
    itsSupervisor->UpdateMenus();   
}

/******************************************************************************
 BecomeGopher
 
 	A Bureaucrat is starting or finishing being the gopher. Returns TRUE
 	is the bureaucrat is willing to resign gopher or successfully
 	became the gopher.
******************************************************************************/

Boolean CBureaucrat::BecomeGopher( Boolean fBecoming)
{
	extern long gSleepTime;
	CBureaucrat	*prevGopher;

	gSleepTime = 0; 	/* force immediate idle */
	if (fBecoming)
	{
		if (gGopher != this)
		{
			prevGopher = gGopher;
			gGopher = this;
			
				// can't set new gopher if previous gopher
				// refuses to resign
				
 			if (!prevGopher->BecomeGopher( FALSE)) 
 			{
 				gGopher = prevGopher;
 				return FALSE; // failed
 			}
 			 			
			BroadcastChange( bureaucratIsGopher, 0);
		}
	}
	else
	{	
		BroadcastChange( bureaucratIsNotGopher, 0);
	}
	return TRUE; // success	
}

/******************************************************************************
 BroadcastChange
 
 	In addition to notifying dependents of change, Bureaucrats should
 	also notify their supervisors.
******************************************************************************/

void CBureaucrat::BroadcastChange( long reason, void* info)
{
	if (itsSupervisor)
		itsSupervisor->ProviderChanged( this, reason, info);
		
	inherited::BroadcastChange( reason, info);

}	/* CBureaucrat::BroadcastChange */

/******************************************************************************
 ProviderChanged
 
	By default, Bureaucrats pass change notifications to their supervisors
******************************************************************************/

void CBureaucrat::ProviderChanged( CCollaborator *aProvider, long reason, void* info)
{
	if (itsSupervisor)
		itsSupervisor->ProviderChanged( aProvider, reason, info);

}	/* CBureaucrat::ProviderChanged */

/******************************************************************************
 DoAppleEvent
 
	Pass the AppleEvent to the supervisor
******************************************************************************/

void CBureaucrat::DoAppleEvent( CAppleEvent *anAppleEvent)
{
	if (itsSupervisor)
		itsSupervisor->DoAppleEvent( anAppleEvent);

}	/* CBureaucrat::DoAppleEvent */