/******************************************************************************
 CClipboard.c

							The Clipboard Class
	
	Implements the "clipboard" used for copying and pasting information
	which is supported by almost all Mac applications. This class uses
	the global scrap and can store and retrieve PICT and TEXT data. Only
	one item may be stored in the Clipboard at a time. This class also
	supports a window which displays the information on the Clipboard.
	
	Hooks are provided for subclasses which can use a private scrap and
	store any application-specific information.
	
	SUPERCLASS = CDirector
	
	Copyright  1989 Symantec Corporation. All rights reserved.

	TCL 1.1 CHANGES
	[
		- added failure handling, use exceptions instead of gError
		- implemented EmptyGlobalScrap, EmptyScrap methods. PutGlobalScrap
		  no longer calls ZeroScrap, so you must call EmptyGlobalScrap
		  or EmptyScrap before calling PutGlobalScrap or PutData.
		- implemented MakeClipView method
	]
 ******************************************************************************/

#include "Global.h"
#include "Constants.h"
#include "Commands.h"
#include "CClipboard.h"
#include "CScrollPane.h"
#include "CPicture.h"
#include "CDesktop.h"
#include "CBartender.h"
#include "CEditText.h"
#include "CApplication.h"
#include "CWindow.h"
#include "CPanorama.h"


/*** Global Variables ***/

extern CDesktop		*gDesktop;			/* The visible Desktop				*/
extern CBartender	*gBartender;		/* Manages all menus				*/
extern CBureaucrat	*gGopher;			/* First in line to get commands	*/
extern Boolean		gInBackground;		/* In background under MultiFinder	*/


/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 IClipboard

		Initialize a Clipboard object
 ******************************************************************************/

void	CClipboard::IClipboard(
    CApplication    *aSupervisor,       /* Boss in chain of command         */
	Boolean			hasWindow)			/* Can contents be displayed?		*/
{
	CDirector::IDirector(aSupervisor);	/* Initialize superclass			*/

	if (hasWindow) {
		itsWindow = new(CWindow);
		itsWindow->IWindow(WIND_CLIPBOARD, FALSE, gDesktop, this);
		
		itsScrollPane = new(CScrollPane);
		itsScrollPane->IScrollPane(itsWindow, this, 0, 0, 0, 0,
								sizELASTIC, sizELASTIC, TRUE, TRUE, TRUE);
		itsScrollPane->FitToEnclFrame(TRUE, TRUE);
	}
	
    else
        itsScrollPane = NULL; 
    
	itsContents = NULL;
	theLength = 0;
	theOffset = 0;
	lastScrapCount = 0;
	privateNewer = FALSE;
	windowVisible = FALSE;
}


/******************************************************************************
 Suspend {OVERRIDE}

		Application is about to be swapped out under MultiFinder or a DA
		has become active. Hide the clipboard window to avoid displaying
		incorrect information, since the clipboard contents may change while
		we are away.
 ******************************************************************************/

void	CClipboard::Suspend()
{
										/* Copy private scrap to the global	*/
										/*   one if necessary				*/
	if (Status() == PRIVATE_SCRAP_NEWER) {
		ConvertPrivate();
		ScrapConverted();
	}
	
	if (windowVisible) {				/* Hide Clipboard window			*/
		itsWindow->HideSuspend();
		if (active) {
			itsWindow->Deactivate();
			active = TRUE;
		}
	}
}


/******************************************************************************
 Resume {OVERRIDE}

		Application is about to be swapped in under MultiFinder or after
		a DA was running
 ******************************************************************************/

void	CClipboard::Resume()
{
										/* If global scrap changed, we must	*/
										/*   update our private one if the	*/
										/*   Clipboard window is visible.	*/
										/*   No need to convert when the	*/
										/*   Clipboard window is hidden; we	*/
										/*   can wait for a Paste. This way	*/
										/*   we avoid having to convert if	*/
										/*   the user never uses it.		*/
	if ((Status() == GLOBAL_SCRAP_NEWER)  &&  windowVisible) {
		ConvertGlobal();
		ScrapConverted();
		UpdateDisplay();
	}
	
	if (windowVisible) {				/* Show window if it was visible	*/
		itsWindow->ShowResume();		/*   before we were suspended		*/
		
		if (active) {					/* Activate Clipboard window if it	*/
			itsWindow->Activate();		/*   was active before suspending	*/
			BecomeGopher( TRUE);
		}
	}
}


/******************************************************************************
 Close

		Close Clipboard by closing the clipboard window. Clipboard
		doesn't casre if application is quitting.
 ******************************************************************************/

Boolean		CClipboard::Close(
	Boolean			quitting)			/* Is this part of a quit sequence?	*/
{
	if (windowVisible) {
		if (gInBackground) {
			itsWindow->ShowResume();
		}
		CloseWind(itsWindow);			/* Close Clipboard window			*/
	}
	return(TRUE);						/* Clipboard never cancels a close	*/
}
		

/******************************************************************************
 CloseWind {OVERRIDE}

		The clipboard window needs to be closed, usually as a result of
		the user clicking in its close box or choosing the "Close" menu
		item. All we have to do is hide the window.
 ******************************************************************************/

void	CClipboard::CloseWind(
	CWindow		*theWindow)
{
	Str255		clipStatus;

	theWindow->Hide();					/* Hide but do NOT dispose			*/
	windowVisible = FALSE;
	Deactivate();
										/* Toggle text to "Hide Clipboard"	*/
	GetIndString(clipStatus, STRcommon, strSHOWCLIP);
	gBartender->SetCmdText(cmdToggleClip, clipStatus);
}
	

/******************************************************************************
 Toggle

		Toggle the visibility of the clipboard. If it is visible, hide it;
		if it is invisible, show it.
 ******************************************************************************/

void	CClipboard::Toggle()
{
	Str255		clipStatus;

	if (windowVisible) {				/* Hide the Clipboard window		*/
		CloseWind(itsWindow);
		
	} else {							/* Show the Clipboard window		*/
										/* Update private scrap if global	*/
										/*   is more recent					*/
		if (Status() == GLOBAL_SCRAP_NEWER) {
			ConvertGlobal();
			ScrapConverted();
		}
		itsWindow->Select();			/* Clipboard window is the front,	*/
		windowVisible = TRUE;			/*   active window					*/
		UpdateDisplay();

										/* Toggle text to "Show Clipboard"	*/
		GetIndString(clipStatus, STRcommon, strHIDECLIP);
		gBartender->SetCmdText(cmdToggleClip, clipStatus);
	}
}
	

/******************************************************************************
 PutGlobalScrap

		Place data on the global desk scrap
 ******************************************************************************/

void	CClipboard::PutGlobalScrap(
	ResType		theType,				/* Resource type of the data		*/
	Handle		theData)
{
	SignedByte		saveHState;			/* Original handle attributes		*/
	register long	errCode;			/* Mac error code					*/

	saveHState = HGetState(theData);	/* Save state of data handle		*/
	HLock(theData);						/* Lock data in place				*/
	
	errCode = PutScrap(GetHandleSize(theData), theType, *theData);
	
	HSetState(theData, saveHState);		/* Restore state of data handle		*/
	
	FailOSErr( errCode);
	
		/* In the event that PutGlobalScrap is being called without			*/
		/* a user cut or copy, we need to call SystemEdit so the			*/
		/* clipboard works properly.										*/
		
	SystemEdit( cmdCopy - cmdUndo);
}
	

/******************************************************************************
 GetGlobalScrap

		Retrieve data from the global desk scrap. Return
		FALSE if no data is found.
 ******************************************************************************/

Boolean	CClipboard::GetGlobalScrap(
	ResType		theType,				/* Resource type of the data		*/
	Handle		theData)
{
	long		length;					/* Amount of data or error code		*/
	long		offset;					/* Offset from start of scrap		*/

										/* Read from the desk scrap			*/
	length = GetScrap(theData, theType, &offset);
	
	theLength = length;					/* Save in case someone wants this	*/
	theOffset = offset;					/*   information					*/
	
	if (length < 0) {					/* Check for an error				*/
										/* Finding no data of the specified	*/
										/*   type does not warrant an error	*/
										/*   message						*/
		if (length != noTypeErr) {
			FailOSErr( length);
		}
		return(FALSE);					/* No data found					*/
	} else
		return(TRUE);					/* Data successfully retrieved		*/
}
	

/******************************************************************************
 Status

		Return the status of the clipboard by determining whether the
		global or private scrap is more recent
 ******************************************************************************/

ScrapStatus		CClipboard::Status()
{
	PScrapStuff	theScrapInfo;

	if (privateNewer) {					/* Private scrap has changed		*/
		return(PRIVATE_SCRAP_NEWER);
		
	} else {
		theScrapInfo = InfoScrap();		/* Get info about global scrap		*/
		if (theScrapInfo->scrapCount > lastScrapCount)
			return(GLOBAL_SCRAP_NEWER);
		else
			return(SCRAPS_THE_SAME);
	}
}
	

/******************************************************************************
 ScrapConverted

		Set flags after the scrap has been converted
 ******************************************************************************/

void	CClipboard::ScrapConverted()
{
	PScrapStuff	theScrapInfo;
	
	theScrapInfo = InfoScrap();			/* Get info about global scrap		*/
	lastScrapCount = theScrapInfo->scrapCount;
	privateNewer = FALSE;
}


/******************************************************************************
 ConvertGlobal

		Take data from the global desk scrap and put it in our private
		scrap, performing any necessary conversions. Subclasses which
		support a private scrap must override this method.
 ******************************************************************************/

void	CClipboard::ConvertGlobal()
{
}										/* Null Method						*/


/******************************************************************************
 ConvertPrivate

		Place information from our private scrap into the global desk scrap,
		performing any necessary conversions. Standard types for the
		global desk scrap are 'PICT' and 'TEXT'. Subclasses which support
		a private scrap must override this method.
 ******************************************************************************/

void	CClipboard::ConvertPrivate()
{
}										/* Null Method						*/


/******************************************************************************
 PutData

		Place information on our scrap. This method uses the global desk
		scrap. Subclasses must override this method to implement a
		private scrap.
		
		NOTE: Subclasses overriding this method should invoke the
				PrivateChanged method after placing new data in the
				private scrap.
 ******************************************************************************/

void	CClipboard::PutData(
	ResType		theType,				/* Resource type of the data		*/
	Handle		theData)
{
	PutGlobalScrap(theType, theData);
	PrivateChanged();
}


/******************************************************************************
 GetData

		Retrieve information from our scrap. This method uses the global desk
		scrap. If successful, function returns TRUE and passes back a
		newly created handle to the data retrieved. Function returns FALSE
		if no data was retrieved. Subclasses must override this method to
		implement a private scrap.
 ******************************************************************************/

Boolean	CClipboard::GetData(
	ResType				theType,		/* Resource type of the data		*/
	register Handle		*theData)
{
	*theData = NewHandle(0);			/* Create space for the data		*/

	if (GetGlobalScrap(theType, *theData)) {
		return(TRUE);					/* Data successfully retrieved		*/
		
	} else {							/* Some kind of error occurred		*/
		DisposHandle(*theData);			/* Kill the data handle				*/
		*theData = NULL;
		return(FALSE);					/* Signal failure					*/
	}
}


/******************************************************************************
 DataSize

		Retrieve the length of the data of the specified type which is in
		the clipboard. Zero (0) is returned if there is no such data. This
		method uses the global desk scrap. Subclasses must override this
		method to implement a private scrap.
 ******************************************************************************/

long	CClipboard::DataSize(
	ResType				theType)		/* Resource type of the data		*/
{
	if (GetGlobalScrap(theType, NULL)) {
		return(theLength);
	
	} else {
		return(0L);
	}
}


/******************************************************************************
 PrivateChanged

		Private scrap has changed. Set flag and update contents of the
		clipboard window, if visible.
 ******************************************************************************/

void	CClipboard::PrivateChanged()
{
	privateNewer = TRUE;
	UpdateDisplay();
}


/******************************************************************************
 UpdateDisplay

		Read from the clipboard and set up a pane to display the contents.
		Try to find a 'PICT' first, then 'TEXT'. Give up if neither of
		these resource types is present.
 ******************************************************************************/

void	CClipboard::UpdateDisplay()
{
	Handle		theData;
	long		theType;
	
	if (windowVisible) {
		if (itsContents != NULL) {
			itsContents->Refresh();
			itsContents->Dispose();
			itsContents = NULL;
			itsScrollPane->itsPanorama = NULL;
		}
		
		if (GetData('PICT', &theData))
			theType = 'PICT';
		else if (GetData('TEXT', &theData))
			theType = 'TEXT';
			
		if (theData)
		{
			itsContents = MakeClipView( theType, theData);
			if (itsContents)
			{
				itsScrollPane->InstallPanorama( itsContents);
				itsContents->Refresh();
			}
		}
	}
}

/******************************************************************************
 MakeClipView
 
	Make a view corresponding to the given clipboard data type. MakeClipView
	is responsible for disposing the dataHandle when appropriate. Override
	this method to make views for different types of data.
 ******************************************************************************/

CPanorama* CClipboard::MakeClipView( long dataType, Handle dataHandle)
{
	Rect		margin;
	CPanorama 	*clipView = NULL;
	
	TRY
	{
		if (dataType == 'PICT') 
		{
			CPicture *pict = new(CPicture);
			clipView = pict;
			
			pict->IPicture(itsScrollPane, this,
											1, 1, 0, 0, sizELASTIC, sizELASTIC);
			pict->SetMacPicture((PicHandle) dataHandle);
			pict->FitToEnclosure(TRUE, TRUE);
			
		} 
		else if (dataType == 'TEXT') 
		{
			CEditText *text = new( CEditText);
			clipView = text;
			
			text->IEditText(itsScrollPane, this,
						1, 1, 0, 0, sizELASTIC, sizELASTIC, 1000);
			text->Specify( kNotEditable, kNotSelectable, kNotStylable);
			text->FitToEnclosure(TRUE, TRUE);
			SetRect(&margin, 2, 2, -2, -2);
			text->ChangeSize( &margin, FALSE);
			text->SetTextHandle( dataHandle);
			DisposHandle( dataHandle);  /* CEditText makes its own copy. */
		}
		if (!clipView)
			ForgetHandle( dataHandle);	/* we didn't make a view, kill the data */
	}
	CATCH
	{
		ForgetObject( clipView);
		ForgetHandle( dataHandle);
	}
	ENDTRY;
	
	return clipView;
}

/******************************************************************************
 EmptyGlobalScrap

	Clear the current contents of the global scrap.
 ******************************************************************************/

void CClipboard::EmptyGlobalScrap(void)
{
	FailOSErr( ZeroScrap());
}

/******************************************************************************
 EmptyScrap

	Clear the current contents of the scrap that is the destination of
	PutData. For subclasses with a private scrap, this could clear the 
	private scrap. This method clears the global scrap. You should call this
	method before putting the contents of a cut or copy to the clipboard
 ******************************************************************************/

void CClipboard::EmptyScrap(void)
{
	EmptyGlobalScrap();
}

