/******************************************************************************
 CCollaborator.c

		Abstract class that represents objects that collaborate
		with other objects. A Collaborator maintains a list of provider
		objects and a list of dependent objects. When a Collaborator
		changes, it sends itself the BroadcastChange message. Each
		dependent then receives the ProviderChanged message.
		
	SUPERCLASS = CObject 
	
	Copyright  1991 Symantec Corporation. All rights reserved.
	

 ******************************************************************************/

#include "CCollaborator.h"
#include "CList.h"
#include "Global.h"

/******************************************************************************
 ICollaborator
 
 	Initialize a Collaborator
 	
******************************************************************************/

void CCollaborator::ICollaborator( void)
{
	itsProviders = NULL;
	itsDependents = NULL;
	
}	/* CCollaborator::ICollaborator */

/******************************************************************************
 DependUpon
 
 	Initiate a dependency by adding a provider to this object's provider list,
 	and adding this object to the provider's dependent list.
 	
******************************************************************************/

void CCollaborator::DependUpon( CCollaborator *aProvider)
{
	AddProvider( aProvider);
	aProvider->AddDependent( this);

}	/* CCollaborator::DependUpon */


/******************************************************************************
 CancelDependency
 
 	Terminate a dependency by removing a provider from this object's
 	provider list, and removing this object from the provider's 
 	dependent list.
******************************************************************************/

void CCollaborator::CancelDependency( CCollaborator *aProvider)
{
	RemoveProvider( aProvider);
	aProvider->RemoveDependent( this);

}	/* CCollaborator::CancelDependency */

/******************************************************************************
 AddDependant
 
 	Add aDependent to the dependent list. Called by DependUpon.
******************************************************************************/

void CCollaborator::AddDependent( CCollaborator *aDependent)
{
	if (!itsDependents)
	{
		itsDependents = new( CList);
		itsDependents->IList();
	}
	itsDependents->Append( aDependent);

}	/* CCollaborator::AddDependant */

/******************************************************************************
 RemoveDependant
 
 	Remove a dependent from the dependency list. The dependent will cease
 	receiving ProviderChanged messages. Called by CancelDependency
******************************************************************************/

void CCollaborator::RemoveDependent( CCollaborator *aDependent)
{
	if (itsDependents)
		itsDependents->Remove( aDependent);

}	/* CCollaborator::RemoveDependant */

/******************************************************************************
 AddProvider
 
 	Add aProvider to a Collaborators provider list. Called by DependUpon
******************************************************************************/

void CCollaborator::AddProvider( CCollaborator *aProvider)
{
	if (!itsProviders)
	{
		itsProviders = new( CList);
		itsProviders->IList();
	}
	itsProviders->Append( aProvider);

}	/* CCollaborator::AddProvider */

/******************************************************************************
 RemoveProvider
 
 	Remove aProvider from a Collaborators provider list. Called by 
 	CancelDependency
******************************************************************************/

void CCollaborator::RemoveProvider( CCollaborator *aProvider)
{
	if (itsProviders)
		itsProviders->Remove( aProvider);

}	/* CCollaborator::RemoveProvider */

/******************************************************************************
 BroadcastChange
 
 	Notify all dependents that a provider has changed. It is the responsibility
 	of subclasses to implement a change protocol useful to dependents. The
 	reason parameter should enumerate the types of changes, and the info parameter
 	can be a pointer to any additional information needed to describe the change.
 	
******************************************************************************/

	typedef struct tUpdateInfo
	{
		CCollaborator	*provider;
		short			reason;
		void			*info;
		
	}	tUpdateInfo;

	static void CCollaborator::Dependent_Update( CObject *aDependent, long infoAddr)
	{
		tUpdateInfo *updateInfo = (tUpdateInfo*) infoAddr;
		
		((CCollaborator*) aDependent)->ProviderChanged(
				updateInfo->provider,
				updateInfo->reason,
				updateInfo->info);
	}

void CCollaborator::BroadcastChange( long reason, void* info)
{
	tUpdateInfo updateInfo;
	long		nullInfo;
	
	/* We allow the senders of BroadcastChange to pass NULL for info	*/
	/* However, assuming dereferencing NULL is bad, it easier for		*/
	/* receivers of ProviderChanged if dereferencing info is always 	*/
	/* safe. Therefore, when info is NULL, we substitute of pointer		*/
	/* to 0L															*/
	
	if (itsDependents)
	{
		if (!info)
		{
			nullInfo = 0L;
			info = &nullInfo;
		}
		
		updateInfo.provider = this;
		updateInfo.reason = reason;
		updateInfo.info = info;
		
		itsDependents->DoForEach1((EachFunc1) CCollaborator::Dependent_Update, 
						(long) &updateInfo);
	}

}	/* CCollaborator::BroadcastChange */

/******************************************************************************
 ProviderChanged
 
 	Respond to the provider change.
 	
******************************************************************************/

void CCollaborator::ProviderChanged( CCollaborator *aProvider, long reason, void* info)

{	
    	 /* null method, subclasses override */

}	/* CCollaborator::ProviderChanged */

/******************************************************************************
 Dispose {OVERRIDE}
 
 	Override Dispose  in order to remove references to this object from any
 	other provider or dependency lists.
 	
******************************************************************************/

	static void CCollaborator::Dependent_Remove( CObject *aDependent, long aProvider)
	{
		((CCollaborator*) aDependent)->RemoveProvider( (CCollaborator*) aProvider);
	}
	static void CCollaborator::Provider_Remove( CObject *aProvider, long aDependent)
	{
		((CCollaborator*) aProvider)->RemoveDependent( (CCollaborator*) aDependent);
	}

void CCollaborator::Dispose( void)
{
	if (itsDependents)
	{
		itsDependents->DoForEach1((EachFunc1) Dependent_Remove, (long) this);
		ForgetObject( itsDependents);
	}
	if (itsProviders)
	{
		itsProviders->DoForEach1((EachFunc1) Provider_Remove, (long) this);
		ForgetObject( itsProviders);
	}
	inherited::Dispose();

}	/* CCollaborator::Dispose */