/******************************************************************************
 CList.c

							The List Class
	
	A linear list of objects.
	
	SUPERCLASS = CCluster
	
	Copyright  1989 Symantec Corporation. All rights reserved.

	TCL 1.1 CHANGES
	[
		- because CList is now a subclass of CArray the implementations
		  of Prepend, InsertAfter, InsertAt, MoveUp, MoveDown, SendBack,
		  MoveToIndex now bottleneck through InsertAtIndex and 
		  MoveItemToIndex.
		- removed the Remove method.
	]
 ******************************************************************************/

#include "Global.h"
#include "CList.h"


/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 IList

		Initialize a List object
 ******************************************************************************/

void	CList::IList()
{
	CCluster::ICluster();				/* Initialize superclass			*/
}
	

/******************************************************************************
 Append

		Add an item to the end of a list
 ******************************************************************************/

void	CList::Append(
	CObject 	*theObject)
{
	CCluster::Add(theObject);
}
	

/******************************************************************************
 Prepend

		Add an item to the beginning of a list
 ******************************************************************************/

void	CList::Prepend(
	CObject 	*theObject)
{
	InsertAtIndex( &theObject, 1);
}
	

/******************************************************************************
 InsertAfter

		Add an item to a list at the position after a specified item
 ******************************************************************************/

void	CList::InsertAfter(
	CObject		*theObject,				/* Object to insert in the list		*/
	CObject		*afterObject)			/* Object to insert it after		*/
{
	long	afterOffset;
	
	afterOffset = Offset(afterObject);	/* Find location of afterObject		*/

	if (afterOffset == BAD_INDEX)
		afterOffset = 0;
		
	InsertAtIndex( &theObject, afterOffset + 2);
}
	

/******************************************************************************
 InsertAt

		Add an item to a list at the specified position
 ******************************************************************************/

void	CList::InsertAt(
	CObject				*theObject,
	register long		index)
{
	InsertAtIndex( &theObject, index);
}
	
	

/******************************************************************************
 BringFront

		Bring an item to the front of a list. Item must already be in the list.
 ******************************************************************************/

void	CList::BringFront(
	CObject 	*theObject)				/* Object to move to the front		*/
{
	register long		i;				/* Index into list					*/
	
	i = Offset(theObject);				/* Find location of the object		*/
										
	if (i > 0) {						/* If it's not already in front ...	*/
	
		MoveItemToIndex( i+1, 1);
	}
}
	

/******************************************************************************
 SendBack

		Send an item to the back of a list. Item must already be in the list.
 ******************************************************************************/

void	CList::SendBack(
	CObject 	*theObject)				/* Object to move to the back		*/
{
	register long	i;					/* Index into list					*/
	
	i = Offset(theObject);				/* Find location of the object		*/
										
	if ((i >= 0) && (i < numItems - 1)) {				
										/* If it's not already in back ...	*/

		MoveItemToIndex( i + 1, numItems);
	}
}
	

/******************************************************************************
 MoveUp

		Move an item up (closer to the front) one position in the list.
 ******************************************************************************/

void	CList::MoveUp(
	CObject		*theObject)				/* Object to move up in list		*/
{
	long	i;					/* Index into list					*/
	
	i = Offset(theObject);				/* Find location of object			*/
										
	if (i > 0) {						/* If it's not at the front ...		*/
	
		MoveItemToIndex( i+1, i);
	}
}
	

/******************************************************************************
 MoveDown

		Move an item down (closer to the back) one position in the list.
 ******************************************************************************/

void	CList::MoveDown(
	CObject		*theObject)				/* Object to move down				*/
{
	long	i;					/* Index into list					*/
	
	i = Offset(theObject);				/* Find locaton of object			*/
                                                            
    if (i >= 0  &&  i < numItems-1) {   /* If it's not at the back ...      */
    
    	MoveItemToIndex( i+1, i + 2);
	}
}
	

/******************************************************************************
 MoveToIndex

		Move an item to a specified position in the list, shifting
		other items in the list appropriately.
 ******************************************************************************/

void	CList::MoveToIndex(
	CObject				*theObject,		/* Object to move within the list	*/
	register long		index)			/* Index at which to move object	*/
{
	long	i;							/* Index into the list				*/
	
	i = Offset(theObject);				/* Find location of the object		*/
	
	if (i != BAD_INDEX)
	{
		MoveItemToIndex( i + 1, index);
	}
}
	

/******************************************************************************
 FirstItem

		Return the first item in the list
 ******************************************************************************/

CObject*	CList::FirstItem()
{
	return(NthItem(1));
}


/******************************************************************************
 LastItem

		Return the last item in the list
 ******************************************************************************/

CObject*	CList::LastItem()
{
	return(NthItem(numItems));
}


/******************************************************************************
 NthItem

		Return the Nth item in the list
 ******************************************************************************/

CObject*	CList::NthItem(
	long		n)
{
	if (n < 1  ||  n > numItems)		/* If index is out of range			*/
		return(NULL);					/*   pass back NULL					*/
	else
										/* Do direct indexing into list		*/
		return( (CObject*) (*items)[n-1]);
}


/******************************************************************************
 FindIndex

		Return the index number in the list of a specified object. Index
		ranges from 1 to numItems.
 ******************************************************************************/

long	CList::FindIndex(
	CObject		*theObject)
{
	return(Offset(theObject) + 1);		/* Index is one greater than the	*/
										/*   offset, which is zero-based	*/
}


/******************************************************************************
 FirstSuccess

		Find the first item in the list which successfully passes (returns
		TRUE) the test function. Return NULL if no item passes the test.
 ******************************************************************************/

CObject*	CList::FirstSuccess(
	TestFunc		testFunc)			/* Ptr to test function				*/
{
	return inherited::FindItem(testFunc);
}


/******************************************************************************
 FirstSuccess1

		Find the first item in the list which successfully passes (returns
		TRUE) the test function. Return NULL if no item passes the test.
 ******************************************************************************/

CObject*	CList::FirstSuccess1(
	TestFunc1		testFunc,			/* Ptr to test function				*/
	long			param)				/* Parameter for test function		*/
{
	return inherited::FindItem1(testFunc, param);
}


/******************************************************************************
 LastSuccess

		Find the last item in the list which successfully passes (returns
		TRUE) the test function. Return NULL if no item passes the test.
 ******************************************************************************/

CObject*	CList::LastSuccess(
	TestFunc		testFunc)			/* Ptr to test function				*/
{
	register long	i;					/* Index into list of items			*/
	CObject  *item = NULL;
	Boolean	 wasLocked;
	
	TRY
	{
		wasLocked = SetLockChanges( TRUE);
		
		for (i = numItems-1; i >= 0; i--) 	/* Scan from last item to first		*/
		{	
			if ( (*testFunc)((CObject*) (*items)[i]) )
			{								/* Success! Pass back the object	*/
				item = (CObject*)(*items)[i];
				break;
			}
		}
		
		SetLockChanges( wasLocked);
	}
	CATCH
	{
		SetLockChanges( wasLocked);
	}
	ENDTRY;
	
	return item;

}


/******************************************************************************
 LastSuccess1

		Find the last item in the list which successfully passes (returns
		TRUE) the test function. Return NULL if no item passes the test.
 ******************************************************************************/

CObject*	CList::LastSuccess1(
	TestFunc1		testFunc,			/* Ptr to test function				*/
	long			param)				/* Parameter for test function		*/
{
	register long	i;					/* Index into list of items			*/
	CObject	*item = NULL;
	Boolean wasLocked;
	
	TRY
	{
		wasLocked = SetLockChanges( TRUE);
		
		for (i = numItems-1; i >= 0; i--) 
		{									/* Scan from last item to first		*/
			if ( (*testFunc)((CObject*)  (*items)[i], param) )
			{
											/* Success! Pass back the object	*/
				item = (CObject*)(*items)[i];
				break;
			}
		}
		
		SetLockChanges( wasLocked);
	}
	CATCH
	{
		SetLockChanges( wasLocked);
	}
	ENDTRY;
	
	return item;

}
