/******************************************************************************
 CScrollPane.c

							The ScrollPane Class
		
		A pane for managing a Panorama and a horizontal and/or vertical
		scroll bar. Handles all communication between a Panorama and
		scroll bars so that they are always in synch. ScrollPane tells
		the Panorama how much to scroll when there are clicks in a
		scroll bar and adjust the scroll bars when the Panorama changes.
	
	SUPERCLASS = CPane
	
	Copyright  1989 Symantec Corporation. All rights reserved.
	
	TCL 1.1 CHANGES
	[
		- implemented DoScroll method
	]

 ******************************************************************************/

#include "CScrollPane.h"
#include "Global.h"
#include "Constants.h"
#include "CScrollBar.h"
#include "CSizeBox.h"	
#include "CPanorama.h"


/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 IScrollPane

		Initialize a ScrollPane object
 ******************************************************************************/

void	CScrollPane::IScrollPane(
	CView			*anEnclosure,
	CBureaucrat		*aSupervisor,
	short			aWidth,
	short			aHeight,
	short			aHEncl,
	short			aVEncl,
	SizingOption	aHSizing,
	SizingOption	aVSizing,
	Boolean			hasHoriz,
	Boolean			hasVert,
	Boolean			hasSizeBox)
{	
	CPane::IPane(anEnclosure, aSupervisor,
					aWidth, aHeight, aHEncl, aVEncl, aHSizing, aVSizing);
	
	hStep = vStep = 1;
	hOverlap = vOverlap = 1;
	hUnit = vUnit = 1;
	
	IScrollPaneX(hasHoriz, hasVert, hasSizeBox);
}
	

/******************************************************************************
 IViewTemp {OVERRIDE}

		Initialize a ScrollPane using a template
 ******************************************************************************/

void	CScrollPane::IViewTemp(
	CView			*anEnclosure,		/* Enclosing view					*/
	CBureaucrat		*aSupervisor,		/* Boss in chain of command			*/
	Ptr				viewData)			/* Template with View data			*/
{
	register ScrollPaneTempP	p;
	
	p = (ScrollPaneTempP) viewData;		/* Initialize Superclass			*/
	inherited::IViewTemp(anEnclosure, aSupervisor, (Ptr) &p->sPaneTemp);
	
	hStep = p->hStep;					/* Set instance vars from template	*/
	vStep = p->vStep;
	hOverlap = p->hOverlap;
	vOverlap = p->vOverlap;
										/* Extra initialization				*/
	IScrollPaneX(p->hasHoriz != 0, p->hasVert != 0, p->hasSizeBox != 0);
}
	

/******************************************************************************
 IScrollPaneX

		Extra initialization
 ******************************************************************************/

void	CScrollPane::IScrollPaneX(
	Boolean		hasHoriz,
	Boolean		hasVert,
	Boolean		hasSizeBox)
{
	short			shrinkage;
	
	wantsClicks = TRUE;
	
	shrinkage = (hasSizeBox || (hasHoriz && hasVert)) ? SBARSIZE1 : 0;
	
	if (hasHoriz) {
		itsHorizSBar = new(CScrollBar);
		itsHorizSBar->IScrollBar(this, this, HORIZONTAL, width - shrinkage,
									frame.left, frame.bottom - SBARSIZE);
		itsHorizSBar->SetActionProc((VoidFunc) SBarActionProc);
		itsHorizSBar->SetThumbFunc((VoidFunc) SBarThumbFunc);
	} else {
		itsHorizSBar = NULL;
	}
	
	if (hasVert) {
		itsVertSBar = new(CScrollBar);
		itsVertSBar->IScrollBar(this, this, VERTICAL, height - shrinkage,
									frame.right - SBARSIZE, frame.top);
		itsVertSBar->SetActionProc((VoidFunc) SBarActionProc);
		itsVertSBar->SetThumbFunc((VoidFunc) SBarThumbFunc);
	} else {
		itsVertSBar = NULL;
	}
	
	if (hasSizeBox) {
		itsSizeBox = new(CSizeBox);
		itsSizeBox->ISizeBox(this, this);
	} else {
		itsSizeBox = NULL;
	}
	
	itsPanorama = NULL;
}
	

/**** A C C E S S I N G   M E T H O D S ****/


/******************************************************************************
 InstallPanorama

		Install a Panorama within a ScrollPane
 ******************************************************************************/

void	CScrollPane::InstallPanorama(
	CPanorama	*aPanorama)
{
	itsPanorama = aPanorama;
	itsPanorama->SetScrollPane(this);
	AdjustScrollMax();
	Calibrate();
}
	

/******************************************************************************
 SetSteps

		Set the horizontal and vertical steps for scrolling when clicking
		in one of the arrows at the end of a scroll bar. Normally, a
		step is one unit, where a unit is defined by the Panorama.
 ******************************************************************************/

void	CScrollPane::SetSteps(
	short		aHStep,
	short		aVStep)
{
	hStep = aHStep;						/* Set instance variables			*/
	vStep = aVStep;
}
	

/******************************************************************************
 GetSteps

		Get the horizontal and vertical steps for scrolling when clicking
		in one of the arrows at the end of a scroll bar.
 ******************************************************************************/

void	CScrollPane::GetSteps(
	short		*theHStep,
	short		*theVStep)
{
	*theHStep = hStep;					/* Get instance variables			*/
	*theVStep = vStep;
}
	

/******************************************************************************
 SetOverlaps

		Set the horizontal and vertical overlaps for page scrolling. The
		overlap determines how much "context" remains when scrolling by
		a screenful. Normally, the overlap is one unit, where a unit is
		defined by the Panorama.
 ******************************************************************************/

void	CScrollPane::SetOverlaps(
	short		aHOverlap,
	short		aVOverlap)
{
	hOverlap = aHOverlap;				/* Set instance variables			*/
	vOverlap = aVOverlap;
}


/******************************************************************************
 GetInterior {OVERRIDE}

		Retrieve the interior of a ScrollPane. Interior excludes the area
		of the frame occupied by the scroll bars.
 ******************************************************************************/

void	CScrollPane::GetInterior(
	LongRect	*theInterior)
{
	*theInterior = frame;				/* Start off with the frame			*/
										/* then provide a 1-pixel thick		*/
										/* border							*/
	InsetLongRect( theInterior, 1, 1);

	if (itsHorizSBar != NULL) {			/* Subtract horizontal scroll bar	*/
		theInterior->bottom -= SBARSIZE1;
	}
	
	if (itsVertSBar != NULL) {			/* Subtract vertical scroll bar		*/
		theInterior->right -= SBARSIZE1;
	}
}


/**** S C R O L L   B A R   M A I N T E N A N C E   M E T H O D S ****/


/******************************************************************************
 AdjustScrollMax

		Adjust the maximum value of the scroll bars to conform to the
		extent and frame size of the Panorama
 ******************************************************************************/

void	CScrollPane::AdjustScrollMax()
{
	register short	newMax;				/* New maximum scroll bar setting	*/
	long			hPos;
	long			vPos;
	short			hFSpan;
	short			vFSpan;
	
	if (itsPanorama == NULL)			/* Take a quick exit if no			*/
		return;							/*   Panorama is installed			*/
	
	itsPanorama->GetExtent(&hPos, &vPos);
	hExtent = hPos;
	vExtent = vPos;
	
	itsPanorama->GetFrameSpan(&hFSpan, &vFSpan);
	hSpan = hFSpan;
	vSpan = vFSpan;
	
	hUnit = hExtent / MAXINT + 1;		/* Controls are limited to shorts	*/
	vUnit = vExtent / MAXINT + 1;		/* Need to scale in case extent is	*/
										/*   larger than a short			*/
	hStep = Max( hUnit, hStep);
	vStep = Max( vUnit, vStep);

	itsPanorama->GetFramePosition(&hPos, &vPos);

	if (itsHorizSBar != NULL) {
		itsHorizSBar->SetMaxValue(Max((hExtent - hSpan), hPos) / hUnit);
	}

	if (itsVertSBar != NULL) {
		itsVertSBar->SetMaxValue(Max((vExtent - vSpan), vPos) / vUnit);
	}
}


/******************************************************************************
 Calibrate

		Adjust scroll bar setting when the position of the frame of the
		ScrollPane's panorama changes
 ******************************************************************************/

void	CScrollPane::Calibrate()
{
	long	hPos;
	long	vPos;
	
	if (itsPanorama == NULL)			/* Take a quick exit if no			*/
		return;							/*   Panorama is installed			*/

	itsPanorama->GetFramePosition(&hPos, &vPos);
	
	if (itsHorizSBar != NULL) {
		itsHorizSBar->SetValue((short) (hPos / hUnit));
	}
	
	if (itsVertSBar != NULL) {
		itsVertSBar->SetValue((short) (vPos / vUnit));
	}
}


/******************************************************************************
 ChangeSize {OVERRIDE}

		Change the size of a ScrollPane
 ******************************************************************************/

void	CScrollPane::ChangeSize(
	Rect		*delta,
	Boolean		redraw)
{
	inherited::ChangeSize(delta, redraw);
	AdjustScrollMax();
	Calibrate();
}


/**** S C R O L L   P E R F O R M A N C E   M E T H O D S ****/


/******************************************************************************
 DoHorizScroll

		Horizontally scroll the Panorama associated with a ScrollPane
 ******************************************************************************/

void	CScrollPane::DoHorizScroll(
	short		whichPart)				/* Part of scroll bar clicked in	*/
{
	register long		delta;			/* Number of pixels to scroll		*/
	register long		minmax;			/* Minimum or Maximum delta			*/
	long				ticks;			/* Tick count at end of Delay		*/
	LongPt	currPos;

	switch (whichPart) {
	
		case inUpButton:
			delta = -hStep;
			break;
			
		case inDownButton:
			delta = hStep;
			break;
			
		case inPageUp:
			Delay(PAGE_DELAY, &ticks);
			delta = Min( hOverlap - hSpan, -1);
			break;
			
		case inPageDown:
			Delay(PAGE_DELAY, &ticks);
			delta = Max( hSpan - hOverlap, 1);
			break;
	}
	
	itsPanorama->GetFramePosition( &currPos.h, &currPos.v);
	if (delta < 0)
	{
		minmax = -currPos.h;
		delta = Max( delta, minmax);
	}
	else
	{
		minmax = hExtent - currPos.h - (long) hSpan;
		delta = Min( delta, minmax);
	}
	if (delta != 0)
	{
		DoScroll( delta, 0);
		AdjustScrollMax();
		Calibrate();
		itsHorizSBar->Prepare();
	}
}
	

/******************************************************************************
 DoVertScroll

		Vertically scroll the Panorama associated with a ScrollPane
 ******************************************************************************/

void	CScrollPane::DoVertScroll(
	short		whichPart)				/* Part of scroll bar clicked in	*/
{
	register long		delta;			/* Number of pixels to scroll		*/
	register long		minmax;			/* Minimum or Maximum delta			*/
	long				ticks;			/* Tick count at end of Delay		*/
	LongPt	currPos;

	switch (whichPart) {
	
		case inUpButton:
			delta = -vStep;
			break;
			
		case inDownButton:
			delta = vStep;
			break;
			
		case inPageUp:
			Delay(PAGE_DELAY, &ticks);
			delta = Min( vOverlap - vSpan, -1);
			break;
			
		case inPageDown:
			Delay(PAGE_DELAY, &ticks);
			delta = Max( vSpan - vOverlap, 1);
			break;
	}
	
	itsPanorama->GetFramePosition( &currPos.h, &currPos.v);
	if (delta < 0)
	{
		minmax = -currPos.v;
		delta = Max( delta, minmax);
	}
	else
	{
		minmax = vExtent - currPos.v - (long) vSpan;
		delta = Min( delta, minmax);
	}
	if (delta != 0)
	{
		DoScroll(0, delta);
		AdjustScrollMax();
		Calibrate();
		itsVertSBar->Prepare();
	}
}
	

/******************************************************************************
 DoThumbDrag

		Adjust associated Panorama when the thumb of a scroll bar is dragged
 ******************************************************************************/

void	CScrollPane::DoThumbDrag(
	short		hDelta,
	short		vDelta)
{
	long	hPixels = 0;
	long	vPixels = 0;
	LongPt	currPos, newPos;
	
	/* Because of the scaling of control values when using long coordinates	*/
	/* dragging the thumb to the beginning or end of the scroll bar will	*/
	/* sometimes not fully scroll to the beginning or end. Catch this by	*/
	/* checking if the current value is the max or min.						*/
	
	itsPanorama->GetFramePosition( &currPos.h, &currPos.v);
	
	if (hDelta && itsHorizSBar)
	{
		if (itsHorizSBar->GetValue() == itsHorizSBar->GetMinValue())
		{
			itsPanorama->GetHomePosition( &newPos);
			hPixels = newPos.h - currPos.h;
		}
		else if (itsHorizSBar->GetValue() == itsHorizSBar->GetMaxValue())
		{
			hPixels = Max(0, hExtent - hSpan - currPos.h);
		}
		else
			hPixels = ((long)hDelta) * hUnit;
	}
 	if (vDelta && itsVertSBar)
	{
		if (itsVertSBar->GetValue() == itsVertSBar->GetMinValue())
		{
			itsPanorama->GetHomePosition( &newPos);
			vPixels = newPos.v - currPos.v;
		}
		else if (itsVertSBar->GetValue() == itsVertSBar->GetMaxValue())
		{
			vPixels = Max( 0, vExtent - vSpan - currPos.v);
		}
		else
			vPixels = ((long)vDelta) * vUnit;
	}
 	if (hPixels || vPixels)
		DoScroll( hPixels, vPixels);
}

/******************************************************************************
 DoScroll

	Scroll the panorama belonging to the scrollpane. All scrolling
	bottlenecks through this method
 ******************************************************************************/

void CScrollPane::DoScroll( long hDelta, long vDelta)
{
	itsPanorama->Scroll( hDelta, vDelta, TRUE);
}


/**** S C R O L L   A C T I O N   F U N C T I O N S ****/


/*=============================================================================
 SBarActionProc
 
		Function called continuously while user holds mouse down within
		a fixed part of a scroll bar. Called by the Toolbox during
		the TrackControl trap.
 =============================================================================*/

pascal void	SBarActionProc(
	ControlHandle	macControl,			/* Toolbox control record			*/
	short			whichPart)			/* Part of scroll bar clicked in	*/
{
	register CScrollBar	*theSBar;		/* Associated scroll bar object		*/
	
	if (whichPart != 0) {
		theSBar = (CScrollBar*) GetCRefCon(macControl);
		if (theSBar->theOrientation == HORIZONTAL)
			((CScrollPane*)theSBar->itsSupervisor)->DoHorizScroll(whichPart);
		else
			((CScrollPane*)theSBar->itsSupervisor)->DoVertScroll(whichPart);
	}
}
		

/*=============================================================================
 SBarThumbFunc

		Function called after user drags the thumb of a scroll bar
 =============================================================================*/

void	SBarThumbFunc(
	CScrollBar		*theSBar,			/* Scroll bar object				*/
	short			delta)				/* Change in scroll bar setting		*/
{

	if (theSBar->theOrientation == HORIZONTAL)
		((CScrollPane*)theSBar->itsSupervisor)->DoThumbDrag(delta, 0);
	else
		((CScrollPane*)theSBar->itsSupervisor)->DoThumbDrag(0, delta);
}
