/******************************************************************************
 CIntegerText.c

		A subclass of CDialogText that accepts only integer entries. Any
		characters are accepted as the user types, and the entry is checked
		at validation. Validation occurs when the user tabs out of the field
		or tries to confirm the dialog.
		
		SANE is used to to convert the text to a number because it provides
		Script Manager support.
		
		As a subclass of CDialogText, CIntegerText fields may or not be required
		and have a maximum allowed length. Required fields are considered invalid
		if left empty. Additionally CIntegerText fields may have a default value,
		which is the value returned if the text is empty. They also have a
		minimum and maximum allowed value.
		
		The showRangeOnErr flag determine is the acceptable range is displayed
		in the validation failure alert. For meaningful ranges, its nice to
		have the alert say someting like "This field accepts only numbers between
		1 and 100", but if the range is MINLONG to MAXLONG, the message would look
		silly. When showRangeOnErr is false, the alert only displays "This field
		accepts only numbers".
		
		To use this class, you must include the SANE library in your project.
		
	SUPERCLASS = CDialogText
	
	Copyright  1991 Symantec Corporation. All rights reserved.
	

 ******************************************************************************/

#include "CIntegerText.h"
#include <SANE.h>
#include "Global.h"
#include "TBUtilities.h"
#include <Packages.h>

/******************************************************************************
 IIntegerText
 
 	Initialize the CIntegerText object. The default constraints are:
 	no max length, required, range (MINLONG, MAXLONG) and showRangeOnErr
 	is FALSE.
******************************************************************************/

void CIntegerText::IIntegerText( CView *anEnclosure, CView *aSupervisor,
					short aWidth, short aHeight,
					short aHEncl, short aVEncl,
					SizingOption aHSizing, SizingOption aVSizing,
					short aLineWidth)
{
	CDialogText::IDialogText( anEnclosure, aSupervisor, aWidth, aHeight,
					aHEncl, aVEncl, aHSizing, aVSizing, aLineWidth);
					
	minValue = MINLONG;
	maxValue = MAXLONG;
	defaultValue = 0;
	isRequired = TRUE;
	showRangeOnErr = FALSE;
}

/******************************************************************************
 IViewTemp
 
 	Initialize the CIntegerText object from a resource template.
******************************************************************************/

void CIntegerText::IViewTemp(CView *anEnclosure, CBureaucrat *aSupervisor,
							Ptr viewData)
{
	tIntegerTextTempP template = (tIntegerTextTempP) viewData;
	
	inherited::IViewTemp( anEnclosure, aSupervisor, viewData);
	
	minValue = template->minValue;
	maxValue = template->maxValue;
	
	// As a special case, we interpret that
	// if minValue == maxValue, the intent is to allow all integers
	
	if (minValue == maxValue)
	{
		minValue = MINLONG;
		maxValue = MAXLONG;
	}
	
	defaultValue = template->defaultValue;
	showRangeOnErr = template->showRangeOnErr != 0;
}

/******************************************************************************
 SpecifyRange
 
 	Specify the allowed range of integral values. If the range is MINLONG-MAXLONG,
 	then showRangeOnErr is turned off.
******************************************************************************/

void CIntegerText::SpecifyRange( long aMinimum, long aMaximum)
{
	ASSERT( aMaximum > aMinimum);
	
	minValue = aMinimum;
	maxValue = aMaximum;
	
	// if range is MINLONG, MAXLONG then any integral entry is
	// allowed, so don't display the acceptable range in the error alert
	
	showRangeOnErr = !((aMinimum == MINLONG) && (aMaximum == MAXLONG));
	
}

/******************************************************************************
 SpecifyDefaultValue
 
 	Specifying a default value implies that the text is valid if empty, 
 	therefore isRequired is set FALSE.
******************************************************************************/

void CIntegerText::SpecifyDefaultValue( long aDefaultValue)
{
	defaultValue = aDefaultValue;
	isRequired = FALSE;
}

/******************************************************************************
 SetIntValue
 
 	Set the text to aValue.
******************************************************************************/

void CIntegerText::SetIntValue( long aValue)
{
	Str255		numStr, oldText;
	extended	x;
	decform		dec;
	long double	tmp;
	
	GetTextString( oldText);
	
	tmp = (double) aValue;
		
#if __option(native_fp) && !__option(mc68881)
	x = tmp;
#else
	x96tox80( &tmp, &x);
#endif

	dec.style = FIXEDDECIMAL;
	dec.digits = 0;
	
	num2str( &dec, x, numStr);
	SetTextString( numStr);
	
	if (!Validate())
		SetTextString( oldText);
}

/******************************************************************************
 GetIntValue
 
 	Get the current value. If the text is empty, the default value is returned.
******************************************************************************/

long CIntegerText::GetIntValue( void)
{
	long 	value;
	Boolean valid;
	
	ConvertToInteger( &value, &valid);
			
	return value;
}

/******************************************************************************
 Validate {OVERRIDE}
 
 	Ensure that all the constraints have been met. If validation fails,
 	the ReportInvalidText method is called to show an alert.
******************************************************************************/

Boolean CIntegerText::Validate( void)
{
	long 	value;
	Boolean valid;
	Str31	minText, maxText;
	
	valid = inherited::Validate();
	if (valid)
	{
		ConvertToInteger( &value, &valid);
		if (valid)
		{
			if ((value < minValue) || (value > maxValue))
				valid = FALSE;
		}
	
		if (!valid)
		{
			if (showRangeOnErr)
			{
				NumToString( minValue, minText);
				NumToString( maxValue, maxText);
				ParamText( NULL, minText, maxText, NULL);
				ReportInvalidText( validateIntRangeFailed);
			}
			else
				ReportInvalidText( validateIntFailed);
		}
	}
	return valid;
}

/******************************************************************************
 ConvertToInteger
 
 	Protected method to convert the text to a long. Returns TRUE if
 	the text was a valid integral entry with the limits of a long.
******************************************************************************/

void CIntegerText::ConvertToInteger( long *intValue, Boolean *valid)
{
	Str255	str;
	short	index, length;
	decimal dec;
	Boolean	validPrefix = 0;
	extended  x;
	double  minLng, maxLng;
	long double val;
	
	GetTextString( str);
		
	length = Length(str);
	if (length == 0)
	{
		*valid = !isRequired;
		*intValue = defaultValue;
	}
	else
	{		
		index = 1;
		
		
		str2dec( str, &index, &dec, &validPrefix);
		
		// if validPrefix is TRUE and index is past the last char,
		// then the string is a valid numeric string. If the
		// exp field is zero, then it is a valid integer
		
		*valid = validPrefix && (index > length) && (dec.exp >= 0);
		if (*valid)
		{
			x = dec2num( &dec);
			
#if __option(native_fp) && !__option(mc68881)
			val = x;
#else
			x80tox96( &x,  &val);
#endif
			minLng = (long) MINLONG;
			maxLng = MAXLONG;
			if ((val >= minLng) && (val <= maxLng))
				*intValue = (long) val;
			else
				*valid = FALSE;
		}
	}
}