/******************************************************************************
 CGridSelector.c

							The GridSelector Class
		
		A rectangular grid of items from which a single selection may
		be made.
	
	SUPERCLASS = CSelector
	
	Copyright  1989 Symantec Corporation. All rights reserved.

 ******************************************************************************/

#include "Global.h"
#include "CGridSelector.h"
	

/******************************************************************************
 IGridSelector

		Initialize a GridSelector object
 ******************************************************************************/

void	CGridSelector::IGridSelector(
	CView			*anEnclosure,
	CBureaucrat		*aSupervisor,
	short			aHEncl,
	short			aVEncl,
	SizingOption	aHSizing,
	SizingOption	aVSizing,
	short			aSelection,
	short			aCommandBase,
	short			aRows,
	short			aColumns,
	short			aBoxWidth,
	short			aBoxHeight)
{
	short			aWidth;
	short			aHeight;
	
	aWidth = aColumns * aBoxWidth - 1;
	aHeight = aRows * aBoxHeight - 1;
	
	gridOn = TRUE;
	rows = aRows;
	columns = aColumns;
	boxWidth = aBoxWidth;
	boxHeight = aBoxHeight;

	CSelector::ISelector(anEnclosure, aSupervisor,
					 aWidth, aHeight, aHEncl, aVEncl, aHSizing, aVSizing,
					 rows * columns, aSelection, aCommandBase);
					 
	autoRefresh = FALSE;
	position.h = position.v = 1;
	bounds.left = bounds.top = 1;
	bounds.right = columns + 1;
	bounds.bottom = rows + 1;
	SetScales(boxWidth, boxHeight);
}
	

/******************************************************************************
 Draw {OVERRIDE}

		Draw a Grid Selector
 ******************************************************************************/

void	CGridSelector::Draw(
	Rect		*area)
{
	short			r;					/* Row index						*/
	short			c;					/* Column index						*/
	register short	theItem;			/* Item number						*/
	Rect			theBox;				/* Bounding box of item				*/
	
	if (gridOn)							/* Draw grid lines between items	*/
		DrawGrid();
	
	theItem = 0;						/* Run thru and draw all items		*/
	for (r = 1; r <= rows; r++) {
		for (c = 1; c <= columns; c++) {
			theItem++;
			FindItemBox(theItem, &theBox);
			DrawItem(theItem, &theBox);
		}
	}
	
	if (active)							/* Hilite the current selection		*/
		HiliteItem(selection, hiliteON);
}
	

/******************************************************************************
 HiliteItem {OVERRIDE}

		Hilite an item in the specified way
 ******************************************************************************/

void	CGridSelector::HiliteItem(
	short		theItem,				/* Item number to hilite			*/
	HiliteState	state)					/* State of hiliting desired		*/
{
	Rect			theRect;
	register Rect	*theBox;
	long			ticks;
	
	if (theItem == NOTHING)
		return;

	FindItemBox(theItem, &theRect);
	theBox = &theRect;
	
	switch (state) {
	
		case hiliteON:
		case hiliteOFF:
			InvertRect(theBox);
			break;
			
		case hiliteDYNAMIC:
			PenNormal();
			PenMode(patXor);
			FrameRect(theBox);
			Delay(2, &ticks);
			InsetRect(theBox, 1, 1);
			FrameRect(theBox);
			Delay(2, &ticks);
			FrameRect(theBox);
			Delay(2, &ticks);
			InsetRect(theBox, -1, -1);
			FrameRect(theBox);
			PenNormal();
			break;
	}
}
	

/******************************************************************************
 FindItem {OVERRIDE}

		Determine which item, if any, is selected by a click at the
		specified hit point. Point is specified in pane coordinates.
 ******************************************************************************/

short	CGridSelector::FindItem(
	Point		hitPt)					/* Click location in pane coords	*/
{
	register short	theRow;
	register short	theCol;
	
	theRow = (hitPt.v - 1) / boxHeight + 1;
	theCol = (hitPt.h - 1) / boxWidth + 1;
	
	if ( (theRow < 1) || (theRow > rows) || (theCol < 1) || (theCol > columns) )
		return(NOTHING);
	else
		return( (theRow - 1) * columns + theCol);
}
	

/******************************************************************************
 FindItemBox

		Return the enclosing box of a particular item. Note that the
		enclosing box does not include the surrounding grid lines.
 ******************************************************************************/

void	CGridSelector::FindItemBox(
	short		theItem,
	Rect		*theBox)
{
	short		theRow;
	short		theCol;
	short		extra;
	
	extra = gridOn ? -1 : 0;
	SetRect(theBox, 0, 0, boxWidth + extra, boxHeight + extra);
	
	theRow = (theItem - 1) / columns + 1;
	theCol = (theItem - 1) % columns + 1;

	OffsetRect(theBox, (theCol - 1) * boxWidth, (theRow - 1) * boxHeight);

}
	

/******************************************************************************
 DrawGrid

		Draw grid lines around the items
 ******************************************************************************/

void	CGridSelector::DrawGrid()
{
	long hExtent, vExtent;
	short	coord, length;
	
	GetPixelExtent( &hExtent, &vExtent);

	length = hExtent - 1;
	for (coord = boxHeight - 1; coord < vExtent; coord += boxHeight) {
		MoveTo(0, coord);
		LineTo(length, coord);
	}
	
	length = vExtent - 1;
	for (coord = boxWidth - 1; coord < hExtent; coord += boxWidth) {
		MoveTo(coord, 0);
		LineTo(coord, length);
	}
}
	

/******************************************************************************
 DrawItem

		Draw an item within the specified box. Subclasses must override
		this method.
 ******************************************************************************/

void	CGridSelector::DrawItem(
	short		theItem,				/* Item number to draw				*/
	Rect		*theBox)				/* Box surrounding the item			*/
{
}										/* Null Method						*/
	

/******************************************************************************
 SetGridOn

		Turn the drawing of grid lines on or off
 ******************************************************************************/

void	CGridSelector::SetGridOn(
	Boolean		aGridOn)
{
	gridOn = aGridOn;					/* Set instance variable			*/
}