/******************************************************************************
 WindoidWDEF.c

		A window definition procedure for a windoid. Typical uses
		are windows for a tear-off menu or floating palette of tools.
		
		This source should be compiled into a WDEF code resource and
		installed in the resource file for a project or application.
		
		The variation code of the window specifies whether the drag bar
		appears on the top or the left of the windoid.
			varCode = 0  -  Drag bar on top
			varCode = 2  -  Drag bar on left
		With any varCode other than 0 or 2, no drag bar is drawn at all.
		
		Note that MultiFinder thinks that any window with a varCode of 1
		is a modal window, meaning that it will not allow context switches
		when such a window is in front.
		
		Remember, when specify the WDEF id for a window, the varCode is
		in the low 4 bits and the actual WDEF id is in the high 12 bits:
			Window DefProc ID = 16 * WDEF id  +  varCode
		
		This, of course, means that varCode must be between 0 and 15, and
		WDEF id's between 0 and 4095 (WDEF's 0 and 1 are used by the
		System for the standard Mac windows).
		
		Copyright  1989 Symantec Corporation. All rights reserved.
		
		Written by:  Gregory H. Dow
		
 ******************************************************************************/

#include "WindoidWDEF.h"

/*** Constants ***/

#define		FRAME_WIDTH		1			/* Thickness of frame around window	*/
#define		DRAG_WIDTH		10			/* Thickness of drag bar			*/
#define		SHADOW_LENGTH	1			/* Size of drop shadow				*/
#define		DRAG_TOP		0			/* VarCode for drag bar on top		*/
#define		DRAG_LEFT		2			/* VarCode for drag bar on left		*/

/******************************************************************************
 Main Function
 ******************************************************************************/

pascal long		main(varCode, macWindow, theMessage, param)
short		varCode;					/* Variety of window				*/
WindowPeek	macWindow;					/* Toolbox window record pointer	*/
short		theMessage;					/* Kind of operation to perform		*/
long		param;						/* Parameter for the message		*/
{
	long	result = 0L;				/* Function result					*/
	
	switch (theMessage) {				/* Take appropriate action			*/
	
		case wDraw:						/* Draw the window frame			*/
		
			if (macWindow->visible) {	/* Only draw if window is visible	*/
				switch ((short)param) {	/* Find out which part to draw		*/
				
					case wNoHit:		/* Draw the entire frame			*/
						DrawFrame(varCode, macWindow);
						break;
						
					case wInGoAway:		/* Toggle hiliting of the close box	*/
						ToggleCloseBox(varCode, macWindow);
						break;
				}
			}
			break;
			
		case wHit:						/* Find part of window clicked in	*/
			result = (long) FindPart(varCode, macWindow, param);
			break;
			
		case wCalcRgns:					/* Build structure & content rgns	*/
			BuildRegions(varCode, macWindow);
			break;
	}

	return(result);						/* Pass back result	code. Used only	*/
										/*   for wHit message to tell what	*/
										/*   part contains the hit point	*/
}


/******************************************************************************
 DrawFrame

		Draw the entire window frame.
 ******************************************************************************/

void	DrawFrame(short varCode,WindowPeek macWindow)
{
	PenState		savePen;			/* Current pen characteristics		*/
	Rect			wFrame;				/* Window frame around content rgn	*/
	Rect			dragBar;			/* Drag bar of window				*/
	Rect			closeBox;			/* Go away box of window			*/
	Pattern			dragPattern;		/* Pattern within drag bar			*/
	register char	*flip;				/* Flip bits in drag pattern		*/
	unsigned long	*shift;				/* Shift bits in drag pattern		*/
	
	GetPenState(&savePen);				/* Save the current pen state		*/
	
		/* Structure region of a window has four parts:	*/
		/*		(1) Frame around the content region		*/
		/*		(2) Drop shadow							*/
		/*		(3) Drag bar							*/
		/*		(4) Close box inside the drag bar		*/

										/* (1) Frame around content region	*/
	wFrame = (**macWindow->contRgn).rgnBBox;	
	InsetRect(&wFrame, -FRAME_WIDTH, -FRAME_WIDTH);
	PenSize(FRAME_WIDTH, FRAME_WIDTH);
	FrameRect(&wFrame);
										/* (2) Drop shadow below and to the	*/
										/*     right of content region		*/
	PenSize(SHADOW_LENGTH, SHADOW_LENGTH);

	if (varCode == DRAG_TOP) {			/* Drag bar is on top of window		*/
										/*   Draw in the drop shadow		*/
		MoveTo(wFrame.left + SHADOW_LENGTH + 2, wFrame.bottom);
		LineTo(wFrame.right, wFrame.bottom);
		LineTo(wFrame.right, wFrame.top - DRAG_WIDTH + SHADOW_LENGTH);
		
	} else if (varCode == DRAG_LEFT) {	/* Drag bar is on left of window	*/
										/*   Draw in the drop shadow		*/
		MoveTo(wFrame.left - DRAG_WIDTH + SHADOW_LENGTH, wFrame.bottom);
		LineTo(wFrame.right, wFrame.bottom);
		LineTo(wFrame.right, wFrame.top + SHADOW_LENGTH);
	}
	
										/* (3) Drag bar						*/
										
	StuffHex(dragPattern, "\pAA00AA00AA00AA00");

		/* Since patterns are fixed to the screen, we must alter the	*/
		/* pattern depending on the location of the window. Patterns	*/
		/* are 8 by 8 bit images which may be thought of as 8			*/
		/* horizontal lines which are 8 pixels long. The pattern for	*/
		/* the drag bar has alternating gray (10101010) and white		*/
		/* (00000000) lines. Shifting the pattern one bit horizontally	*/
		/* is done by flipping the bits in the gray lines. Shifting one	*/
		/* bit vertically is done by rotating the lines one position.	*/
		
	if ( (!(wFrame.left & 1)  &&  varCode == DRAG_TOP)  ||
		 ( (wFrame.left & 1)  &&  varCode == DRAG_LEFT) ) {
		flip = (char*)dragPattern;		/* Flip bits on every other line	*/
		flip[0] = ~flip[0];				/*   in the pattern					*/
		flip[2] = flip[0];
		flip[4] = flip[0];
		flip[6] = flip[0];
	}
	
	if ( ( (wFrame.top & 1)  &&  varCode == DRAG_TOP)  ||
		 (!(wFrame.top & 1)  &&  varCode == DRAG_LEFT) ) {
		shift = (unsigned long*)dragPattern;
		shift[0] = shift[0] >> 8;		/* Perform a single rotation of the	*/
		shift[1] = shift[0];			/*   lines in the pattern			*/
	}
	
	GetDragBar(varCode, macWindow, &dragBar);
	PenSize(1, 1);						/* Draw outline of drag bar and		*/
	FillRect(&dragBar, &dragPattern);	/*   fill it with the proper		*/
	FrameRect(&dragBar);				/*   pattern						*/
	
										/* (4) Close box inside drag bar	*/
	if (macWindow->goAwayFlag) {		/* Make sure window has a close box	*/
		GetCloseBox(varCode, &dragBar, &closeBox);
		InsetRect(&closeBox, -1, -1);	/* Blast a hole in the drag bar		*/
		EraseRect(&closeBox);			/*   pattern						*/
		InsetRect(&closeBox, 1, 1);
		FrameRect(&closeBox);			/* Now draw the close box			*/
	}
	
	SetPenState(&savePen);				/* Restore original pen state		*/
}


/******************************************************************************
 ToggleCloseBox

		Toggle hiliting of the close box.
 ******************************************************************************/

void	ToggleCloseBox(short varCode,WindowPeek macWindow)
{
	Rect		dragBar;				/* Rectangle enclosing drag bar		*/
	Rect		closeBox;				/* Rectangle enclosing close box	*/
	
	GetDragBar(varCode, macWindow, &dragBar);
	GetCloseBox(varCode, &dragBar, &closeBox);
	InvertRect(&closeBox);
}
	

/******************************************************************************
 FindPart

		Find which part of the window was hit by a mouse click at the
		specified location.
 ******************************************************************************/

short	FindPart(short varCode,WindowPeek macWindow,long location)
{
	Point		theClick;				/* Global coords of mouse click		*/
	Rect		dragBar;
	Rect		closeBox;

	theClick.h = LoShort(location);		/* Extract horizontal component		*/
	theClick.v = HiShort(location);		/* Extract vertical component		*/

										/* Take a quick exit if click is    */
										/* not inside this window structure */
	if (!PtInRgn(theClick, macWindow->strucRgn))
		return(wNoHit);
		
	if (PtInRgn(theClick, macWindow->contRgn))
		return(wInContent);				/* Click is inside content region	*/
		
										/* Is click inside the drag bar?	*/	
	GetDragBar(varCode, macWindow, &dragBar);
	if (PtInRect(theClick, &dragBar)) {
										/*   Yes, now check the close box	*/
										/*   since the close box is inside	*/
										/*   the drag bar					*/
		GetCloseBox(varCode, &dragBar, &closeBox);
		if (macWindow->goAwayFlag && PtInRect(theClick, &closeBox))
			return(wInGoAway);			/*   Click is inside close box		*/
		else
			return(wInDrag);			/*   Click is inside the drag bar	*/
										/*   but NOT inside the close box	*/
	}
	
	return(wNoHit);						/* We've fallen through all checks	*/
										/* Click is in some insignificant	*/
										/* part of the frame				*/
}
	

/******************************************************************************
 BuildRegions

		Build the structure and content region of a window.
 ******************************************************************************/

void	BuildRegions(short varCode,WindowPeek macWindow)
{
	GrafPtr		macGrafPtr;				/* Ptr to window's graf port		*/
	Rect		theRect;				/* Window regions are rectangular	*/
	RgnHandle	shadowRgn;				/* Region defining drop shadow		*/
	
		/* For our window, the port rectangle corresponds to the content */
		/* region.  However, the port rectangle is in local coords, and  */
		/* we must specify the content region in global coords.  Since   */
		/* the bit map bounds (portBits.bounds) of the window define the */
		/* local coords, we must find the distance between the top left  */
		/* corners of the bit map bounds and the port rectangle.  This   */
		/* will give the global coords of the top left corner of the     */
		/* port rectangle since the top left corner of the screen is     */
		/* (0,0) in global coords.										 */
	
	macGrafPtr = (GrafPtr) macWindow;	/* Need to access the graf port		*/
	theRect = macGrafPtr->portRect;
	OffsetRect(&theRect, -macGrafPtr->portBits.bounds.left,
						 -macGrafPtr->portBits.bounds.top);

										/* theRect is the content region	*/							   
	RectRgn(macWindow->contRgn, &theRect);

		/* The structure region of the window includes a frame around the */
		/* content region, the drag bar, and a drop shadow				  */
		
										/* Add thickness of frame			*/
	InsetRect(&theRect, -FRAME_WIDTH, -FRAME_WIDTH);

										/* Add size of drag bar				*/
	if (varCode == DRAG_TOP)			/* Location depends on the varCode	*/
		theRect.top -= DRAG_WIDTH;		/*   Drag bar on top of window		*/
	else if (varCode == DRAG_LEFT)
		theRect.left -= DRAG_WIDTH;		/*   Drag bar on left of window		*/
 	
 										/* First stage of structure region	*/
 	RectRgn(macWindow->strucRgn, &theRect);
 										/* Now add the drop shadow which	*/
 										/*   hangs below and to the right	*/
 	OffsetRect(&theRect, SHADOW_LENGTH, SHADOW_LENGTH);
 	shadowRgn = NewRgn();				/* Define region for the shadow		*/
 	RectRgn(shadowRgn, &theRect);
	
										/* Add shadow region to the struct	*/
										/*   region							*/
	UnionRgn(macWindow->strucRgn, shadowRgn, macWindow->strucRgn);
	DisposeRgn(shadowRgn);				/* All thru shadow region			*/
}
	

/******************************************************************************
 GetDragBar

		Pass back a rectangle which encloses the drag bar of the window
 ******************************************************************************/

void	GetDragBar(short varCode,WindowPeek macWindow,Rect *dragBar)
{
										/* Start with the content rectangle	*/
										/*   and enlarge it to include the	*/
										/*   frame around the window		*/
	*dragBar = (**macWindow->contRgn).rgnBBox;	
	InsetRect(dragBar, -FRAME_WIDTH, -FRAME_WIDTH);

	if (varCode == DRAG_TOP) {			/* Drag bar is on top of window		*/
		dragBar->bottom = dragBar->top + 1;
		dragBar->top -= DRAG_WIDTH;
		
	} else if (varCode == DRAG_LEFT) {	/* Drag bar is on left of window	*/
		dragBar->right = dragBar->left + 1;
		dragBar->left -= DRAG_WIDTH;
	
	} else {							/* Some other variation code		*/
		SetRect(dragBar, 0, 0, 0, 0);	/*   Specify no drag bar at all		*/
	}
}
	

/******************************************************************************
 GetCloseBox

		Pass back a rectangle which encloses the close box of the window
 ******************************************************************************/

void	GetCloseBox(short varCode,Rect *dragBar,Rect *closeBox)
{
	if (varCode == DRAG_TOP) {			/* Drag bar is on top of window		*/
		SetRect(closeBox, dragBar->left + 8, dragBar->top + 2,
						  dragBar->left + 15, dragBar->bottom - 2);
		
	} else if (varCode == DRAG_LEFT) {	/* Drag bar is on left of window	*/
		SetRect(closeBox, dragBar->left + 2, dragBar->top + 8,
						  dragBar->right - 2, dragBar->top + 15);
	
	} else {							/* Some other variation code		*/
		SetRect(dragBar, 0, 0, 0, 0);	/*   Specify no close box at all	*/
	}

}