/******************************************************************************
 CBitMap.c

							The BitMap Class
		
	SUPERCLASS = CObject
	
	Copyright  1989 Symantec Corporation. All rights reserved.

 ******************************************************************************/

#include "Global.h"
#include "CBitMap.h"
#include "TCLUtilities.h"
#include "CDesktop.h"

#include "LongQD.h"

extern CDesktop		*gDesktop;

/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 IBitMap

		Initialize a BitMap object
 ******************************************************************************/

void	CBitMap::IBitMap(
	short		width,					/* Width of BitMap in pixels		*/
	short		height,					/* Height of BitMap in pixels		*/
	Boolean		makePort)				/* Create offscreen port?			*/
{
	short		rowBytes;				/* Width of BitMap in bytes			*/
	Boolean		savedAlloc;
	
		/* rowBytes of a bitmap must be a even number. There are	*/
		/* 8 bits (pixels) in a byte. The formula below makes		*/
		/* rowBytes the smallest multiple of 16 which is greater	*/
		/* than or equal to the requested width.					*/

	rowBytes = ((width + 15) / 16) * 2;
	
		/* Create a block which contains not only the bit image,	*/
		/* but also the BitMap record. The bit image follows the	*/
		/* the BitMap record in the block.							*/
		
	macPort = NULL;
	
	savedAlloc = SetAllocation(kAllocCanFail);

	macBitMap = (BitMap*) NewPtrClear((long) rowBytes * height +
										sizeof(BitMap));
										
	SetAllocation(savedAlloc);
	
	FailNIL(macBitMap);
											
	macBitMap->rowBytes = rowBytes;
		
		/* Since macBitMap points to the start of the block, we can	*/
		/* use pointer arithmetic and just add one to it. This		*/
		/* gives us a pointer to the byte following the BitMap		*/
		/* record, which is just what we want. The bit image		*/
		/* comes right after the BitMap record in our block.		*/
	
	macBitMap->baseAddr = (Ptr) (macBitMap + 1);
	
										/* Default origin is at (0, 0)		*/
	SetRect(&macBitMap->bounds, 0, 0, width, height);
	
	xferMode = srcCopy;					/* Most common transfer mode. It	*/
										/*   overwrites the source on top	*/
										/*   of the destination.			*/
										
	if (makePort) {
		GetPort(&savePort);

		savedAlloc = SetAllocation( kAllocCanFail);
		macPort = (GrafPtr) NewPtr(sizeof(GrafPort));
		SetAllocation( savedAlloc);
	
		FailNIL( macPort);
		
		OpenPort(macPort);
		SetPort(macPort);
		SetPortBits(macBitMap);
		PortSize(width, height);
		SetOrigin(0, 0);
		ClipRect(&macBitMap->bounds);
		RectRgn(macPort->visRgn, &macBitMap->bounds);
		SetPort(savePort);
		CView::ForceNextPrepare();
	} else {
		macPort = NULL;
	}
}


/******************************************************************************
 Dispose {OVERRIDE}

		Dispose of a BitMap
 ******************************************************************************/

void	CBitMap::Dispose()
{
	if (macBitMap != NULL) {
		DisposPtr( (Ptr) macBitMap);
	}

	if (macPort != NULL) {
		ClosePort(macPort);
		DisposPtr( (Ptr) macPort);
	}

	inherited::Dispose();				/* Pass message on to superclass	*/
}


/**** A C C E S S I N G   M E T H O D S ****/


/******************************************************************************
 GetBounds

		Return the bounding rectangle of a BitMap. This rectangle defines
		the size and coordinate system of the BitMap.
 ******************************************************************************/

void	CBitMap::GetBounds(
	LongRect		*theBounds)
{
	QDToLongRect( &macBitMap->bounds, theBounds);
						/* Return copy of instance variable	*/
}


/******************************************************************************
 SetBoundsOrigin

		Set the coordinates of the top left corner of the bounds of a
		BitMap. This changes the coordinate system of the BitMap.
 ******************************************************************************/

void	CBitMap::SetBoundsOrigin(
	short		hOrigin,
	short		vOrigin)
{
	macBitMap->bounds.right += hOrigin - macBitMap->bounds.left;
	macBitMap->bounds.bottom += vOrigin - macBitMap->bounds.top;
	macBitMap->bounds.left = hOrigin;
	macBitMap->bounds.top = vOrigin;
}


/******************************************************************************
 SetXferMode

		Set the transfer mode used by CopyBits operations on a BitMap
 ******************************************************************************/

void	CBitMap::SetXferMode(
	short		aXferMode)
{
	xferMode = aXferMode;				/* Set instance variable			*/
}


/******************************************************************************
 GetXferMode

		Retrieve the transfer mode used by CopyBits operations on a BitMap
 ******************************************************************************/

short	CBitMap::GetXferMode()
{
	return(xferMode);					/* Return instance variable			*/
}


/******************************************************************************
 PixelIsBlack

		Determine whether the pixel at the specified coordinates is black.
		Returns FALSE if the pixel is white or the point is not within
		the BitMap.
 ******************************************************************************/

Boolean		CBitMap::PixelIsBlack(
	LongPt		*pixelPos)
{
	short		hPos;
	short		vPos;
	long		bitNum;
	LongRect	bounds;
	
	GetBounds( &bounds);
	if (!PtInLongRect( pixelPos, &bounds))
		return FALSE;		/* point is not in bitmap	*/
	
	hPos = pixelPos->h - macBitMap->bounds.left;
	vPos = pixelPos->v - macBitMap->bounds.top;
	
										/* Point is within BitMap			*/
										/* Convert point into a bit offset	*/
										/*   from the start of the image	*/
	bitNum = (long) vPos * macBitMap->rowBytes * 8L + hPos;
	return(BitTst(macBitMap->baseAddr, bitNum));
		
}


/**** I M A G E   C O P Y I N G   M E T H O D S ****/


/******************************************************************************
 CopyTo

		Copy bits to a BitMap from the bit map of the current port. The
		fromRect is a rectangle in the current port's bit map (source rect),
		and the toRect is a rectangle in this BitMap (dest rect). maskRgn is
		a clipping region specified in the same coords as the dest rect,
		i.e., the coords of this BitMap. A NULL maskRgn means that no extra
		clipping is performed. Copying takes place using the transfer mode
		stored in the xferMode instance variable.
 ******************************************************************************/

void	CBitMap::CopyTo(
	LongRect	*fromRect,
	LongRect	*toRect,
	RgnHandle	maskRgn)
{
	LCopyBits(&thePort->portBits, macBitMap, fromRect, toRect,
				xferMode, maskRgn);
}


/******************************************************************************
 CopyFrom

		Copy bits from a BitMap to the bit map of the current port. The
		fromRect is a rectangle in this BitMap (source rect), and the
		toRect is a rectangle in the current port's bit map (dest rect).
		maskRgn is a clipping region specified in the same coords as the
		dest rect, i.e., the coords of the current port. A NULL maskRgn means
		that no extra clipping is performed. Copying takes place using the
		transfer mode stored in the xferMode instance variable.
 ******************************************************************************/

void	CBitMap::CopyFrom(
	LongRect		*fromRect,
	LongRect		*toRect,
	RgnHandle	maskRgn)
{
	LCopyBits(macBitMap, &thePort->portBits, fromRect, toRect,
				xferMode, maskRgn);
}


/**** P R E P A R I N G   M E T H O D S ****/


/******************************************************************************
 BeginDrawing

		Set up for drawing to a BitMap. If it doesn't have its own port,
		make it the port bits of the current QuickDraw grafPort. Save the
		current port bits so they can be restored later by the EndDrawing
		method. If it has its own port, save the current port and SetPort
		to the BitMap's port.
 ******************************************************************************/

void	CBitMap::BeginDrawing()
{
	if (macPort == NULL) {
		saveBitMap = thePort->portBits;	/* Save current bit map for later	*/
										/*   restoration					*/
		SetPortBits(macBitMap);			/* Subsequent drawing will take		*/
										/*   place in this BitMap			*/
	} else {
		GetPort(&savePort);
		SetPort(macPort);
	}
}


/******************************************************************************
 EndDrawing

		Reset the port to the way it was before the BeginDrawing message
		was sent. If BitMap has its own port, restore the saved Grafport.
		Otherwise, restore the saved bitmap in the current port.
 ******************************************************************************/

void	CBitMap::EndDrawing()
{
	if (macPort == NULL) {
		SetPortBits(&saveBitMap);
	} else {
		SetPort(savePort);
	}
}