/******************************************************************************
 LongCoordinates.c

		Utility functions to convert QuickDraw and their long coordinate
		counterparts back and forth, and long coordinate versions
		of the most frequently used point and rectangle utilities.
	
	Copyright  1991 Symantec Corporation. All rights reserved.
	

 ******************************************************************************/

#include "LongCoordinates.h"
#include "Global.h"

/******************************************************************************
 QDPtToLongPt
 
 	Convert a Point to a LongPt
******************************************************************************/

pascal void	QDToLongPt( Point srcPt, LongPt *destPt)
{
	destPt->h = srcPt.h;
	destPt->v = srcPt.v;

}	/* QDToLongPt */

/******************************************************************************
 LongPtToQDPt
 
 	Convert a LongPt to a Point. Values are clipped to 16 bits.
******************************************************************************/

pascal void	LongToQDPt( LongPt *srcPt, Point *destPt)
{
	destPt->h = Min( Max((long)MININT, srcPt->h), (long)MAXINT);
	destPt->v = Min( Max((long)MININT, srcPt->v), (long)MAXINT);

}	/* LongToQDPt */

/******************************************************************************
 SetLongPt
 
 	Set the members of a LongPt. 
******************************************************************************/

pascal void SetLongPt( LongPt *pt, long h, long v)
{
	pt->h = h;
	pt->v = v;

}	/* SetLongPt */

/******************************************************************************
 AddLongPt
 
 	Adds srcPt and destPt, returns the result in destPt.
******************************************************************************/

pascal void	AddLongPt( LongPt *srcPt, LongPt *destPt)
{
	destPt->h += srcPt->h;
	destPt->v += srcPt->v;

}	/* AddLongPt */

/******************************************************************************
 SubLongPt
 
 	Subtracts srcPt from destPt, returns the result in destPt.
******************************************************************************/

pascal void	SubLongPt( LongPt *srcPt, LongPt *destPt)
{
	destPt->h -= srcPt->h;
	destPt->v -= srcPt->v;

}	/* SubLongPt */

/******************************************************************************
 EqualLongPt
 
 	Returns TRUE if two LongPts are equal.
******************************************************************************/

pascal Boolean EqualLongPt( LongPt *pt1, LongPt *pt2)
{
	return (pt1->h == pt2->h) && (pt1->v == pt2->v);

}	/* EqualLongPt */

/******************************************************************************
 PtInQDSpace
 
 	Returns TRUE if a LongPt is within the 16-bit QuickDraw coordinate space
******************************************************************************/

pascal Boolean PtInQDSpace( LongPt *pt)
{
	return ((pt->h >= MININT) && (pt->h <= MAXINT) &&
			(pt->v >= MININT) && (pt->v <= MAXINT));

}	/* PtInQDSpace */

/******************************************************************************
 QDRectToLongRect
 
 	Convert a Rect to a LongRect
******************************************************************************/

pascal void	QDToLongRect( register Rect *srcRect, register LongRect *destRect)
{
	destRect->left = srcRect->left;
	destRect->top = srcRect->top;
	destRect->right = srcRect->right;
	destRect->bottom = srcRect->bottom;

}	/* QDToLongRect */

/******************************************************************************
 LongRectToQDRect
 
 	Convert aLongRect to a Rect. Values are clipped to 16 bit QuickDraw space.
******************************************************************************/

pascal void	LongToQDRect( register LongRect *srcRect, register Rect *destRect)
{
	destRect->left = Min( Max((long)MININT, srcRect->left), (long)MAXINT);
	destRect->top = Min( Max((long)MININT, srcRect->top), (long)MAXINT);
	destRect->right = Min( Max((long)MININT, srcRect->right), (long)MAXINT);
	destRect->bottom = Min( Max((long)MININT, srcRect->bottom), (long)MAXINT);

}	/* LongToQDRect */

/******************************************************************************
 SetLongRect
 
 	Fill in the members of a LongRect.
******************************************************************************/

pascal void SetLongRect( register LongRect *r, long left, long top, long right, long bottom)
{
	r->left = left;
	r->right = right;
	r->top = top;
	r->bottom = bottom;

}	/* SetLongRect */

/******************************************************************************
 OffsetLongRect
 
 	Translates a LongRect. Positive values are to the right and down.
******************************************************************************/

pascal void OffsetLongRect( register LongRect *r, long dh, long dv)
{
	r->left += dh;
	r->right += dh;
	r->top += dv;
	r->bottom += dv;

}	/* OffsetLongRect */

/******************************************************************************
 InsetLongRect
 
 	Insets the sides of a LongRect. Positive values move the sizes inward.
******************************************************************************/

pascal void InsetLongRect( register LongRect *r, long dh, long dv)
{
	r->left += dh;
	r->right -= dh;
	r->top += dv;
	r->bottom -= dv;

}	/* InsetLongRect */

/******************************************************************************
 SectLongRect
 
 	Calculates the intersection of two LongRects and returns the result
 	in destRect. destRect may be the same as either src1 or src2. Returns TRUE
 	if the result is non-empty.
******************************************************************************/

pascal Boolean SectLongRect( register LongRect *src1, register LongRect *src2, 
								register LongRect *destRect)
{
	destRect->left = Max( src1->left, src2->left);
	destRect->right = Min( src1->right, src2->right);
	destRect->top = Max( src1->top, src2->top);
	destRect->bottom = Min( src1->bottom, src2->bottom);
	
	return !EmptyLongRect( destRect);

}	/* SectLongRect */

/******************************************************************************
 UnionLongRect
 
	Calculates the union of two LongRects and returns the result
 	in destRect. destRect may be the same as either src1 or src2.
******************************************************************************/

pascal void UnionLongRect( LongRect *src1, LongRect *src2, LongRect *destRect)
{
	destRect->left = Min( src1->left, src2->left);
	destRect->right = Max( src1->right, src2->right);
	destRect->top = Min( src1->top, src2->top);
	destRect->bottom = Max( src1->bottom, src2->bottom);

}	/* UnionLongRect */

/******************************************************************************
 PtInLongRect
 
 	Returns TRUE if pt lies within r.
******************************************************************************/

pascal Boolean PtInLongRect( register LongPt *pt, register LongRect *r)
{
	return ((pt->h >= r->left) && (pt->h < r->right) && 
			(pt->v >= r->top) && (pt->v < r->bottom));

}	/* PtInLongRect */

/******************************************************************************
 Pt2LongRect
 
 	Calculates the minimal rect enclose the two given points.
******************************************************************************/

pascal void Pt2LongRect( register LongPt *pt1, register LongPt *pt2, 
							register LongRect *r)
{
	r->left = Min( pt1->h, pt2->h);
	r->top = Min( pt1->v, pt2->v);
	r->right = Max( pt1->h, pt2->h);
	r->bottom = Max( pt1->v, pt2->v);

}	/* Pt2LongRect */

/******************************************************************************
 EqualLongRect
 
 	Returns true if r1 and r2 are equal.
******************************************************************************/

pascal Boolean EqualLongRect( register LongRect *r1, register LongRect *r2)
{
	return ((r1->left == r2->left) && (r1->top == r2->top) &&
			(r1->right == r2->right) && (r1->bottom == r2->bottom));

}	/* EqualLongRect */

/******************************************************************************
 EmptyLongRect
 
 	Returns TRUE if r encloses no points.
******************************************************************************/

pascal Boolean EmptyLongRect( register LongRect *r)
{
	return ((r->top >= r->bottom) || (r->left >= r->right));

}	/* EmptyLongRect */

/******************************************************************************
 RectInQDSpace
 
 	Returns TRUE if r is entirely within QD space.
******************************************************************************/

pascal Boolean	RectInQDSpace( LongRect *r)
{
	return	PtInQDSpace( &topLeftL(r)) && PtInQDSpace( &botRightL(r));

}	/* RectInQDSpace */