/******************************************************************************
 CTableDragger.c

	Handles mouse selection for the CTable class. An instance of this
	class is create in response to a mouse click by CTable::MakeMouseTask.
	
	To use a subclass of this class, or substitute a different CMouseTask
	subclass, you should override CTable::MakeMouseTask in your CTable
	subclass.
		
	SUPERCLASS = CMouseTask 
	
	Copyright  1991 Symantec Corporation. All rights reserved.
	

 ******************************************************************************/

#include "CTableDragger.h"
#include "CTable.h"

/******************************************************************************
 ITableDragger
 
 	Initialize a CTableDragger. theModifers should be the modifier flags
 	from DoClick. selFlags should be the table's selection flags.
 	
 	This method initializes the instance variables fUseSense, fSelectRects,
 	and fExtend based upon the table's selection flags and the modifer keys.
 	
******************************************************************************/

void CTableDragger::ITableDragger( CTable *aTable, short theModifiers, long selFlags)
{
	CMouseTask::IMouseTask( 0);
	itsTable = aTable;
	selectionFlags = selFlags;
	modifierKeys = theModifiers;
	
		// fUseSense determines whether a cell is selected or deselected
		// as the mouse moves over it. Normally it are always selected,
		// but if the command key is down, we use the state of the first
		// cell and apply it to subsequent cells. Works like fatbits.
		
	fUseSense = (modifierKeys & cmdKey) != 0;
	
		// fSelectsRects determines whether dragging selects individual
		// cells or drags out rectangles. fSelectFlags is set TRUE if
		// fUseSense is FALSE, the selOnlyOne selection flag is off,
		// and either the selDragRects flag is on or the shift key
		// was help down.
		
	fSelectRects =  ((selFlags & selDragRects) || (modifierKeys & shiftKey)) &&
					!fUseSense && !(selFlags & selOnlyOne);
					
		// fExtend determines whether new selections are added to 
		// or replace the previous selection. They are added if fExtend
		// is TRUE. It is set TRUE if the selOnlyOne flag is off, and either
		// fUseSense is TRUE, fSelectsRects is TRUE and the shift key is down,
		// or the selExtendDrag flag is on.
		
	fExtend = !(selFlags & selOnlyOne) && 
			(fUseSense || (!fSelectRects && (modifierKeys & shiftKey))
			|| (selFlags & selExtendDrag));
	
	SetCell( anchorCell, 0, 0);
	
}	/* CTableDragger::ITableDragger */


/******************************************************************************
 Dispose
  	
  	Dispose of a CTableDragger. This method clears the table's reference to
  	this task.
******************************************************************************/

void CTableDragger::Dispose(void)
{
	if (itsTable->itsLastTask == this)
		itsTable->itsLastTask = NULL;
		
	inherited::Dispose();
}

/******************************************************************************
 CalcAnchorCell
 
 	Calculates the anchor cell used when selecting rectangles.
 	
******************************************************************************/

void CTableDragger::CalcAnchorCell( LongPt *startPt, Cell hitCell)
{
	RgnHandle	selection;
	Rect		r;

	selection = itsTable->GetSelection();
	if ((modifierKeys && shiftKey) && !EmptyRgn( selection))
	{
		r = (**selection).rgnBBox;
		if (((hitCell.h < r.left) || (hitCell.v < r.top)) ||
							EqualPt( hitCell, topLeft( r)))
		{
			anchorCell = botRight( r);
			anchorCell.h--; anchorCell.v--;
		}
		else
			anchorCell = topLeft( r);
	}
	else
		anchorCell = hitCell;

}	/* CTableDragger::CalcAnchorCell */

/******************************************************************************
 BeginTracking
 
 	Called at the start of mouse tracking. This method calculates the
 	anchor cell, and makes the initial selection based upon the
 	current selection settings (i.e. the selection flags, modifier keys,
 	and the previously initialized instance variables that control selection).
 	
******************************************************************************/

void CTableDragger::BeginTracking( LongPt *startPt)
{
	Cell		aCell;
	Rect		r;
	Boolean		fExtendInitially;
	Boolean		fInitialSelectRects;
	
	itsTable->FindHitCell( startPt, &aCell);
	
	CalcAnchorCell( startPt, aCell);
	
		/* if no disjoint selections are allowed, then the first	*/
		/* click always deselects everything						*/
		
	fExtendInitially = fExtend && !(selectionFlags & selNoDisjoint);
	
		/* If fSelectsRects is true soley because selDragsRects is	*/
		/* true but the shift key wasn't down, then we should not	*/
		/* select a rectangle initially								*/
		
		fInitialSelectRects = fSelectRects && (modifierKeys & shiftKey);
	
	if (fInitialSelectRects)
	{		
		Pt2Rect( anchorCell, aCell, &r);
		r.bottom++; r.right++;
		itsTable->SelectRect( &r, fExtendInitially, TRUE);
	}			
	else if (fUseSense)
	{
		fFirstWasSelected = itsTable->IsSelected( aCell);
		if (fFirstWasSelected)
			itsTable->DeselectCell( aCell, TRUE);
		else
			itsTable->SelectCell( aCell, fExtendInitially, TRUE);
	}
	else	
		itsTable->SelectCell( aCell, fExtendInitially, TRUE);
		
	prevCell = aCell;

}	/* CTableDragger::BeginTracking */

/******************************************************************************
 KeepTracking
 
 	Called continously while the mouse button is help down. This method
 	continously updates the selection and autoscrolls the table.
******************************************************************************/

void CTableDragger::KeepTracking(LongPt *currPt, LongPt *prevPt, LongPt *startPt)
{
	Cell newCell;
	Rect	r;
	Point	pt;

	itsTable->FindHitCell( currPt, &newCell);
	
	itsTable->AutoScroll( currPt);
			
	if (EqualPt( prevCell, newCell)) return;
	
	if (fSelectRects)
	{		
		Pt2Rect( anchorCell, newCell, &r);
		r.bottom++; r.right++;
		itsTable->SelectRect( &r, fExtend, TRUE);
	}
	else if (fUseSense)
	{
		if (fFirstWasSelected)
			itsTable->DeselectCell( newCell, TRUE);
		else
			itsTable->SelectCell( newCell, fExtend, TRUE);
	}
	else
	{
		itsTable->SelectCell( newCell, fExtend, TRUE);
	}
		
	prevCell = newCell;

}	/* CTableDragger::KeepTracking */
